from cgstats.utils import xstats

def test_parse(x_run_dir):
    assert xstats.parse(x_run_dir) == {
        'SVE2274A10_TCTCGCGC': {'barcodes': 475692348,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 20694926,
                                'perfect_barcodes': 454997422,
                                'pf_Q30': 83.67,
                                'pf_clusters': 496446277,
                                'pf_qscore': 35.96,
                                'pf_read1_q30': 88.79,
                                'pf_read2_q30': 78.56,
                                'pf_yield': 149926775654,
                                'pf_yield_pc': 79.92,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A10_TCTCGCGC',
                                'undetermined_pc': 4.180498467108054,
                                'undetermined_proc': 0},
        'SVE2274A11_TCTCGCGC': {'barcodes': 487052714,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 208158124,
                                'perfect_barcodes': 278894590,
                                'pf_Q30': 87.88,
                                'pf_clusters': 509529736,
                                'pf_qscore': 37.15,
                                'pf_read1_q30': 91.63,
                                'pf_read2_q30': 84.13,
                                'pf_yield': 153877980272,
                                'pf_yield_pc': 82.02,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A11_TCTCGCGC',
                                'undetermined_pc': 4.4113268396174625,
                                'undetermined_proc': 0},
         'SVE2274A2_TCCGCGAA': {'barcodes': 481129304,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '1',
                                'one_mismatch_barcodes': 20574865,
                                'perfect_barcodes': 460554439,
                                'pf_Q30': 87.58,
                                'pf_clusters': 500161560,
                                'pf_qscore': 37.05,
                                'pf_read1_q30': 91.8,
                                'pf_read2_q30': 83.36,
                                'pf_yield': 151048791120,
                                'pf_yield_pc': 80.51,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A2_TCCGCGAA',
                                'undetermined_pc': 3.805221656778262,
                                'undetermined_proc': 0},
         'SVE2274A4_TCCGCGAA': {'barcodes': 461509460,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '2',
                                'one_mismatch_barcodes': 50449617,
                                'perfect_barcodes': 411059843,
                                'pf_Q30': 86.71,
                                'pf_clusters': 504919125,
                                'pf_qscore': 36.82,
                                'pf_read1_q30': 91.57,
                                'pf_read2_q30': 81.84,
                                'pf_yield': 152485575750,
                                'pf_yield_pc': 81.28,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A4_TCCGCGAA',
                                'undetermined_pc': 8.597350120180138,
                                'undetermined_proc': 0},
         'SVE2274A6_TCCGCGAA': {'barcodes': 475599205,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '3',
                                'one_mismatch_barcodes': 25894701,
                                'perfect_barcodes': 449704504,
                                'pf_Q30': 87.38,
                                'pf_clusters': 496182972,
                                'pf_qscore': 36.98,
                                'pf_read1_q30': 91.2,
                                'pf_read2_q30': 83.57,
                                'pf_yield': 149847257544,
                                'pf_yield_pc': 79.87,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A6_TCCGCGAA',
                                'undetermined_pc': 4.148422691135802,
                                'undetermined_proc': 0},
         'SVE2274A7_TCCGCGAA': {'barcodes': 473147079,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '4',
                                'one_mismatch_barcodes': 209429007,
                                'perfect_barcodes': 263718072,
                                'pf_Q30': 87.12,
                                'pf_clusters': 500501443,
                                'pf_qscore': 36.91,
                                'pf_read1_q30': 90.53,
                                'pf_read2_q30': 83.72,
                                'pf_yield': 151151435786,
                                'pf_yield_pc': 80.57,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A7_TCCGCGAA',
                                'undetermined_pc': 5.465391635244496,
                                'undetermined_proc': 0},
         'SVE2274A8_TCCGCGAA': {'barcodes': 489960378,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '5',
                                'one_mismatch_barcodes': 22970952,
                                'perfect_barcodes': 466989426,
                                'pf_Q30': 88.84,
                                'pf_clusters': 510310624,
                                'pf_qscore': 37.4,
                                'pf_read1_q30': 92.27,
                                'pf_read2_q30': 85.42,
                                'pf_yield': 154113808448,
                                'pf_yield_pc': 82.15,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A8_TCCGCGAA',
                                'undetermined_pc': 3.987815468250961,
                                'undetermined_proc': 0},
         'SVE2274A9_TCTCGCGC': {'barcodes': 480369230,
                                'flowcell': 'HC7H2ALXX',
                                'lane': '6',
                                'one_mismatch_barcodes': 217616245,
                                'perfect_barcodes': 262752985,
                                'pf_Q30': 87.03,
                                'pf_clusters': 503887902,
                                'pf_qscore': 36.91,
                                'pf_read1_q30': 91.29,
                                'pf_read2_q30': 82.78,
                                'pf_yield': 152174146404,
                                'pf_yield_pc': 81.11,
                                'raw_clusters_pc': 100,
                                'sample_name': 'SVE2274A9_TCTCGCGC',
                                'undetermined_pc': 4.667441291337056,
                                'undetermined_proc': 0}}


def test_parse_samples(x_run_dir):
    assert xstats.parse_samples(x_run_dir) == {
            '1': {'SVE2274A2_TCCGCGAA': {'barcodes': 481129304,
                              'flowcell': 'HC7H2ALXX',
                              'lane': '1',
                              'one_mismatch_barcodes': 20574865,
                              'perfect_barcodes': 460554439,
                              'pf_Q30': 87.93,
                              'pf_clusters': 481129304,
                              'pf_qscore': 37.14,
                              'pf_read1_q30': 92.11,
                              'pf_read2_q30': 83.75,
                              'pf_yield': 145301049808,
                              'pf_yield_pc': 100.0,
                              'raw_clusters_pc': 100.0,
                              'sample_name': 'SVE2274A2_TCCGCGAA',
                              'undetermined_pc': 0.0,
                              'undetermined_proc': 0}},
 '2': {'SVE2274A4_TCCGCGAA': {'barcodes': 461509460,
                              'flowcell': 'HC7H2ALXX',
                              'lane': '2',
                              'one_mismatch_barcodes': 50449617,
                              'perfect_barcodes': 411059843,
                              'pf_Q30': 87.32,
                              'pf_clusters': 461509460,
                              'pf_qscore': 36.99,
                              'pf_read1_q30': 92.0,
                              'pf_read2_q30': 82.65,
                              'pf_yield': 139375856920,
                              'pf_yield_pc': 100.0,
                              'raw_clusters_pc': 100.0,
                              'sample_name': 'SVE2274A4_TCCGCGAA',
                              'undetermined_pc': 0.0,
                              'undetermined_proc': 0}},
 '3': {'SVE2274A6_TCCGCGAA': {'barcodes': 475599205,
                              'flowcell': 'HC7H2ALXX',
                              'lane': '3',
                              'one_mismatch_barcodes': 25894701,
                              'perfect_barcodes': 449704504,
                              'pf_Q30': 87.71,
                              'pf_clusters': 475599205,
                              'pf_qscore': 37.07,
                              'pf_read1_q30': 91.48,
                              'pf_read2_q30': 83.94,
                              'pf_yield': 143630959910,
                              'pf_yield_pc': 100.0,
                              'raw_clusters_pc': 100.0,
                              'sample_name': 'SVE2274A6_TCCGCGAA',
                              'undetermined_pc': 0.0,
                              'undetermined_proc': 0}},
 '4': {'SVE2274A7_TCCGCGAA': {'barcodes': 473147079,
                              'flowcell': 'HC7H2ALXX',
                              'lane': '4',
                              'one_mismatch_barcodes': 209429007,
                              'perfect_barcodes': 263718072,
                              'pf_Q30': 87.57,
                              'pf_clusters': 473147079,
                              'pf_qscore': 37.03,
                              'pf_read1_q30': 90.94,
                              'pf_read2_q30': 84.21,
                              'pf_yield': 142890417858,
                              'pf_yield_pc': 100.0,
                              'raw_clusters_pc': 100.0,
                              'sample_name': 'SVE2274A7_TCCGCGAA',
                              'undetermined_pc': 0.0,
                              'undetermined_proc': 0}},
 '5': {'SVE2274A8_TCCGCGAA': {'barcodes': 489960378,
                              'flowcell': 'HC7H2ALXX',
                              'lane': '5',
                              'one_mismatch_barcodes': 22970952,
                              'perfect_barcodes': 466989426,
                              'pf_Q30': 89.26,
                              'pf_clusters': 489960378,
                              'pf_qscore': 37.52,
                              'pf_read1_q30': 92.67,
                              'pf_read2_q30': 85.85,
                              'pf_yield': 147968034156,
                              'pf_yield_pc': 100.0,
                              'raw_clusters_pc': 100.0,
                              'sample_name': 'SVE2274A8_TCCGCGAA',
                              'undetermined_pc': 0.0,
                              'undetermined_proc': 0}},
 '6': {'SVE2274A9_TCTCGCGC': {'barcodes': 480369230,
                              'flowcell': 'HC7H2ALXX',
                              'lane': '6',
                              'one_mismatch_barcodes': 217616245,
                              'perfect_barcodes': 262752985,
                              'pf_Q30': 87.43,
                              'pf_clusters': 480369230,
                              'pf_qscore': 37.02,
                              'pf_read1_q30': 91.64,
                              'pf_read2_q30': 83.21,
                              'pf_yield': 145071507460,
                              'pf_yield_pc': 100.0,
                              'raw_clusters_pc': 100.0,
                              'sample_name': 'SVE2274A9_TCTCGCGC',
                              'undetermined_pc': 0.0,
                              'undetermined_proc': 0}},
 '7': {'SVE2274A10_TCTCGCGC': {'barcodes': 475692348,
                               'flowcell': 'HC7H2ALXX',
                               'lane': '7',
                               'one_mismatch_barcodes': 20694926,
                               'perfect_barcodes': 454997422,
                               'pf_Q30': 84.0,
                               'pf_clusters': 475692348,
                               'pf_qscore': 36.05,
                               'pf_read1_q30': 89.09,
                               'pf_read2_q30': 78.92,
                               'pf_yield': 143659089096,
                               'pf_yield_pc': 100.0,
                               'raw_clusters_pc': 100.0,
                               'sample_name': 'SVE2274A10_TCTCGCGC',
                               'undetermined_pc': 0.0,
                               'undetermined_proc': 0}},
 '8': {'SVE2274A11_TCTCGCGC': {'barcodes': 487052714,
                               'flowcell': 'HC7H2ALXX',
                               'lane': '8',
                               'one_mismatch_barcodes': 208158124,
                               'perfect_barcodes': 278894590,
                               'pf_Q30': 88.32,
                               'pf_clusters': 487052714,
                               'pf_qscore': 37.27,
                               'pf_read1_q30': 92.06,
                               'pf_read2_q30': 84.58,
                               'pf_yield': 147089919628,
                               'pf_yield_pc': 100.0,
                               'raw_clusters_pc': 100.0,
                               'sample_name': 'SVE2274A11_TCTCGCGC',
                               'undetermined_pc': 0.0,
                               'undetermined_proc': 0}}}


def test_parse_pooled(x_pooled_run_dir):
    assert xstats.parse(x_pooled_run_dir) == {
            'MIC2051A644_ACGATCAG': {'barcodes': 419722973,
                          'flowcell': 'H37GVALXX',
                          'lane': '8',
                          'one_mismatch_barcodes': 29469988,
                          'perfect_barcodes': 390252985,
                          'pf_Q30': 91.35,
                          'pf_clusters': 420558643,
                          'pf_qscore': 38.1,
                          'pf_read1_q30': 91.88,
                          'pf_read2_q30': 90.82,
                          'pf_yield': 127008710186,
                          'pf_yield_pc': 67.7,
                          'raw_clusters_pc': 100,
                          'sample_name': 'MIC2051A644_ACGATCAG',
                          'undetermined_pc': 0.19870474995802193,
                          'undetermined_proc': 0},
 'SVE2012A1_dual9': {'barcodes': 443653304,
                     'flowcell': 'H37GVALXX',
                     'lane': '5',
                     'one_mismatch_barcodes': 14313561,
                     'perfect_barcodes': 429339743,
                     'pf_Q30': 91.12,
                     'pf_clusters': 477695486,
                     'pf_qscore': 38.03,
                     'pf_read1_q30': 93.85,
                     'pf_read2_q30': 88.39,
                     'pf_yield': 144264036772,
                     'pf_yield_pc': 76.9,
                     'raw_clusters_pc': 100,
                     'sample_name': 'SVE2012A1_dual9',
                     'undetermined_pc': 7.12633529051183,
                     'undetermined_proc': 0},
 'SVE2013A1_dual17': {'barcodes': 443344256,
                      'flowcell': 'H37GVALXX',
                      'lane': '6',
                      'one_mismatch_barcodes': 36500580,
                      'perfect_barcodes': 406843676,
                      'pf_Q30': 90.3,
                      'pf_clusters': 458881716,
                      'pf_qscore': 37.83,
                      'pf_read1_q30': 93.33,
                      'pf_read2_q30': 87.27,
                      'pf_yield': 138582278232,
                      'pf_yield_pc': 73.87,
                      'raw_clusters_pc': 100,
                      'sample_name': 'SVE2013A1_dual17',
                      'undetermined_pc': 3.3859400926752112,
                      'undetermined_proc': 0},
 'SVE2105A1_dual11': {'barcodes': 442642020,
                      'flowcell': 'H37GVALXX',
                      'lane': '2',
                      'one_mismatch_barcodes': 81848242,
                      'perfect_barcodes': 360793778,
                      'pf_Q30': 90.73,
                      'pf_clusters': 477763969,
                      'pf_qscore': 37.92,
                      'pf_read1_q30': 93.7,
                      'pf_read2_q30': 87.77,
                      'pf_yield': 144284718638,
                      'pf_yield_pc': 76.91,
                      'raw_clusters_pc': 100,
                      'sample_name': 'SVE2105A1_dual11',
                      'undetermined_pc': 7.351318073129956,
                      'undetermined_proc': 0},
 'SVE2105A2_dual19': {'barcodes': 465609224,
                      'flowcell': 'H37GVALXX',
                      'lane': '4',
                      'one_mismatch_barcodes': 115764547,
                      'perfect_barcodes': 349844677,
                      'pf_Q30': 91.22,
                      'pf_clusters': 483997698,
                      'pf_qscore': 38.07,
                      'pf_read1_q30': 94.04,
                      'pf_read2_q30': 88.4,
                      'pf_yield': 146167304796,
                      'pf_yield_pc': 77.91,
                      'raw_clusters_pc': 100,
                      'sample_name': 'SVE2105A2_dual19',
                      'undetermined_pc': 3.7992895577780206,
                      'undetermined_proc': 0}}


def test_parse_samples_pooled(x_pooled_run_dir):
    assert xstats.parse_samples(x_pooled_run_dir) == {
            '1': {'SVE2105A1_dual11': {'barcodes': 437047823,
                            'flowcell': 'H37GVALXX',
                            'lane': '1',
                            'one_mismatch_barcodes': 15761064,
                            'perfect_barcodes': 421286759,
                            'pf_Q30': 90.43,
                            'pf_clusters': 437047823,
                            'pf_qscore': 37.83,
                            'pf_read1_q30': 93.55,
                            'pf_read2_q30': 87.3,
                            'pf_yield': 131988442546,
                            'pf_yield_pc': 100.0,
                            'raw_clusters_pc': 100.0,
                            'sample_name': 'SVE2105A1_dual11',
                            'undetermined_pc': 0.0,
                            'undetermined_proc': 0}},
 '2': {'SVE2105A1_dual11': {'barcodes': 442642020,
                            'flowcell': 'H37GVALXX',
                            'lane': '2',
                            'one_mismatch_barcodes': 81848242,
                            'perfect_barcodes': 360793778,
                            'pf_Q30': 91.0,
                            'pf_clusters': 442642020,
                            'pf_qscore': 38.0,
                            'pf_read1_q30': 93.91,
                            'pf_read2_q30': 88.08,
                            'pf_yield': 133677890040,
                            'pf_yield_pc': 100.0,
                            'raw_clusters_pc': 100.0,
                            'sample_name': 'SVE2105A1_dual11',
                            'undetermined_pc': 0.0,
                            'undetermined_proc': 0}},
 '3': {'SVE2105A2_dual19': {'barcodes': 474870927,
                            'flowcell': 'H37GVALXX',
                            'lane': '3',
                            'one_mismatch_barcodes': 19715794,
                            'perfect_barcodes': 455155133,
                            'pf_Q30': 91.61,
                            'pf_clusters': 474870927,
                            'pf_qscore': 38.17,
                            'pf_read1_q30': 94.41,
                            'pf_read2_q30': 88.82,
                            'pf_yield': 143411019954,
                            'pf_yield_pc': 100.0,
                            'raw_clusters_pc': 100.0,
                            'sample_name': 'SVE2105A2_dual19',
                            'undetermined_pc': 0.0,
                            'undetermined_proc': 0}},
 '4': {'SVE2105A2_dual19': {'barcodes': 465609224,
                            'flowcell': 'H37GVALXX',
                            'lane': '4',
                            'one_mismatch_barcodes': 115764547,
                            'perfect_barcodes': 349844677,
                            'pf_Q30': 91.54,
                            'pf_clusters': 465609224,
                            'pf_qscore': 38.15,
                            'pf_read1_q30': 94.3,
                            'pf_read2_q30': 88.78,
                            'pf_yield': 140613985648,
                            'pf_yield_pc': 100.0,
                            'raw_clusters_pc': 100.0,
                            'sample_name': 'SVE2105A2_dual19',
                            'undetermined_pc': 0.0,
                            'undetermined_proc': 0}},
 '5': {'SVE2012A1_dual9': {'barcodes': 443653304,
                           'flowcell': 'H37GVALXX',
                           'lane': '5',
                           'one_mismatch_barcodes': 14313561,
                           'perfect_barcodes': 429339743,
                           'pf_Q30': 91.37,
                           'pf_clusters': 443653304,
                           'pf_qscore': 38.1,
                           'pf_read1_q30': 94.07,
                           'pf_read2_q30': 88.68,
                           'pf_yield': 133983297808,
                           'pf_yield_pc': 100.0,
                           'raw_clusters_pc': 100.0,
                           'sample_name': 'SVE2012A1_dual9',
                           'undetermined_pc': 0.0,
                           'undetermined_proc': 0}},
 '6': {'SVE2013A1_dual17': {'barcodes': 443344256,
                            'flowcell': 'H37GVALXX',
                            'lane': '6',
                            'one_mismatch_barcodes': 36500580,
                            'perfect_barcodes': 406843676,
                            'pf_Q30': 90.64,
                            'pf_clusters': 443344256,
                            'pf_qscore': 37.92,
                            'pf_read1_q30': 93.62,
                            'pf_read2_q30': 87.66,
                            'pf_yield': 133889965312,
                            'pf_yield_pc': 100.0,
                            'raw_clusters_pc': 100.0,
                            'sample_name': 'SVE2013A1_dual17',
                            'undetermined_pc': 0.0,
                            'undetermined_proc': 0}},
 '7': {'MIC2051A321_GACACAGT': {'barcodes': 1170813,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 22226,
                                'perfect_barcodes': 1148587,
                                'pf_Q30': 91.12,
                                'pf_clusters': 1170813,
                                'pf_qscore': 37.99,
                                'pf_read1_q30': 90.02,
                                'pf_read2_q30': 92.21,
                                'pf_yield': 353585526,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A321_GACACAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A322_TTGCTTGG': {'barcodes': 12924,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 5953,
                                'perfect_barcodes': 6971,
                                'pf_Q30': 88.02,
                                'pf_clusters': 12924,
                                'pf_qscore': 37.22,
                                'pf_read1_q30': 88.74,
                                'pf_read2_q30': 87.3,
                                'pf_yield': 3903048,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A322_TTGCTTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A323_TATGGCAC': {'barcodes': 1411107,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27347,
                                'perfect_barcodes': 1383760,
                                'pf_Q30': 91.33,
                                'pf_clusters': 1411107,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 93.05,
                                'pf_read2_q30': 89.6,
                                'pf_yield': 426154314,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A323_TATGGCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A324_GGCGAATA': {'barcodes': 3108002,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 63750,
                                'perfect_barcodes': 3044252,
                                'pf_Q30': 91.75,
                                'pf_clusters': 3108002,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.75,
                                'pf_read2_q30': 89.76,
                                'pf_yield': 938616604,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.64,
                                'sample_name': 'MIC2051A324_GGCGAATA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A325_TCACCTAG': {'barcodes': 9516,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 2383,
                                'perfect_barcodes': 7133,
                                'pf_Q30': 91.83,
                                'pf_clusters': 9516,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 90.93,
                                'pf_read2_q30': 92.73,
                                'pf_yield': 2873832,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A325_TCACCTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A326_AACCGTGT': {'barcodes': 1292548,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26282,
                                'perfect_barcodes': 1266266,
                                'pf_Q30': 90.68,
                                'pf_clusters': 1292548,
                                'pf_qscore': 37.89,
                                'pf_read1_q30': 92.9,
                                'pf_read2_q30': 88.46,
                                'pf_yield': 390349496,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A326_AACCGTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A327_ATAGGTCC': {'barcodes': 1332832,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27224,
                                'perfect_barcodes': 1305608,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1332832,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 93.62,
                                'pf_read2_q30': 89.58,
                                'pf_yield': 402515264,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A327_ATAGGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A328_AGTATGCC': {'barcodes': 1116438,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1116438,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.44,
                                'pf_clusters': 1116438,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 93.76,
                                'pf_read2_q30': 89.12,
                                'pf_yield': 337164276,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A328_AGTATGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A329_GCATAACG': {'barcodes': 1244800,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26646,
                                'perfect_barcodes': 1218154,
                                'pf_Q30': 91.36,
                                'pf_clusters': 1244800,
                                'pf_qscore': 38.12,
                                'pf_read1_q30': 91.89,
                                'pf_read2_q30': 90.84,
                                'pf_yield': 375929600,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A329_GCATAACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A330_CCTGTCAA': {'barcodes': 1303210,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27367,
                                'perfect_barcodes': 1275843,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1303210,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.46,
                                'pf_read2_q30': 90.68,
                                'pf_yield': 393569420,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A330_CCTGTCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A331_ATAACGCC': {'barcodes': 1475250,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30891,
                                'perfect_barcodes': 1444359,
                                'pf_Q30': 91.64,
                                'pf_clusters': 1475250,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 92.27,
                                'pf_read2_q30': 91.0,
                                'pf_yield': 445525500,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A331_ATAACGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A332_AAGTGCAG': {'barcodes': 1334304,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27132,
                                'perfect_barcodes': 1307172,
                                'pf_Q30': 90.62,
                                'pf_clusters': 1334304,
                                'pf_qscore': 37.89,
                                'pf_read1_q30': 90.52,
                                'pf_read2_q30': 90.72,
                                'pf_yield': 402959808,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A332_AAGTGCAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A333_CAAGTCGT': {'barcodes': 1407746,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29988,
                                'perfect_barcodes': 1377758,
                                'pf_Q30': 92.05,
                                'pf_clusters': 1407746,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.52,
                                'pf_read2_q30': 90.58,
                                'pf_yield': 425139292,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A333_CAAGTCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A334_CGGTGGTA': {'barcodes': 1338316,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37281,
                                'perfect_barcodes': 1301035,
                                'pf_Q30': 91.47,
                                'pf_clusters': 1338316,
                                'pf_qscore': 38.13,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 89.5,
                                'pf_yield': 404171432,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A334_CGGTGGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A335_ACCGACAA': {'barcodes': 1283312,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25765,
                                'perfect_barcodes': 1257547,
                                'pf_Q30': 91.85,
                                'pf_clusters': 1283312,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.5,
                                'pf_read2_q30': 90.21,
                                'pf_yield': 387560224,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A335_ACCGACAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A336_TACTGCTC': {'barcodes': 1441039,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26960,
                                'perfect_barcodes': 1414079,
                                'pf_Q30': 91.31,
                                'pf_clusters': 1441039,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 91.78,
                                'pf_read2_q30': 90.84,
                                'pf_yield': 435193778,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A336_TACTGCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A337_ACAGAGGT': {'barcodes': 1355223,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32959,
                                'perfect_barcodes': 1322264,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1355223,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.45,
                                'pf_read2_q30': 90.83,
                                'pf_yield': 409277346,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A337_ACAGAGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A338_AGCCTATC': {'barcodes': 1504698,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31776,
                                'perfect_barcodes': 1472922,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1504698,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.61,
                                'pf_read2_q30': 90.67,
                                'pf_yield': 454418796,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A338_AGCCTATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A339_GTAGTACC': {'barcodes': 1527768,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31031,
                                'perfect_barcodes': 1496737,
                                'pf_Q30': 91.78,
                                'pf_clusters': 1527768,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 92.7,
                                'pf_read2_q30': 90.86,
                                'pf_yield': 461385936,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A339_GTAGTACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A340_GCAATTCC': {'barcodes': 2722580,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1088992,
                                'perfect_barcodes': 1633588,
                                'pf_Q30': 91.94,
                                'pf_clusters': 2722580,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.7,
                                'pf_read2_q30': 91.17,
                                'pf_yield': 822219160,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.56,
                                'sample_name': 'MIC2051A340_GCAATTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A341_CTGTATGC': {'barcodes': 2927331,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 55701,
                                'perfect_barcodes': 2871630,
                                'pf_Q30': 91.86,
                                'pf_clusters': 2927331,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.34,
                                'pf_read2_q30': 90.39,
                                'pf_yield': 884053962,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.61,
                                'sample_name': 'MIC2051A341_CTGTATGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A342_CTAGCAGT': {'barcodes': 1539337,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25432,
                                'perfect_barcodes': 1513905,
                                'pf_Q30': 91.7,
                                'pf_clusters': 1539337,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 93.53,
                                'pf_read2_q30': 89.87,
                                'pf_yield': 464879774,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A342_CTAGCAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A343_AACACTGG': {'barcodes': 1429067,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25737,
                                'perfect_barcodes': 1403330,
                                'pf_Q30': 91.71,
                                'pf_clusters': 1429067,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 93.18,
                                'pf_read2_q30': 90.24,
                                'pf_yield': 431578234,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A343_AACACTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A344_TGCTTGCT': {'barcodes': 1732679,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35734,
                                'perfect_barcodes': 1696945,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1732679,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.87,
                                'pf_read2_q30': 91.36,
                                'pf_yield': 523269058,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A344_TGCTTGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A345_CCACTAAG': {'barcodes': 1214580,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23541,
                                'perfect_barcodes': 1191039,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1214580,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.36,
                                'pf_read2_q30': 91.05,
                                'pf_yield': 366803160,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A345_CCACTAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A346_TGATCACG': {'barcodes': 1359223,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26442,
                                'perfect_barcodes': 1332781,
                                'pf_Q30': 91.26,
                                'pf_clusters': 1359223,
                                'pf_qscore': 38.02,
                                'pf_read1_q30': 90.66,
                                'pf_read2_q30': 91.86,
                                'pf_yield': 410485346,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A346_TGATCACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A347_CGCGTATT': {'barcodes': 1594477,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36946,
                                'perfect_barcodes': 1557531,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1594477,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.28,
                                'pf_read2_q30': 90.76,
                                'pf_yield': 481532054,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A347_CGCGTATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A348_CTTCGCAA': {'barcodes': 1531292,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32247,
                                'perfect_barcodes': 1499045,
                                'pf_Q30': 91.77,
                                'pf_clusters': 1531292,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.25,
                                'pf_read2_q30': 90.3,
                                'pf_yield': 462450184,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A348_CTTCGCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A349_AGTTCGCA': {'barcodes': 1404042,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28601,
                                'perfect_barcodes': 1375441,
                                'pf_Q30': 91.68,
                                'pf_clusters': 1404042,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.68,
                                'pf_read2_q30': 89.69,
                                'pf_yield': 424020684,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A349_AGTTCGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A350_ACCTCTTC': {'barcodes': 1486272,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34190,
                                'perfect_barcodes': 1452082,
                                'pf_Q30': 92.05,
                                'pf_clusters': 1486272,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.16,
                                'pf_read2_q30': 90.94,
                                'pf_yield': 448854144,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A350_ACCTCTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A351_ACCATAGG': {'barcodes': 1340457,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28686,
                                'perfect_barcodes': 1311771,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1340457,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 90.53,
                                'pf_yield': 404818014,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A351_ACCATAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A352_TCCACGTT': {'barcodes': 13621,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1477,
                                'perfect_barcodes': 12144,
                                'pf_Q30': 90.22,
                                'pf_clusters': 13621,
                                'pf_qscore': 37.76,
                                'pf_read1_q30': 89.49,
                                'pf_read2_q30': 90.95,
                                'pf_yield': 4113542,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A352_TCCACGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A353_TGTTCCGT': {'barcodes': 1526443,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35399,
                                'perfect_barcodes': 1491044,
                                'pf_Q30': 90.79,
                                'pf_clusters': 1526443,
                                'pf_qscore': 37.95,
                                'pf_read1_q30': 90.52,
                                'pf_read2_q30': 91.07,
                                'pf_yield': 460985786,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A353_TGTTCCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A354_CCACATTG': {'barcodes': 1358191,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29222,
                                'perfect_barcodes': 1328969,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1358191,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.33,
                                'pf_read2_q30': 90.92,
                                'pf_yield': 410173682,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A354_CCACATTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A355_ATCCACGA': {'barcodes': 1504628,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30312,
                                'perfect_barcodes': 1474316,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1504628,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.13,
                                'pf_read2_q30': 90.88,
                                'pf_yield': 454397656,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A355_ATCCACGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A356_CATTGACG': {'barcodes': 1424784,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29449,
                                'perfect_barcodes': 1395335,
                                'pf_Q30': 91.88,
                                'pf_clusters': 1424784,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.96,
                                'pf_read2_q30': 90.8,
                                'pf_yield': 430284768,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A356_CATTGACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A357_ATCGGAGA': {'barcodes': 1547980,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35609,
                                'perfect_barcodes': 1512371,
                                'pf_Q30': 91.68,
                                'pf_clusters': 1547980,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 93.26,
                                'pf_read2_q30': 90.1,
                                'pf_yield': 467489960,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A357_ATCGGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A358_TACTAGCG': {'barcodes': 1555114,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28918,
                                'perfect_barcodes': 1526196,
                                'pf_Q30': 91.88,
                                'pf_clusters': 1555114,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.18,
                                'pf_read2_q30': 90.58,
                                'pf_yield': 469644428,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A358_TACTAGCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A359_TCGATGAC': {'barcodes': 1601392,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30889,
                                'perfect_barcodes': 1570503,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1601392,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.8,
                                'pf_read2_q30': 91.35,
                                'pf_yield': 483620384,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A359_TCGATGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A360_AACCAGAG': {'barcodes': 1588554,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34197,
                                'perfect_barcodes': 1554357,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1588554,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.3,
                                'pf_read2_q30': 91.41,
                                'pf_yield': 479743308,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A360_AACCAGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A361_GATACCTG': {'barcodes': 1163747,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 18824,
                                'perfect_barcodes': 1144923,
                                'pf_Q30': 85.97,
                                'pf_clusters': 1163747,
                                'pf_qscore': 36.53,
                                'pf_read1_q30': 79.0,
                                'pf_read2_q30': 92.93,
                                'pf_yield': 351451594,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A361_GATACCTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A362_TCGAGAGT': {'barcodes': 1622997,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 52016,
                                'perfect_barcodes': 1570981,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1622997,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.96,
                                'pf_read2_q30': 90.84,
                                'pf_yield': 490145094,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A362_TCGAGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A363_TAACGTCG': {'barcodes': 1463723,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29606,
                                'perfect_barcodes': 1434117,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1463723,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.04,
                                'pf_read2_q30': 90.45,
                                'pf_yield': 442044346,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A363_TAACGTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A364_TCTGGACA': {'barcodes': 1561121,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34994,
                                'perfect_barcodes': 1526127,
                                'pf_Q30': 91.72,
                                'pf_clusters': 1561121,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 92.01,
                                'pf_read2_q30': 91.44,
                                'pf_yield': 471458542,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A364_TCTGGACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A365_AAGTCCTC': {'barcodes': 1689731,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 39896,
                                'perfect_barcodes': 1649835,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1689731,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.49,
                                'pf_read2_q30': 90.35,
                                'pf_yield': 510298762,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A365_AAGTCCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A366_ACAACAGC': {'barcodes': 1446284,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26053,
                                'perfect_barcodes': 1420231,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1446284,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.42,
                                'pf_read2_q30': 90.19,
                                'pf_yield': 436777768,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A366_ACAACAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A367_GACTTGTG': {'barcodes': 727774,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 12732,
                                'perfect_barcodes': 715042,
                                'pf_Q30': 89.35,
                                'pf_clusters': 727774,
                                'pf_qscore': 37.46,
                                'pf_read1_q30': 84.69,
                                'pf_read2_q30': 94.01,
                                'pf_yield': 219787748,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.15,
                                'sample_name': 'MIC2051A367_GACTTGTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A368_ACGAACGA': {'barcodes': 1316170,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28582,
                                'perfect_barcodes': 1287588,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1316170,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.25,
                                'pf_read2_q30': 90.88,
                                'pf_yield': 397483340,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A368_ACGAACGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A369_AGCCGTAA': {'barcodes': 1563988,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32290,
                                'perfect_barcodes': 1531698,
                                'pf_Q30': 92.05,
                                'pf_clusters': 1563988,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 90.77,
                                'pf_yield': 472324376,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A369_AGCCGTAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A370_GGTCGTAT': {'barcodes': 1701546,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40734,
                                'perfect_barcodes': 1660812,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1701546,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 92.6,
                                'pf_read2_q30': 90.59,
                                'pf_yield': 513866892,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A370_GGTCGTAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A371_CCTTCCAT': {'barcodes': 1567130,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35372,
                                'perfect_barcodes': 1531758,
                                'pf_Q30': 91.88,
                                'pf_clusters': 1567130,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 90.44,
                                'pf_yield': 473273260,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A371_CCTTCCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A372_GCTACAAC': {'barcodes': 1493326,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28967,
                                'perfect_barcodes': 1464359,
                                'pf_Q30': 91.73,
                                'pf_clusters': 1493326,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 92.41,
                                'pf_read2_q30': 91.06,
                                'pf_yield': 450984452,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A372_GCTACAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A373_TGGATGGT': {'barcodes': 1836389,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1836389,
                                'perfect_barcodes': 0,
                                'pf_Q30': 92.19,
                                'pf_clusters': 1836389,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 91.01,
                                'pf_yield': 554589478,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A373_TGGATGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A374_CGCAATGT': {'barcodes': 1569472,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33715,
                                'perfect_barcodes': 1535757,
                                'pf_Q30': 92.27,
                                'pf_clusters': 1569472,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.54,
                                'pf_read2_q30': 90.99,
                                'pf_yield': 473980544,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A374_CGCAATGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A375_CCGTTATG': {'barcodes': 1591781,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37358,
                                'perfect_barcodes': 1554423,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1591781,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.84,
                                'pf_read2_q30': 91.24,
                                'pf_yield': 480717862,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A375_CCGTTATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A376_ATAGTCGG': {'barcodes': 1664839,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33982,
                                'perfect_barcodes': 1630857,
                                'pf_Q30': 91.08,
                                'pf_clusters': 1664839,
                                'pf_qscore': 38.04,
                                'pf_read1_q30': 91.3,
                                'pf_read2_q30': 90.86,
                                'pf_yield': 502781378,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A376_ATAGTCGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A377_CTCCTGAA': {'barcodes': 1423402,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29645,
                                'perfect_barcodes': 1393757,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1423402,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.82,
                                'pf_read2_q30': 90.99,
                                'pf_yield': 429867404,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A377_CTCCTGAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A378_ACAGGCAT': {'barcodes': 1615405,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31586,
                                'perfect_barcodes': 1583819,
                                'pf_Q30': 91.69,
                                'pf_clusters': 1615405,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 89.87,
                                'pf_yield': 487852310,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A378_ACAGGCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A379_GATCAAGG': {'barcodes': 1007884,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 15418,
                                'perfect_barcodes': 992466,
                                'pf_Q30': 82.04,
                                'pf_clusters': 1007884,
                                'pf_qscore': 35.54,
                                'pf_read1_q30': 68.59,
                                'pf_read2_q30': 95.5,
                                'pf_yield': 304380968,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A379_GATCAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A380_CCGATGTA': {'barcodes': 1659901,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35855,
                                'perfect_barcodes': 1624046,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1659901,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.97,
                                'pf_read2_q30': 91.01,
                                'pf_yield': 501290102,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A380_CCGATGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A381_AGGTGTTG': {'barcodes': 12013,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 4522,
                                'perfect_barcodes': 7491,
                                'pf_Q30': 89.82,
                                'pf_clusters': 12013,
                                'pf_qscore': 37.71,
                                'pf_read1_q30': 91.14,
                                'pf_read2_q30': 88.51,
                                'pf_yield': 3627926,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A381_AGGTGTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A382_CAGTGCTT': {'barcodes': 1401251,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29604,
                                'perfect_barcodes': 1371647,
                                'pf_Q30': 91.06,
                                'pf_clusters': 1401251,
                                'pf_qscore': 38.01,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 88.61,
                                'pf_yield': 423177802,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A382_CAGTGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A383_CAACGAGT': {'barcodes': 14003,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 3242,
                                'perfect_barcodes': 10761,
                                'pf_Q30': 87.09,
                                'pf_clusters': 14003,
                                'pf_qscore': 36.98,
                                'pf_read1_q30': 88.52,
                                'pf_read2_q30': 85.67,
                                'pf_yield': 4228906,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A383_CAACGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A384_CCATGAAC': {'barcodes': 1530199,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31748,
                                'perfect_barcodes': 1498451,
                                'pf_Q30': 92.29,
                                'pf_clusters': 1530199,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.17,
                                'pf_read2_q30': 91.42,
                                'pf_yield': 462120098,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A384_CCATGAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A385_ACGAATCC': {'barcodes': 1432849,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29245,
                                'perfect_barcodes': 1403604,
                                'pf_Q30': 92.3,
                                'pf_clusters': 1432849,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.25,
                                'pf_read2_q30': 91.34,
                                'pf_yield': 432720398,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A385_ACGAATCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A386_GTGATCCA': {'barcodes': 1521860,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35860,
                                'perfect_barcodes': 1486000,
                                'pf_Q30': 91.95,
                                'pf_clusters': 1521860,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.4,
                                'pf_read2_q30': 91.5,
                                'pf_yield': 459601720,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A386_GTGATCCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A387_AAGCATCG': {'barcodes': 1533309,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29466,
                                'perfect_barcodes': 1503843,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1533309,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 90.43,
                                'pf_yield': 463059318,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A387_AAGCATCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A388_TAGGAGCT': {'barcodes': 1786234,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41097,
                                'perfect_barcodes': 1745137,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1786234,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 93.18,
                                'pf_read2_q30': 90.82,
                                'pf_yield': 539442668,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A388_TAGGAGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A389_GACGAACT': {'barcodes': 1274289,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 22270,
                                'perfect_barcodes': 1252019,
                                'pf_Q30': 90.84,
                                'pf_clusters': 1274289,
                                'pf_qscore': 37.91,
                                'pf_read1_q30': 91.93,
                                'pf_read2_q30': 89.75,
                                'pf_yield': 384835278,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A389_GACGAACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A390_TCTAGGAG': {'barcodes': 1825514,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 51325,
                                'perfect_barcodes': 1774189,
                                'pf_Q30': 91.73,
                                'pf_clusters': 1825514,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.04,
                                'pf_read2_q30': 90.41,
                                'pf_yield': 551305228,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A390_TCTAGGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A391_TTACCGAC': {'barcodes': 1454289,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26680,
                                'perfect_barcodes': 1427609,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1454289,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.77,
                                'pf_read2_q30': 91.34,
                                'pf_yield': 439195278,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A391_TTACCGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A392_GAGCAATC': {'barcodes': 1675264,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35974,
                                'perfect_barcodes': 1639290,
                                'pf_Q30': 91.51,
                                'pf_clusters': 1675264,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 91.83,
                                'pf_read2_q30': 91.19,
                                'pf_yield': 505929728,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A392_GAGCAATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A393_AATGGTCG': {'barcodes': 1473532,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33206,
                                'perfect_barcodes': 1440326,
                                'pf_Q30': 91.35,
                                'pf_clusters': 1473532,
                                'pf_qscore': 38.09,
                                'pf_read1_q30': 91.19,
                                'pf_read2_q30': 91.5,
                                'pf_yield': 445006664,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A393_AATGGTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A394_TTCGTACG': {'barcodes': 1389973,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26042,
                                'perfect_barcodes': 1363931,
                                'pf_Q30': 91.3,
                                'pf_clusters': 1389973,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 92.2,
                                'pf_read2_q30': 90.4,
                                'pf_yield': 419771846,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A394_TTCGTACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A395_AGGATAGC': {'barcodes': 1378973,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26869,
                                'perfect_barcodes': 1352104,
                                'pf_Q30': 91.57,
                                'pf_clusters': 1378973,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 93.9,
                                'pf_read2_q30': 89.24,
                                'pf_yield': 416449846,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A395_AGGATAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A396_AACAAGGC': {'barcodes': 1429050,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 24686,
                                'perfect_barcodes': 1404364,
                                'pf_Q30': 91.38,
                                'pf_clusters': 1429050,
                                'pf_qscore': 38.11,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 89.26,
                                'pf_yield': 431573100,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A396_AACAAGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A397_GTTCTTCG': {'barcodes': 1189145,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 20139,
                                'perfect_barcodes': 1169006,
                                'pf_Q30': 90.92,
                                'pf_clusters': 1189145,
                                'pf_qscore': 37.92,
                                'pf_read1_q30': 90.32,
                                'pf_read2_q30': 91.52,
                                'pf_yield': 359121790,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A397_GTTCTTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A398_GATTGTCC': {'barcodes': 1216395,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 21764,
                                'perfect_barcodes': 1194631,
                                'pf_Q30': 89.55,
                                'pf_clusters': 1216395,
                                'pf_qscore': 37.51,
                                'pf_read1_q30': 85.69,
                                'pf_read2_q30': 93.41,
                                'pf_yield': 367351290,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A398_GATTGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A400_CAACTTGG': {'barcodes': 1625639,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35964,
                                'perfect_barcodes': 1589675,
                                'pf_Q30': 90.97,
                                'pf_clusters': 1625639,
                                'pf_qscore': 38.01,
                                'pf_read1_q30': 91.39,
                                'pf_read2_q30': 90.56,
                                'pf_yield': 490942978,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A400_CAACTTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A401_CGCTACAT': {'barcodes': 2086282,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 42029,
                                'perfect_barcodes': 2044253,
                                'pf_Q30': 90.28,
                                'pf_clusters': 2086282,
                                'pf_qscore': 37.83,
                                'pf_read1_q30': 90.21,
                                'pf_read2_q30': 90.36,
                                'pf_yield': 630057164,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A401_CGCTACAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A402_ATGACAGG': {'barcodes': 7935,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1863,
                                'perfect_barcodes': 6072,
                                'pf_Q30': 89.35,
                                'pf_clusters': 7935,
                                'pf_qscore': 37.58,
                                'pf_read1_q30': 90.85,
                                'pf_read2_q30': 87.86,
                                'pf_yield': 2396370,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A402_ATGACAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A403_GGCTCAAT': {'barcodes': 1944958,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 44731,
                                'perfect_barcodes': 1900227,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1944958,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 90.76,
                                'pf_yield': 587377316,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A403_GGCTCAAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A404_CTCGGTAA': {'barcodes': 1443890,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36555,
                                'perfect_barcodes': 1407335,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1443890,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.6,
                                'pf_read2_q30': 90.43,
                                'pf_yield': 436054780,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A404_CTCGGTAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A405_TTCGCCAT': {'barcodes': 1804790,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32906,
                                'perfect_barcodes': 1771884,
                                'pf_Q30': 91.97,
                                'pf_clusters': 1804790,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.43,
                                'pf_read2_q30': 90.52,
                                'pf_yield': 545046580,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A405_TTCGCCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A406_GGTACGAA': {'barcodes': 1532968,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31361,
                                'perfect_barcodes': 1501607,
                                'pf_Q30': 91.75,
                                'pf_clusters': 1532968,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 90.06,
                                'pf_yield': 462956336,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A406_GGTACGAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A407_CTGTACCA': {'barcodes': 1505648,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33146,
                                'perfect_barcodes': 1472502,
                                'pf_Q30': 91.4,
                                'pf_clusters': 1505648,
                                'pf_qscore': 38.12,
                                'pf_read1_q30': 93.65,
                                'pf_read2_q30': 89.16,
                                'pf_yield': 454705696,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A407_CTGTACCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A408_CCACAACA': {'barcodes': 1768848,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40278,
                                'perfect_barcodes': 1728570,
                                'pf_Q30': 91.12,
                                'pf_clusters': 1768848,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 91.69,
                                'pf_read2_q30': 90.56,
                                'pf_yield': 534192096,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A408_CCACAACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A409_GAGCTCAA': {'barcodes': 1518851,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29480,
                                'perfect_barcodes': 1489371,
                                'pf_Q30': 90.79,
                                'pf_clusters': 1518851,
                                'pf_qscore': 37.94,
                                'pf_read1_q30': 91.77,
                                'pf_read2_q30': 89.81,
                                'pf_yield': 458693002,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A409_GAGCTCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A410_TCGAAGGT': {'barcodes': 1419887,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36117,
                                'perfect_barcodes': 1383770,
                                'pf_Q30': 91.23,
                                'pf_clusters': 1419887,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 92.92,
                                'pf_read2_q30': 89.53,
                                'pf_yield': 428805874,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A410_TCGAAGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A411_GAGACGAT': {'barcodes': 1474908,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29325,
                                'perfect_barcodes': 1445583,
                                'pf_Q30': 91.64,
                                'pf_clusters': 1474908,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 92.33,
                                'pf_read2_q30': 90.94,
                                'pf_yield': 445422216,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A411_GAGACGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A412_ACCTAAGG': {'barcodes': 1409716,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29792,
                                'perfect_barcodes': 1379924,
                                'pf_Q30': 91.29,
                                'pf_clusters': 1409716,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 88.99,
                                'pf_yield': 425734232,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A412_ACCTAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A413_ATGACCAG': {'barcodes': 1570685,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31230,
                                'perfect_barcodes': 1539455,
                                'pf_Q30': 91.39,
                                'pf_clusters': 1570685,
                                'pf_qscore': 38.11,
                                'pf_read1_q30': 93.35,
                                'pf_read2_q30': 89.44,
                                'pf_yield': 474346870,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A413_ATGACCAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A414_ACATAGGC': {'barcodes': 1492650,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31622,
                                'perfect_barcodes': 1461028,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1492650,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 90.39,
                                'pf_yield': 450780300,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A414_ACATAGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A415_CTTAGGAC': {'barcodes': 3118699,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 59235,
                                'perfect_barcodes': 3059464,
                                'pf_Q30': 91.83,
                                'pf_clusters': 3118699,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 90.34,
                                'pf_yield': 941847098,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.65,
                                'sample_name': 'MIC2051A415_CTTAGGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A416_TTGCTGGA': {'barcodes': 1631851,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31774,
                                'perfect_barcodes': 1600077,
                                'pf_Q30': 91.07,
                                'pf_clusters': 1631851,
                                'pf_qscore': 38.02,
                                'pf_read1_q30': 91.96,
                                'pf_read2_q30': 90.18,
                                'pf_yield': 492819002,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A416_TTGCTGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A417_GTGGTGTT': {'barcodes': 1749693,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 58960,
                                'perfect_barcodes': 1690733,
                                'pf_Q30': 91.95,
                                'pf_clusters': 1749693,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.78,
                                'pf_read2_q30': 91.11,
                                'pf_yield': 528407286,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A417_GTGGTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A418_TACTGCGT': {'barcodes': 1817708,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 39809,
                                'perfect_barcodes': 1777899,
                                'pf_Q30': 92.13,
                                'pf_clusters': 1817708,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 90.68,
                                'pf_yield': 548947816,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A418_TACTGCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A419_GCACTCAT': {'barcodes': 1575233,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32231,
                                'perfect_barcodes': 1543002,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1575233,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 91.94,
                                'pf_read2_q30': 92.2,
                                'pf_yield': 475720366,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A419_GCACTCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A420_CGTATTCG': {'barcodes': 1376691,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29963,
                                'perfect_barcodes': 1346728,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1376691,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.69,
                                'pf_read2_q30': 90.78,
                                'pf_yield': 415760682,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A420_CGTATTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A421_TACCACAG': {'barcodes': 1428503,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28303,
                                'perfect_barcodes': 1400200,
                                'pf_Q30': 92.05,
                                'pf_clusters': 1428503,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.46,
                                'pf_read2_q30': 91.63,
                                'pf_yield': 431407906,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A421_TACCACAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A422_TGATACGC': {'barcodes': 1519818,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29459,
                                'perfect_barcodes': 1490359,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1519818,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.54,
                                'pf_read2_q30': 91.32,
                                'pf_yield': 458985036,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A422_TGATACGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A423_GTAACGAC': {'barcodes': 1279000,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29227,
                                'perfect_barcodes': 1249773,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1279000,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.21,
                                'pf_read2_q30': 91.51,
                                'pf_yield': 386258000,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A423_GTAACGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A424_AGTCAGGA': {'barcodes': 1818415,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40975,
                                'perfect_barcodes': 1777440,
                                'pf_Q30': 90.86,
                                'pf_clusters': 1818415,
                                'pf_qscore': 37.99,
                                'pf_read1_q30': 91.37,
                                'pf_read2_q30': 90.35,
                                'pf_yield': 549161330,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A424_AGTCAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A425_TCGCTGTT': {'barcodes': 1798900,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41011,
                                'perfect_barcodes': 1757889,
                                'pf_Q30': 91.25,
                                'pf_clusters': 1798900,
                                'pf_qscore': 38.07,
                                'pf_read1_q30': 91.27,
                                'pf_read2_q30': 91.23,
                                'pf_yield': 543267800,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A425_TCGCTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A426_CGTTGAGT': {'barcodes': 1730058,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 53729,
                                'perfect_barcodes': 1676329,
                                'pf_Q30': 91.33,
                                'pf_clusters': 1730058,
                                'pf_qscore': 38.1,
                                'pf_read1_q30': 91.71,
                                'pf_read2_q30': 90.96,
                                'pf_yield': 522477516,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A426_CGTTGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A427_GTCCACAT': {'barcodes': 3363195,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 66423,
                                'perfect_barcodes': 3296772,
                                'pf_Q30': 91.97,
                                'pf_clusters': 3363195,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.62,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 1015684890,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.7,
                                'sample_name': 'MIC2051A427_GTCCACAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A428_TTCTCTCG': {'barcodes': 1349954,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29287,
                                'perfect_barcodes': 1320667,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1349954,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.47,
                                'pf_read2_q30': 91.51,
                                'pf_yield': 407686108,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A428_TTCTCTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A429_GAAGGAAG': {'barcodes': 1408366,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28035,
                                'perfect_barcodes': 1380331,
                                'pf_Q30': 91.2,
                                'pf_clusters': 1408366,
                                'pf_qscore': 38.0,
                                'pf_read1_q30': 89.36,
                                'pf_read2_q30': 93.05,
                                'pf_yield': 425326532,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A429_GAAGGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A430_AGAGTAGC': {'barcodes': 1613483,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40211,
                                'perfect_barcodes': 1573272,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1613483,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.12,
                                'pf_read2_q30': 91.16,
                                'pf_yield': 487271866,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A430_AGAGTAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A431_CATACCAC': {'barcodes': 1325335,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25297,
                                'perfect_barcodes': 1300038,
                                'pf_Q30': 91.65,
                                'pf_clusters': 1325335,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 93.21,
                                'pf_read2_q30': 90.09,
                                'pf_yield': 400251170,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A431_CATACCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A432_GATAGCGA': {'barcodes': 1560721,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28060,
                                'perfect_barcodes': 1532661,
                                'pf_Q30': 88.42,
                                'pf_clusters': 1560721,
                                'pf_qscore': 37.22,
                                'pf_read1_q30': 82.78,
                                'pf_read2_q30': 94.06,
                                'pf_yield': 471337742,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A432_GATAGCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A433_GTTCGGTT': {'barcodes': 1388731,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29636,
                                'perfect_barcodes': 1359095,
                                'pf_Q30': 91.42,
                                'pf_clusters': 1388731,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 90.83,
                                'pf_read2_q30': 92.02,
                                'pf_yield': 419396762,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A433_GTTCGGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A434_AGTGCAGT': {'barcodes': 3243536,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 68804,
                                'perfect_barcodes': 3174732,
                                'pf_Q30': 92.33,
                                'pf_clusters': 3243536,
                                'pf_qscore': 38.39,
                                'pf_read1_q30': 93.53,
                                'pf_read2_q30': 91.12,
                                'pf_yield': 979547872,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.67,
                                'sample_name': 'MIC2051A434_AGTGCAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A435_GTCAGTTG': {'barcodes': 1085477,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25177,
                                'perfect_barcodes': 1060300,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1085477,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.48,
                                'pf_read2_q30': 91.64,
                                'pf_yield': 327814054,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A435_GTCAGTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A436_ATAAGGCG': {'barcodes': 1497627,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28563,
                                'perfect_barcodes': 1469064,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1497627,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.35,
                                'pf_read2_q30': 90.64,
                                'pf_yield': 452283354,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A436_ATAAGGCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A437_GAAGGTTC': {'barcodes': 1524648,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36206,
                                'perfect_barcodes': 1488442,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1524648,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.23,
                                'pf_read2_q30': 91.84,
                                'pf_yield': 460443696,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A437_GAAGGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A438_TGTGAAGC': {'barcodes': 1592903,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33120,
                                'perfect_barcodes': 1559783,
                                'pf_Q30': 92.16,
                                'pf_clusters': 1592903,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.18,
                                'pf_read2_q30': 91.13,
                                'pf_yield': 481056706,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A438_TGTGAAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A439_CGCTTAAC': {'barcodes': 1623776,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31360,
                                'perfect_barcodes': 1592416,
                                'pf_Q30': 91.95,
                                'pf_clusters': 1623776,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.37,
                                'pf_read2_q30': 91.52,
                                'pf_yield': 490380352,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A439_CGCTTAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A440_GGACTAGA': {'barcodes': 1775369,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38902,
                                'perfect_barcodes': 1736467,
                                'pf_Q30': 91.57,
                                'pf_clusters': 1775369,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 91.94,
                                'pf_read2_q30': 91.19,
                                'pf_yield': 536161438,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A440_GGACTAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A441_GGCAAGTT': {'barcodes': 1687856,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40166,
                                'perfect_barcodes': 1647690,
                                'pf_Q30': 91.59,
                                'pf_clusters': 1687856,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 94.13,
                                'pf_read2_q30': 89.04,
                                'pf_yield': 509732512,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A441_GGCAAGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A442_AGCCAAGT': {'barcodes': 2097602,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 43117,
                                'perfect_barcodes': 2054485,
                                'pf_Q30': 92.06,
                                'pf_clusters': 2097602,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.68,
                                'pf_read2_q30': 90.44,
                                'pf_yield': 633475804,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A442_AGCCAAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A443_GGCTATTG': {'barcodes': 1645129,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37203,
                                'perfect_barcodes': 1607926,
                                'pf_Q30': 91.68,
                                'pf_clusters': 1645129,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.72,
                                'pf_read2_q30': 89.65,
                                'pf_yield': 496828958,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A443_GGCTATTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A444_TTGATCCG': {'barcodes': 1327139,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 22769,
                                'perfect_barcodes': 1304370,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1327139,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 91.5,
                                'pf_read2_q30': 92.1,
                                'pf_yield': 400795978,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A444_TTGATCCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A445_CAGTCTTC': {'barcodes': 1475879,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30883,
                                'perfect_barcodes': 1444996,
                                'pf_Q30': 92.18,
                                'pf_clusters': 1475879,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 445715458,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A445_CAGTCTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A446_ATCTGTCC': {'barcodes': 1651942,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33797,
                                'perfect_barcodes': 1618145,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1651942,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.74,
                                'pf_read2_q30': 91.28,
                                'pf_yield': 498886484,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A446_ATCTGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A447_ACAGCAAC': {'barcodes': 3506374,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1752512,
                                'perfect_barcodes': 1753862,
                                'pf_Q30': 92.01,
                                'pf_clusters': 3506374,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.86,
                                'pf_read2_q30': 91.17,
                                'pf_yield': 1058924948,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.73,
                                'sample_name': 'MIC2051A447_ACAGCAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A448_GGTTGTCA': {'barcodes': 1671322,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37927,
                                'perfect_barcodes': 1633395,
                                'pf_Q30': 91.53,
                                'pf_clusters': 1671322,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 91.29,
                                'pf_read2_q30': 91.76,
                                'pf_yield': 504739244,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A448_GGTTGTCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A449_TCAGGCTT': {'barcodes': 13309,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1599,
                                'perfect_barcodes': 11710,
                                'pf_Q30': 90.64,
                                'pf_clusters': 13309,
                                'pf_qscore': 37.89,
                                'pf_read1_q30': 90.54,
                                'pf_read2_q30': 90.74,
                                'pf_yield': 4019318,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A449_TCAGGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A450_TGGACTCT': {'barcodes': 1756802,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 39285,
                                'perfect_barcodes': 1717517,
                                'pf_Q30': 92.22,
                                'pf_clusters': 1756802,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 92.97,
                                'pf_read2_q30': 91.47,
                                'pf_yield': 530554204,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A450_TGGACTCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A451_TCCGTATG': {'barcodes': 1348607,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27598,
                                'perfect_barcodes': 1321009,
                                'pf_Q30': 91.84,
                                'pf_clusters': 1348607,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 91.91,
                                'pf_read2_q30': 91.78,
                                'pf_yield': 407279314,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A451_TCCGTATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A452_GAGATACG': {'barcodes': 1461053,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29599,
                                'perfect_barcodes': 1431454,
                                'pf_Q30': 91.83,
                                'pf_clusters': 1461053,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 92.14,
                                'pf_read2_q30': 91.52,
                                'pf_yield': 441238006,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A452_GAGATACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A453_GAGATGTC': {'barcodes': 1414502,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33018,
                                'perfect_barcodes': 1381484,
                                'pf_Q30': 91.75,
                                'pf_clusters': 1414502,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 92.24,
                                'pf_read2_q30': 91.25,
                                'pf_yield': 427179604,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A453_GAGATGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A454_AAGACTCC': {'barcodes': 1817441,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35719,
                                'perfect_barcodes': 1781722,
                                'pf_Q30': 92.31,
                                'pf_clusters': 1817441,
                                'pf_qscore': 38.39,
                                'pf_read1_q30': 93.3,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 548867182,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A454_AAGACTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A455_GCCACTTA': {'barcodes': 1571436,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37644,
                                'perfect_barcodes': 1533792,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1571436,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.13,
                                'pf_read2_q30': 91.21,
                                'pf_yield': 474573672,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A455_GCCACTTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A456_GCCTATCA': {'barcodes': 1549382,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31651,
                                'perfect_barcodes': 1517731,
                                'pf_Q30': 91.68,
                                'pf_clusters': 1549382,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 90.84,
                                'pf_read2_q30': 92.51,
                                'pf_yield': 467913364,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A456_GCCTATCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A457_GCTGGATT': {'barcodes': 1554436,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35364,
                                'perfect_barcodes': 1519072,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1554436,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 91.42,
                                'pf_read2_q30': 92.3,
                                'pf_yield': 469439672,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A457_GCTGGATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A458_GTGTTCCT': {'barcodes': 1309156,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27385,
                                'perfect_barcodes': 1281771,
                                'pf_Q30': 91.3,
                                'pf_clusters': 1309156,
                                'pf_qscore': 38.07,
                                'pf_read1_q30': 93.13,
                                'pf_read2_q30': 89.47,
                                'pf_yield': 395365112,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A458_GTGTTCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A459_CTTGGATG': {'barcodes': 1483785,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33913,
                                'perfect_barcodes': 1449872,
                                'pf_Q30': 91.54,
                                'pf_clusters': 1483785,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 93.55,
                                'pf_read2_q30': 89.53,
                                'pf_yield': 448103070,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A459_CTTGGATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A460_CAGGTTAG': {'barcodes': 1613358,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41465,
                                'perfect_barcodes': 1571893,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1613358,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 90.4,
                                'pf_yield': 487234116,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A460_CAGGTTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A461_TCGTAGTC': {'barcodes': 1539648,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32796,
                                'perfect_barcodes': 1506852,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1539648,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 93.4,
                                'pf_read2_q30': 90.6,
                                'pf_yield': 464973696,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A461_TCGTAGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A462_AACTTGCC': {'barcodes': 1466709,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27099,
                                'perfect_barcodes': 1439610,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1466709,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 90.49,
                                'pf_yield': 442946118,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A462_AACTTGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A463_CTGGAGTA': {'barcodes': 1748455,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 54482,
                                'perfect_barcodes': 1693973,
                                'pf_Q30': 92.08,
                                'pf_clusters': 1748455,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.13,
                                'pf_read2_q30': 91.02,
                                'pf_yield': 528033410,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A463_CTGGAGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A464_CTATCGCA': {'barcodes': 2150278,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 42572,
                                'perfect_barcodes': 2107706,
                                'pf_Q30': 91.87,
                                'pf_clusters': 2150278,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.9,
                                'pf_read2_q30': 90.84,
                                'pf_yield': 649383956,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.44,
                                'sample_name': 'MIC2051A464_CTATCGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A465_CACGTTGT': {'barcodes': 1889391,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 48578,
                                'perfect_barcodes': 1840813,
                                'pf_Q30': 89.59,
                                'pf_clusters': 1889391,
                                'pf_qscore': 37.62,
                                'pf_read1_q30': 89.53,
                                'pf_read2_q30': 89.64,
                                'pf_yield': 570596082,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A465_CACGTTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A466_TTCAGCCT': {'barcodes': 1571688,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30705,
                                'perfect_barcodes': 1540983,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1571688,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.4,
                                'pf_read2_q30': 91.74,
                                'pf_yield': 474649776,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A466_TTCAGCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A467_AGTGTTGG': {'barcodes': 1521456,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 43601,
                                'perfect_barcodes': 1477855,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1521456,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.18,
                                'pf_read2_q30': 91.07,
                                'pf_yield': 459479712,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A467_AGTGTTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A468_TGAGCTAG': {'barcodes': 1627026,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34303,
                                'perfect_barcodes': 1592723,
                                'pf_Q30': 91.13,
                                'pf_clusters': 1627026,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 93.16,
                                'pf_read2_q30': 89.11,
                                'pf_yield': 491361852,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A468_TGAGCTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A469_TAGAGCTC': {'barcodes': 1452596,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30704,
                                'perfect_barcodes': 1421892,
                                'pf_Q30': 91.5,
                                'pf_clusters': 1452596,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 93.35,
                                'pf_read2_q30': 89.66,
                                'pf_yield': 438683992,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A469_TAGAGCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A470_ACGTTACC': {'barcodes': 1472320,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29305,
                                'perfect_barcodes': 1443015,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1472320,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.07,
                                'pf_read2_q30': 90.97,
                                'pf_yield': 444640640,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A470_ACGTTACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A471_CATGGCTA': {'barcodes': 1597297,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30223,
                                'perfect_barcodes': 1567074,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1597297,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.12,
                                'pf_read2_q30': 90.97,
                                'pf_yield': 482383694,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A471_CATGGCTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A472_TGCTTCCA': {'barcodes': 2552185,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 54657,
                                'perfect_barcodes': 2497528,
                                'pf_Q30': 91.94,
                                'pf_clusters': 2552185,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.57,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 770759870,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.53,
                                'sample_name': 'MIC2051A472_TGCTTCCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A473_ATCGGTGT': {'barcodes': 1883716,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 48511,
                                'perfect_barcodes': 1835205,
                                'pf_Q30': 91.25,
                                'pf_clusters': 1883716,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 91.73,
                                'pf_read2_q30': 90.77,
                                'pf_yield': 568882232,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A473_ATCGGTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A474_CGTGTACT': {'barcodes': 1453349,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30609,
                                'perfect_barcodes': 1422740,
                                'pf_Q30': 91.32,
                                'pf_clusters': 1453349,
                                'pf_qscore': 38.09,
                                'pf_read1_q30': 93.65,
                                'pf_read2_q30': 88.98,
                                'pf_yield': 438911398,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A474_CGTGTACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A475_GATACTGG': {'barcodes': 1219405,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 24132,
                                'perfect_barcodes': 1195273,
                                'pf_Q30': 88.16,
                                'pf_clusters': 1219405,
                                'pf_qscore': 37.13,
                                'pf_read1_q30': 81.52,
                                'pf_read2_q30': 94.8,
                                'pf_yield': 368260310,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A475_GATACTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A476_AGCTTGAG': {'barcodes': 1727353,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 90830,
                                'perfect_barcodes': 1636523,
                                'pf_Q30': 92.28,
                                'pf_clusters': 1727353,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.3,
                                'pf_read2_q30': 91.27,
                                'pf_yield': 521660606,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A476_AGCTTGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A477_TGGCTATC': {'barcodes': 1095027,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 21720,
                                'perfect_barcodes': 1073307,
                                'pf_Q30': 92.15,
                                'pf_clusters': 1095027,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.26,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 330698154,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A477_TGGCTATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A478_TCATCACC': {'barcodes': 1518608,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28095,
                                'perfect_barcodes': 1490513,
                                'pf_Q30': 91.79,
                                'pf_clusters': 1518608,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 91.18,
                                'pf_read2_q30': 92.4,
                                'pf_yield': 458619616,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A478_TCATCACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A479_GCTGACTA': {'barcodes': 1346520,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 24688,
                                'perfect_barcodes': 1321832,
                                'pf_Q30': 91.29,
                                'pf_clusters': 1346520,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 92.32,
                                'pf_read2_q30': 90.26,
                                'pf_yield': 406649040,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A479_GCTGACTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A480_GTGTGACA': {'barcodes': 1600751,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36897,
                                'perfect_barcodes': 1563854,
                                'pf_Q30': 91.81,
                                'pf_clusters': 1600751,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 92.25,
                                'pf_read2_q30': 91.38,
                                'pf_yield': 483426802,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A480_GTGTGACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A481_GTCACTGT': {'barcodes': 1706215,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38971,
                                'perfect_barcodes': 1667244,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1706215,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.41,
                                'pf_read2_q30': 91.43,
                                'pf_yield': 515276930,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A481_GTCACTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A482_AGGTCACT': {'barcodes': 1504573,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33174,
                                'perfect_barcodes': 1471399,
                                'pf_Q30': 91.96,
                                'pf_clusters': 1504573,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.79,
                                'pf_read2_q30': 90.12,
                                'pf_yield': 454381046,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A482_AGGTCACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A483_TTAAGCGG': {'barcodes': 1602319,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31058,
                                'perfect_barcodes': 1571261,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1602319,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.56,
                                'pf_read2_q30': 91.46,
                                'pf_yield': 483900338,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A483_TTAAGCGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A484_TGTTCGAG': {'barcodes': 1600172,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37886,
                                'perfect_barcodes': 1562286,
                                'pf_Q30': 92.44,
                                'pf_clusters': 1600172,
                                'pf_qscore': 38.4,
                                'pf_read1_q30': 92.85,
                                'pf_read2_q30': 92.04,
                                'pf_yield': 483251944,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A484_TGTTCGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A485_GCAAGATC': {'barcodes': 1626794,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34366,
                                'perfect_barcodes': 1592428,
                                'pf_Q30': 92.19,
                                'pf_clusters': 1626794,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 92.51,
                                'pf_read2_q30': 91.88,
                                'pf_yield': 491291788,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A485_GCAAGATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A486_TACCAACC': {'barcodes': 1627600,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31193,
                                'perfect_barcodes': 1596407,
                                'pf_Q30': 92.18,
                                'pf_clusters': 1627600,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 92.61,
                                'pf_read2_q30': 91.76,
                                'pf_yield': 491535200,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A486_TACCAACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A487_CCAGGATA': {'barcodes': 1784877,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38260,
                                'perfect_barcodes': 1746617,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1784877,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.11,
                                'pf_read2_q30': 91.24,
                                'pf_yield': 539032854,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A487_CCAGGATA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A488_GCACGTAA': {'barcodes': 1702735,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34342,
                                'perfect_barcodes': 1668393,
                                'pf_Q30': 91.49,
                                'pf_clusters': 1702735,
                                'pf_qscore': 38.12,
                                'pf_read1_q30': 90.97,
                                'pf_read2_q30': 92.01,
                                'pf_yield': 514225970,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A488_GCACGTAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A489_AGGATGGT': {'barcodes': 1554000,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1554000,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.7,
                                'pf_clusters': 1554000,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.9,
                                'pf_read2_q30': 90.49,
                                'pf_yield': 469308000,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A489_AGGATGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A490_GCACAACT': {'barcodes': 1658140,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32018,
                                'perfect_barcodes': 1626122,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1658140,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.97,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 500758280,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A490_GCACAACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A491_CTGTTAGG': {'barcodes': 1719039,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41049,
                                'perfect_barcodes': 1677990,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1719039,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.96,
                                'pf_read2_q30': 91.03,
                                'pf_yield': 519149778,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A491_CTGTTAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A492_CGATAGAG': {'barcodes': 1565598,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38003,
                                'perfect_barcodes': 1527595,
                                'pf_Q30': 91.89,
                                'pf_clusters': 1565598,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.72,
                                'pf_read2_q30': 91.05,
                                'pf_yield': 472810596,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A492_CGATAGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A493_ATGGTTGC': {'barcodes': 1618133,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31694,
                                'perfect_barcodes': 1586439,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1618133,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.41,
                                'pf_read2_q30': 90.82,
                                'pf_yield': 488676166,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A493_ATGGTTGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A494_AGGTGTAC': {'barcodes': 1562576,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36804,
                                'perfect_barcodes': 1525772,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1562576,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.52,
                                'pf_read2_q30': 90.81,
                                'pf_yield': 471897952,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A494_AGGTGTAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A495_CGGATTGA': {'barcodes': 1475427,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31674,
                                'perfect_barcodes': 1443753,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1475427,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.12,
                                'pf_read2_q30': 90.6,
                                'pf_yield': 445578954,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A495_CGGATTGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A496_AGGAGGAA': {'barcodes': 2078671,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 56423,
                                'perfect_barcodes': 2022248,
                                'pf_Q30': 89.98,
                                'pf_clusters': 2078671,
                                'pf_qscore': 37.73,
                                'pf_read1_q30': 90.22,
                                'pf_read2_q30': 89.75,
                                'pf_yield': 627758642,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A496_AGGAGGAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A497_AACAACCG': {'barcodes': 17351,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 2749,
                                'perfect_barcodes': 14602,
                                'pf_Q30': 90.42,
                                'pf_clusters': 17351,
                                'pf_qscore': 37.85,
                                'pf_read1_q30': 91.46,
                                'pf_read2_q30': 89.38,
                                'pf_yield': 5240002,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A497_AACAACCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A498_ACGGACTT': {'barcodes': 1692303,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34537,
                                'perfect_barcodes': 1657766,
                                'pf_Q30': 91.16,
                                'pf_clusters': 1692303,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 93.64,
                                'pf_read2_q30': 88.68,
                                'pf_yield': 511075506,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A498_ACGGACTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A499_ATCTCCTG': {'barcodes': 1320112,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28414,
                                'perfect_barcodes': 1291698,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1320112,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.75,
                                'pf_read2_q30': 91.66,
                                'pf_yield': 398673824,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A499_ATCTCCTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A500_CGAACAAC': {'barcodes': 1358945,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27954,
                                'perfect_barcodes': 1330991,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1358945,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 92.98,
                                'pf_read2_q30': 90.22,
                                'pf_yield': 410401390,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A500_CGAACAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A501_CTGTATGC': {'barcodes': 2927331,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 55701,
                                'perfect_barcodes': 2871630,
                                'pf_Q30': 91.86,
                                'pf_clusters': 2927331,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.34,
                                'pf_read2_q30': 90.39,
                                'pf_yield': 884053962,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.61,
                                'sample_name': 'MIC2051A501_CTGTATGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A502_GCCTTCTT': {'barcodes': 1418561,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32548,
                                'perfect_barcodes': 1386013,
                                'pf_Q30': 92.1,
                                'pf_clusters': 1418561,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.36,
                                'pf_read2_q30': 91.85,
                                'pf_yield': 428405422,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A502_GCCTTCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A503_GTCCTTGA': {'barcodes': 1424947,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31580,
                                'perfect_barcodes': 1393367,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1424947,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 91.97,
                                'pf_read2_q30': 92.05,
                                'pf_yield': 430333994,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A503_GTCCTTGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A504_TCCGATCA': {'barcodes': 1484899,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30029,
                                'perfect_barcodes': 1454870,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1484899,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 91.66,
                                'pf_read2_q30': 92.58,
                                'pf_yield': 448439498,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A504_TCCGATCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A505_AACAGTCC': {'barcodes': 1990396,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37328,
                                'perfect_barcodes': 1953068,
                                'pf_Q30': 92.29,
                                'pf_clusters': 1990396,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.02,
                                'pf_read2_q30': 91.56,
                                'pf_yield': 601099592,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A505_AACAGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A506_ACTCGATC': {'barcodes': 1914738,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 39280,
                                'perfect_barcodes': 1875458,
                                'pf_Q30': 92.08,
                                'pf_clusters': 1914738,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.25,
                                'pf_read2_q30': 91.9,
                                'pf_yield': 578250876,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A506_ACTCGATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A507_ATGCGCTT': {'barcodes': 2344585,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 50384,
                                'perfect_barcodes': 2294201,
                                'pf_Q30': 92.14,
                                'pf_clusters': 2344585,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.78,
                                'pf_read2_q30': 91.49,
                                'pf_yield': 708064670,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.49,
                                'sample_name': 'MIC2051A507_ATGCGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A508_CGACACTT': {'barcodes': 1981535,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40462,
                                'perfect_barcodes': 1941073,
                                'pf_Q30': 92.05,
                                'pf_clusters': 1981535,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.86,
                                'pf_read2_q30': 91.24,
                                'pf_yield': 598423570,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A508_CGACACTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A509_CTTAGGAC': {'barcodes': 3118699,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 59235,
                                'perfect_barcodes': 3059464,
                                'pf_Q30': 91.83,
                                'pf_clusters': 3118699,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 90.34,
                                'pf_yield': 941847098,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.65,
                                'sample_name': 'MIC2051A509_CTTAGGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A510_GCTAAGGA': {'barcodes': 1646015,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35534,
                                'perfect_barcodes': 1610481,
                                'pf_Q30': 91.61,
                                'pf_clusters': 1646015,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 90.39,
                                'pf_read2_q30': 92.83,
                                'pf_yield': 497096530,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A510_GCTAAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A511_GTCTGAGT': {'barcodes': 1817615,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 44737,
                                'perfect_barcodes': 1772878,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1817615,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 90.75,
                                'pf_read2_q30': 92.77,
                                'pf_yield': 548919730,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A511_GTCTGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A512_TCGAACCT': {'barcodes': 2022054,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41606,
                                'perfect_barcodes': 1980448,
                                'pf_Q30': 92.24,
                                'pf_clusters': 2022054,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.49,
                                'pf_read2_q30': 91.99,
                                'pf_yield': 610660308,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.42,
                                'sample_name': 'MIC2051A512_TCGAACCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A513_AACGCACA': {'barcodes': 1638820,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28739,
                                'perfect_barcodes': 1610081,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1638820,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 494923640,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A513_AACGCACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A514_ACTGCTTG': {'barcodes': 1678076,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35287,
                                'perfect_barcodes': 1642789,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1678076,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.96,
                                'pf_read2_q30': 91.39,
                                'pf_yield': 506778952,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A514_ACTGCTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A515_ATTCCGCT': {'barcodes': 1842792,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36447,
                                'perfect_barcodes': 1806345,
                                'pf_Q30': 92.3,
                                'pf_clusters': 1842792,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.06,
                                'pf_read2_q30': 91.54,
                                'pf_yield': 556523184,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A515_ATTCCGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A516_CGATCGAT': {'barcodes': 1545779,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26547,
                                'perfect_barcodes': 1519232,
                                'pf_Q30': 91.18,
                                'pf_clusters': 1545779,
                                'pf_qscore': 37.99,
                                'pf_read1_q30': 89.2,
                                'pf_read2_q30': 93.16,
                                'pf_yield': 466825258,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A516_CGATCGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A517_CTTGCTAG': {'barcodes': 1652503,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33762,
                                'perfect_barcodes': 1618741,
                                'pf_Q30': 92.2,
                                'pf_clusters': 1652503,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 90.89,
                                'pf_yield': 499055906,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A517_CTTGCTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A518_GCTGAATC': {'barcodes': 1641736,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34688,
                                'perfect_barcodes': 1607048,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1641736,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.29,
                                'pf_read2_q30': 91.96,
                                'pf_yield': 495804272,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A518_GCTGAATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A519_GTGTGTTC': {'barcodes': 1651546,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 43776,
                                'perfect_barcodes': 1607770,
                                'pf_Q30': 92.26,
                                'pf_clusters': 1651546,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.49,
                                'pf_read2_q30': 92.04,
                                'pf_yield': 498766892,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A519_GTGTGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A520_TCGTCTGA': {'barcodes': 1629889,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34070,
                                'perfect_barcodes': 1595819,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1629889,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 92.84,
                                'pf_read2_q30': 90.37,
                                'pf_yield': 492226478,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A520_TCGTCTGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A521_AAGACCGT': {'barcodes': 1868685,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38119,
                                'perfect_barcodes': 1830566,
                                'pf_Q30': 92.08,
                                'pf_clusters': 1868685,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.62,
                                'pf_read2_q30': 91.55,
                                'pf_yield': 564342870,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A521_AAGACCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A522_AGAAGCCT': {'barcodes': 35825,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35825,
                                'perfect_barcodes': 0,
                                'pf_Q30': 81.06,
                                'pf_clusters': 35825,
                                'pf_qscore': 35.26,
                                'pf_read1_q30': 83.22,
                                'pf_read2_q30': 78.9,
                                'pf_yield': 10819150,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A522_AGAAGCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A523_CAAGCCAA': {'barcodes': 1523918,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32282,
                                'perfect_barcodes': 1491636,
                                'pf_Q30': 92.26,
                                'pf_clusters': 1523918,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.31,
                                'pf_read2_q30': 91.22,
                                'pf_yield': 460223236,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A523_CAAGCCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A524_CGGAGTAT': {'barcodes': 1983509,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 65074,
                                'perfect_barcodes': 1918435,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1983509,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.72,
                                'pf_read2_q30': 91.52,
                                'pf_yield': 599019718,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A524_CGGAGTAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A525_GAACGGTT': {'barcodes': 1671026,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35405,
                                'perfect_barcodes': 1635621,
                                'pf_Q30': 91.04,
                                'pf_clusters': 1671026,
                                'pf_qscore': 37.96,
                                'pf_read1_q30': 88.49,
                                'pf_read2_q30': 93.58,
                                'pf_yield': 504649852,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A525_GAACGGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A526_GGAAGAGA': {'barcodes': 1706879,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40473,
                                'perfect_barcodes': 1666406,
                                'pf_Q30': 91.84,
                                'pf_clusters': 1706879,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 91.75,
                                'pf_read2_q30': 91.92,
                                'pf_yield': 515477458,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A526_GGAAGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A527_GTTCCATG': {'barcodes': 1436715,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28156,
                                'perfect_barcodes': 1408559,
                                'pf_Q30': 91.34,
                                'pf_clusters': 1436715,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 89.56,
                                'pf_read2_q30': 93.11,
                                'pf_yield': 433887930,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A527_GTTCCATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A528_TCTTACGG': {'barcodes': 1856820,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38380,
                                'perfect_barcodes': 1818440,
                                'pf_Q30': 91.65,
                                'pf_clusters': 1856820,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 91.62,
                                'pf_read2_q30': 91.68,
                                'pf_yield': 560759640,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A528_TCTTACGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A529_AAGCGTTC': {'barcodes': 1903545,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40221,
                                'perfect_barcodes': 1863324,
                                'pf_Q30': 92.29,
                                'pf_clusters': 1903545,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.06,
                                'pf_read2_q30': 91.51,
                                'pf_yield': 574870590,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A529_AAGCGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A530_AGATCGTC': {'barcodes': 1944327,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37482,
                                'perfect_barcodes': 1906845,
                                'pf_Q30': 92.24,
                                'pf_clusters': 1944327,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 91.39,
                                'pf_yield': 587186754,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A530_AGATCGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A531_CAATGCGA': {'barcodes': 1829705,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34783,
                                'perfect_barcodes': 1794922,
                                'pf_Q30': 91.91,
                                'pf_clusters': 1829705,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.67,
                                'pf_read2_q30': 91.15,
                                'pf_yield': 552570910,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A531_CAATGCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A532_CGGTAATC': {'barcodes': 1906082,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41349,
                                'perfect_barcodes': 1864733,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1906082,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.74,
                                'pf_read2_q30': 91.29,
                                'pf_yield': 575636764,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A532_CGGTAATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A533_GAAGTGCT': {'barcodes': 1339652,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26964,
                                'perfect_barcodes': 1312688,
                                'pf_Q30': 91.35,
                                'pf_clusters': 1339652,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 89.22,
                                'pf_read2_q30': 93.49,
                                'pf_yield': 404574904,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A533_GAAGTGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A534_GGACTACT': {'barcodes': 1810178,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35684,
                                'perfect_barcodes': 1774494,
                                'pf_Q30': 92.1,
                                'pf_clusters': 1810178,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.15,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 546673756,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A534_GGACTACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A535_TAAGCGCA': {'barcodes': 1802309,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36272,
                                'perfect_barcodes': 1766037,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1802309,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.61,
                                'pf_read2_q30': 91.42,
                                'pf_yield': 544297318,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A535_TAAGCGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A536_TGAGACGA': {'barcodes': 2017160,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 42379,
                                'perfect_barcodes': 1974781,
                                'pf_Q30': 92.02,
                                'pf_clusters': 2017160,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.3,
                                'pf_read2_q30': 91.74,
                                'pf_yield': 609182320,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.42,
                                'sample_name': 'MIC2051A536_TGAGACGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A537_AATGACGC': {'barcodes': 1680974,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31646,
                                'perfect_barcodes': 1649328,
                                'pf_Q30': 91.49,
                                'pf_clusters': 1680974,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 91.67,
                                'pf_read2_q30': 91.32,
                                'pf_yield': 507654148,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A537_AATGACGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A538_AGCGTGTA': {'barcodes': 1406138,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1406138,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.62,
                                'pf_clusters': 1406138,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.65,
                                'pf_read2_q30': 90.59,
                                'pf_yield': 424653676,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A538_AGCGTGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A539_CATCAACC': {'barcodes': 1777748,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33812,
                                'perfect_barcodes': 1743936,
                                'pf_Q30': 92.18,
                                'pf_clusters': 1777748,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 92.46,
                                'pf_read2_q30': 91.89,
                                'pf_yield': 536879896,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A539_CATCAACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A540_CGTCCATT': {'barcodes': 2174298,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 49818,
                                'perfect_barcodes': 2124480,
                                'pf_Q30': 91.44,
                                'pf_clusters': 2174298,
                                'pf_qscore': 38.13,
                                'pf_read1_q30': 91.73,
                                'pf_read2_q30': 91.16,
                                'pf_yield': 656637996,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.45,
                                'sample_name': 'MIC2051A540_CGTCCATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A541_GACGTCAT': {'barcodes': 2065146,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 47949,
                                'perfect_barcodes': 2017197,
                                'pf_Q30': 91.81,
                                'pf_clusters': 2065146,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.72,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 623674092,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A541_GACGTCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A542_GGATGTAG': {'barcodes': 1878942,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 42570,
                                'perfect_barcodes': 1836372,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1878942,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 91.25,
                                'pf_read2_q30': 92.35,
                                'pf_yield': 567440484,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A542_GGATGTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A543_TACAGAGC': {'barcodes': 14491,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1692,
                                'perfect_barcodes': 12799,
                                'pf_Q30': 90.61,
                                'pf_clusters': 14491,
                                'pf_qscore': 37.91,
                                'pf_read1_q30': 91.49,
                                'pf_read2_q30': 89.73,
                                'pf_yield': 4376282,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A543_TACAGAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A544_TGCACTTG': {'barcodes': 1893135,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 39078,
                                'perfect_barcodes': 1854057,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1893135,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.4,
                                'pf_read2_q30': 91.74,
                                'pf_yield': 571726770,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A544_TGCACTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A545_ACAAGACG': {'barcodes': 1879754,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35842,
                                'perfect_barcodes': 1843912,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1879754,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.75,
                                'pf_read2_q30': 91.43,
                                'pf_yield': 567685708,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A545_ACAAGACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A546_AGGCAATG': {'barcodes': 1784552,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37950,
                                'perfect_barcodes': 1746602,
                                'pf_Q30': 92.24,
                                'pf_clusters': 1784552,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 538934704,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A546_AGGCAATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A547_CATGAGCA': {'barcodes': 1587824,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33602,
                                'perfect_barcodes': 1554222,
                                'pf_Q30': 92.22,
                                'pf_clusters': 1587824,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 91.13,
                                'pf_yield': 479522848,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A547_CATGAGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A548_CTACAAGG': {'barcodes': 1623624,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32351,
                                'perfect_barcodes': 1591273,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1623624,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 90.62,
                                'pf_yield': 490334448,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A548_CTACAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A549_GAGACCAA': {'barcodes': 1663382,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38191,
                                'perfect_barcodes': 1625191,
                                'pf_Q30': 91.66,
                                'pf_clusters': 1663382,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 90.7,
                                'pf_read2_q30': 92.62,
                                'pf_yield': 502341364,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A549_GAGACCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A550_GGCGAATA': {'barcodes': 3108002,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 63750,
                                'perfect_barcodes': 3044252,
                                'pf_Q30': 91.75,
                                'pf_clusters': 3108002,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.75,
                                'pf_read2_q30': 89.76,
                                'pf_yield': 938616604,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.64,
                                'sample_name': 'MIC2051A550_GGCGAATA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A551_TAGCAGGA': {'barcodes': 1218576,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1218576,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.62,
                                'pf_clusters': 1218576,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 92.08,
                                'pf_read2_q30': 91.15,
                                'pf_yield': 368009952,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A551_TAGCAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A552_TGCTCTAC': {'barcodes': 15935,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1651,
                                'perfect_barcodes': 14284,
                                'pf_Q30': 91.09,
                                'pf_clusters': 15935,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 91.41,
                                'pf_read2_q30': 90.76,
                                'pf_yield': 4812370,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A552_TGCTCTAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A553_ACACGAGA': {'barcodes': 1605468,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35939,
                                'perfect_barcodes': 1569529,
                                'pf_Q30': 91.52,
                                'pf_clusters': 1605468,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 92.22,
                                'pf_read2_q30': 90.81,
                                'pf_yield': 484851336,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A553_ACACGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A554_AGGTTCCT': {'barcodes': 2014182,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 55271,
                                'perfect_barcodes': 1958911,
                                'pf_Q30': 91.97,
                                'pf_clusters': 2014182,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.66,
                                'pf_read2_q30': 91.27,
                                'pf_yield': 608282964,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.42,
                                'sample_name': 'MIC2051A554_AGGTTCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A555_CCAACTTC': {'barcodes': 1740655,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36132,
                                'perfect_barcodes': 1704523,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1740655,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.38,
                                'pf_read2_q30': 91.63,
                                'pf_yield': 525677810,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A555_CCAACTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A556_CTATCCAC': {'barcodes': 1464852,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28261,
                                'perfect_barcodes': 1436591,
                                'pf_Q30': 91.7,
                                'pf_clusters': 1464852,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 92.09,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 442385304,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A556_CTATCCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A557_GAGTAGAG': {'barcodes': 1515908,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 43591,
                                'perfect_barcodes': 1472317,
                                'pf_Q30': 90.84,
                                'pf_clusters': 1515908,
                                'pf_qscore': 37.9,
                                'pf_read1_q30': 88.18,
                                'pf_read2_q30': 93.5,
                                'pf_yield': 457804216,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A557_GAGTAGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A558_GGTATAGG': {'barcodes': 1700868,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 41738,
                                'perfect_barcodes': 1659130,
                                'pf_Q30': 91.37,
                                'pf_clusters': 1700868,
                                'pf_qscore': 38.09,
                                'pf_read1_q30': 91.01,
                                'pf_read2_q30': 91.74,
                                'pf_yield': 513662136,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A558_GGTATAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A559_TAGCTTCC': {'barcodes': 1659862,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33854,
                                'perfect_barcodes': 1626008,
                                'pf_Q30': 91.53,
                                'pf_clusters': 1659862,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 91.59,
                                'pf_read2_q30': 91.48,
                                'pf_yield': 501278324,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A559_TAGCTTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A560_TGGCTACA': {'barcodes': 1967535,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40746,
                                'perfect_barcodes': 1926789,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1967535,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.96,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 594195570,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A560_TGGCTACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A561_ACAGCAAG': {'barcodes': 33202,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33202,
                                'perfect_barcodes': 0,
                                'pf_Q30': 81.54,
                                'pf_clusters': 33202,
                                'pf_qscore': 35.37,
                                'pf_read1_q30': 83.2,
                                'pf_read2_q30': 79.87,
                                'pf_yield': 10027004,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A561_ACAGCAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A562_AGTCGAAG': {'barcodes': 1928566,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40142,
                                'perfect_barcodes': 1888424,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1928566,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.06,
                                'pf_read2_q30': 91.22,
                                'pf_yield': 582426932,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A562_AGTCGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A563_CCGAAGAT': {'barcodes': 1975864,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 43908,
                                'perfect_barcodes': 1931956,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1975864,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.73,
                                'pf_read2_q30': 91.46,
                                'pf_yield': 596710928,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A563_CCGAAGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A564_CTCAGAAG': {'barcodes': 1574544,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32816,
                                'perfect_barcodes': 1541728,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1574544,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.38,
                                'pf_read2_q30': 90.81,
                                'pf_yield': 475512288,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A564_CTCAGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A565_GATCCACT': {'barcodes': 1157170,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 15724,
                                'perfect_barcodes': 1141446,
                                'pf_Q30': 80.65,
                                'pf_clusters': 1157170,
                                'pf_qscore': 35.2,
                                'pf_read1_q30': 65.21,
                                'pf_read2_q30': 96.09,
                                'pf_yield': 349465340,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A565_GATCCACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A566_GGTTAGCT': {'barcodes': 1224099,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26230,
                                'perfect_barcodes': 1197869,
                                'pf_Q30': 91.65,
                                'pf_clusters': 1224099,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 91.62,
                                'pf_read2_q30': 91.68,
                                'pf_yield': 369677898,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A566_GGTTAGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A567_TAGTGCCA': {'barcodes': 1559132,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33704,
                                'perfect_barcodes': 1525428,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1559132,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.32,
                                'pf_read2_q30': 91.54,
                                'pf_yield': 470857864,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A567_TAGTGCCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A568_TGTCGACT': {'barcodes': 1645447,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35485,
                                'perfect_barcodes': 1609962,
                                'pf_Q30': 91.75,
                                'pf_clusters': 1645447,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.12,
                                'pf_read2_q30': 91.39,
                                'pf_yield': 496924994,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A568_TGTCGACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A569_ACCGAATG': {'barcodes': 2234724,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 45282,
                                'perfect_barcodes': 2189442,
                                'pf_Q30': 89.54,
                                'pf_clusters': 2234724,
                                'pf_qscore': 37.61,
                                'pf_read1_q30': 89.3,
                                'pf_read2_q30': 89.78,
                                'pf_yield': 674886648,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.46,
                                'sample_name': 'MIC2051A569_ACCGAATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A570_AGTGGCAA': {'barcodes': 1830171,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 40010,
                                'perfect_barcodes': 1790161,
                                'pf_Q30': 91.7,
                                'pf_clusters': 1830171,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.66,
                                'pf_read2_q30': 90.75,
                                'pf_yield': 552711642,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A570_AGTGGCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A571_CCGTAACT': {'barcodes': 1628203,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31622,
                                'perfect_barcodes': 1596581,
                                'pf_Q30': 91.87,
                                'pf_clusters': 1628203,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.32,
                                'pf_read2_q30': 91.43,
                                'pf_yield': 491717306,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A571_CCGTAACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A572_CTCGTTCT': {'barcodes': 3160088,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1315846,
                                'perfect_barcodes': 1844242,
                                'pf_Q30': 92.2,
                                'pf_clusters': 3160088,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.06,
                                'pf_read2_q30': 91.34,
                                'pf_yield': 954346576,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.65,
                                'sample_name': 'MIC2051A572_CTCGTTCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A573_GATGGAGT': {'barcodes': 1322032,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30967,
                                'perfect_barcodes': 1291065,
                                'pf_Q30': 88.59,
                                'pf_clusters': 1322032,
                                'pf_qscore': 37.27,
                                'pf_read1_q30': 82.92,
                                'pf_read2_q30': 94.26,
                                'pf_yield': 399253664,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A573_GATGGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A574_GTAAGCAC': {'barcodes': 1941347,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 37625,
                                'perfect_barcodes': 1903722,
                                'pf_Q30': 91.75,
                                'pf_clusters': 1941347,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.18,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 586286794,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A574_GTAAGCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A575_TATGACCG': {'barcodes': 1372677,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27811,
                                'perfect_barcodes': 1344866,
                                'pf_Q30': 91.62,
                                'pf_clusters': 1372677,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 91.86,
                                'pf_read2_q30': 91.38,
                                'pf_yield': 414548454,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A575_TATGACCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A576_TTCCAGGT': {'barcodes': 1905295,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34657,
                                'perfect_barcodes': 1870638,
                                'pf_Q30': 91.82,
                                'pf_clusters': 1905295,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 91.64,
                                'pf_read2_q30': 91.99,
                                'pf_yield': 575399090,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A576_TTCCAGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A577_ACCTAGAC': {'barcodes': 2226741,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 38270,
                                'perfect_barcodes': 2188471,
                                'pf_Q30': 92.32,
                                'pf_clusters': 2226741,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 92.88,
                                'pf_read2_q30': 91.77,
                                'pf_yield': 672475782,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.46,
                                'sample_name': 'MIC2051A577_ACCTAGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A578_ATACTGGC': {'barcodes': 1903978,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 35462,
                                'perfect_barcodes': 1868516,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1903978,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.9,
                                'pf_read2_q30': 91.09,
                                'pf_yield': 575001356,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A578_ATACTGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A579_CCTAGAGA': {'barcodes': 1571490,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34423,
                                'perfect_barcodes': 1537067,
                                'pf_Q30': 91.83,
                                'pf_clusters': 1571490,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.49,
                                'pf_read2_q30': 90.16,
                                'pf_yield': 474589980,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A579_CCTAGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A580_CTCTGGAT': {'barcodes': 2095300,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 47184,
                                'perfect_barcodes': 2048116,
                                'pf_Q30': 91.92,
                                'pf_clusters': 2095300,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.54,
                                'pf_read2_q30': 91.29,
                                'pf_yield': 632780600,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A580_CTCTGGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A581_GCACACAA': {'barcodes': 1441401,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31849,
                                'perfect_barcodes': 1409552,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1441401,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.21,
                                'pf_read2_q30': 91.59,
                                'pf_yield': 435303102,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A581_GCACACAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A582_GTACGATC': {'barcodes': 1627667,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 33226,
                                'perfect_barcodes': 1594441,
                                'pf_Q30': 91.89,
                                'pf_clusters': 1627667,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 91.65,
                                'pf_read2_q30': 92.14,
                                'pf_yield': 491555434,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A582_GTACGATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A583_TCACTCGA': {'barcodes': 1109440,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 24923,
                                'perfect_barcodes': 1084517,
                                'pf_Q30': 91.61,
                                'pf_clusters': 1109440,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 90.74,
                                'pf_read2_q30': 92.48,
                                'pf_yield': 335050880,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A583_TCACTCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A584_TTCGGCTA': {'barcodes': 1801166,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36881,
                                'perfect_barcodes': 1764285,
                                'pf_Q30': 91.69,
                                'pf_clusters': 1801166,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 91.85,
                                'pf_read2_q30': 91.53,
                                'pf_yield': 543952132,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A584_TTCGGCTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A585_AGCGTGTT': {'barcodes': 1912174,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1912174,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.91,
                                'pf_clusters': 1912174,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.51,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 577476548,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A585_AGCGTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A586_AGAAGCGT': {'barcodes': 3239748,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1765584,
                                'perfect_barcodes': 1474164,
                                'pf_Q30': 91.09,
                                'pf_clusters': 3239748,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 91.59,
                                'pf_read2_q30': 90.6,
                                'pf_yield': 978403896,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.67,
                                'sample_name': 'MIC2051A586_AGAAGCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A587_ACCACGAT': {'barcodes': 1349529,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27766,
                                'perfect_barcodes': 1321763,
                                'pf_Q30': 91.94,
                                'pf_clusters': 1349529,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.62,
                                'pf_read2_q30': 90.26,
                                'pf_yield': 407557758,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A587_ACCACGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A588_TTCCAAGG': {'barcodes': 1049138,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27697,
                                'perfect_barcodes': 1021441,
                                'pf_Q30': 90.7,
                                'pf_clusters': 1049138,
                                'pf_qscore': 37.89,
                                'pf_read1_q30': 92.15,
                                'pf_read2_q30': 89.25,
                                'pf_yield': 316839676,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A588_TTCCAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A589_TGGTACAG': {'barcodes': 1141509,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25057,
                                'perfect_barcodes': 1116452,
                                'pf_Q30': 90.74,
                                'pf_clusters': 1141509,
                                'pf_qscore': 37.92,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 88.11,
                                'pf_yield': 344735718,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A589_TGGTACAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A590_GTTAAGGC': {'barcodes': 1032656,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 18727,
                                'perfect_barcodes': 1013929,
                                'pf_Q30': 90.8,
                                'pf_clusters': 1032656,
                                'pf_qscore': 37.91,
                                'pf_read1_q30': 92.11,
                                'pf_read2_q30': 89.49,
                                'pf_yield': 311862112,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A590_GTTAAGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A591_AACAGGAC': {'barcodes': 1348375,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25293,
                                'perfect_barcodes': 1323082,
                                'pf_Q30': 92.18,
                                'pf_clusters': 1348375,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 90.91,
                                'pf_yield': 407209250,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A591_AACAGGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A592_GCAATGGA': {'barcodes': 1232140,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27804,
                                'perfect_barcodes': 1204336,
                                'pf_Q30': 90.66,
                                'pf_clusters': 1232140,
                                'pf_qscore': 37.89,
                                'pf_read1_q30': 90.11,
                                'pf_read2_q30': 91.21,
                                'pf_yield': 372106280,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A592_GCAATGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A593_CACCTGTT': {'barcodes': 1474375,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32082,
                                'perfect_barcodes': 1442293,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1474375,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.22,
                                'pf_read2_q30': 91.2,
                                'pf_yield': 445261250,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A593_CACCTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A594_GTAGGAGT': {'barcodes': 1289591,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 42342,
                                'perfect_barcodes': 1247249,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1289591,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 91.78,
                                'pf_read2_q30': 91.73,
                                'pf_yield': 389456482,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A594_GTAGGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A595_CTAGGCAT': {'barcodes': 1421250,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29686,
                                'perfect_barcodes': 1391564,
                                'pf_Q30': 91.78,
                                'pf_clusters': 1421250,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 92.78,
                                'pf_read2_q30': 90.77,
                                'pf_yield': 429217500,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A595_CTAGGCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A596_GCAATTCG': {'barcodes': 23307,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23307,
                                'perfect_barcodes': 0,
                                'pf_Q30': 81.89,
                                'pf_clusters': 23307,
                                'pf_qscore': 35.46,
                                'pf_read1_q30': 82.93,
                                'pf_read2_q30': 80.85,
                                'pf_yield': 7038714,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A596_GCAATTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A597_GCTGTAAG': {'barcodes': 1231613,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 22924,
                                'perfect_barcodes': 1208689,
                                'pf_Q30': 91.68,
                                'pf_clusters': 1231613,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 92.24,
                                'pf_read2_q30': 91.12,
                                'pf_yield': 371947126,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A597_GCTGTAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A599_GATGAGAC': {'barcodes': 1066121,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 20263,
                                'perfect_barcodes': 1045858,
                                'pf_Q30': 91.08,
                                'pf_clusters': 1066121,
                                'pf_qscore': 37.97,
                                'pf_read1_q30': 88.71,
                                'pf_read2_q30': 93.44,
                                'pf_yield': 321968542,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A599_GATGAGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A600_TACCAGGA': {'barcodes': 1651801,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1651801,
                                'perfect_barcodes': 0,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1651801,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.78,
                                'pf_read2_q30': 91.64,
                                'pf_yield': 498843902,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A600_TACCAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A601_CGTAGGTT': {'barcodes': 1205364,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30363,
                                'perfect_barcodes': 1175001,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1205364,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 89.63,
                                'pf_yield': 364019928,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A601_CGTAGGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A602_TTCGCAGT': {'barcodes': 1466696,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31612,
                                'perfect_barcodes': 1435084,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1466696,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.2,
                                'pf_read2_q30': 91.09,
                                'pf_yield': 442942192,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A602_TTCGCAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A603_CGGCTAAT': {'barcodes': 1273302,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27439,
                                'perfect_barcodes': 1245863,
                                'pf_Q30': 91.82,
                                'pf_clusters': 1273302,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.94,
                                'pf_read2_q30': 89.69,
                                'pf_yield': 384537204,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A603_CGGCTAAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A604_CTAACTCG': {'barcodes': 1371912,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28559,
                                'perfect_barcodes': 1343353,
                                'pf_Q30': 91.96,
                                'pf_clusters': 1371912,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.65,
                                'pf_read2_q30': 91.27,
                                'pf_yield': 414317424,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A604_CTAACTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A605_AACCGAAG': {'barcodes': 1265227,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26856,
                                'perfect_barcodes': 1238371,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1265227,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 93.19,
                                'pf_read2_q30': 90.8,
                                'pf_yield': 382098554,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A605_AACCGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A606_AACTGAGC': {'barcodes': 1258857,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23724,
                                'perfect_barcodes': 1235133,
                                'pf_Q30': 92.2,
                                'pf_clusters': 1258857,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.35,
                                'pf_read2_q30': 91.06,
                                'pf_yield': 380174814,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A606_AACTGAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A607_AAGGACAC': {'barcodes': 1219178,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 24328,
                                'perfect_barcodes': 1194850,
                                'pf_Q30': 92.2,
                                'pf_clusters': 1219178,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.11,
                                'pf_read2_q30': 91.3,
                                'pf_yield': 368191756,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A607_AAGGACAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A608_GAATCCGA': {'barcodes': 1022487,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 22010,
                                'perfect_barcodes': 1000477,
                                'pf_Q30': 90.57,
                                'pf_clusters': 1022487,
                                'pf_qscore': 37.83,
                                'pf_read1_q30': 87.33,
                                'pf_read2_q30': 93.81,
                                'pf_yield': 308791074,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A608_GAATCCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A609_GGTGTCTT': {'barcodes': 1321250,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 34281,
                                'perfect_barcodes': 1286969,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1321250,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.5,
                                'pf_read2_q30': 91.65,
                                'pf_yield': 399017500,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A609_GGTGTCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A610_CTGGTTCT': {'barcodes': 32487,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 32487,
                                'perfect_barcodes': 0,
                                'pf_Q30': 78.01,
                                'pf_clusters': 32487,
                                'pf_qscore': 34.45,
                                'pf_read1_q30': 80.18,
                                'pf_read2_q30': 75.83,
                                'pf_yield': 9811074,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A610_CTGGTTCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A611_GCTTCTTG': {'barcodes': 1091205,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 21063,
                                'perfect_barcodes': 1070142,
                                'pf_Q30': 91.36,
                                'pf_clusters': 1091205,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 90.02,
                                'pf_read2_q30': 92.7,
                                'pf_yield': 329543910,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A611_GCTTCTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A612_TATCAGCG': {'barcodes': 955728,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 19156,
                                'perfect_barcodes': 936572,
                                'pf_Q30': 91.55,
                                'pf_clusters': 955728,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 93.71,
                                'pf_read2_q30': 89.4,
                                'pf_yield': 288629856,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.2,
                                'sample_name': 'MIC2051A612_TATCAGCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A613_CTTCGTTC': {'barcodes': 1076274,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25534,
                                'perfect_barcodes': 1050740,
                                'pf_Q30': 91.89,
                                'pf_clusters': 1076274,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.69,
                                'pf_read2_q30': 90.1,
                                'pf_yield': 325034748,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A613_CTTCGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A614_CATGTTCC': {'barcodes': 1159727,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23678,
                                'perfect_barcodes': 1136049,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1159727,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.4,
                                'pf_read2_q30': 90.65,
                                'pf_yield': 350237554,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A614_CATGTTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A615_GCCATAAC': {'barcodes': 1225778,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25051,
                                'perfect_barcodes': 1200727,
                                'pf_Q30': 92.13,
                                'pf_clusters': 1225778,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.78,
                                'pf_read2_q30': 90.48,
                                'pf_yield': 370184956,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A615_GCCATAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A616_ACGACAGA': {'barcodes': 1141264,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26005,
                                'perfect_barcodes': 1115259,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1141264,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.96,
                                'pf_read2_q30': 90.83,
                                'pf_yield': 344661728,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A616_ACGACAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A617_AGTCGCTT': {'barcodes': 1198145,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27182,
                                'perfect_barcodes': 1170963,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1198145,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.9,
                                'pf_read2_q30': 90.16,
                                'pf_yield': 361839790,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A617_AGTCGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A618_CCGTATCT': {'barcodes': 1166249,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25646,
                                'perfect_barcodes': 1140603,
                                'pf_Q30': 91.69,
                                'pf_clusters': 1166249,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 93.52,
                                'pf_read2_q30': 89.87,
                                'pf_yield': 352207198,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A618_CCGTATCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A619_CCTGATTG': {'barcodes': 1130296,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 24232,
                                'perfect_barcodes': 1106064,
                                'pf_Q30': 92.29,
                                'pf_clusters': 1130296,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.38,
                                'pf_read2_q30': 91.21,
                                'pf_yield': 341349392,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A619_CCTGATTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A620_TAATGCCG': {'barcodes': 1008952,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 19337,
                                'perfect_barcodes': 989615,
                                'pf_Q30': 91.41,
                                'pf_clusters': 1008952,
                                'pf_qscore': 38.1,
                                'pf_read1_q30': 91.04,
                                'pf_read2_q30': 91.79,
                                'pf_yield': 304703504,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A620_TAATGCCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A621_GGATCTTC': {'barcodes': 1032058,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23241,
                                'perfect_barcodes': 1008817,
                                'pf_Q30': 91.37,
                                'pf_clusters': 1032058,
                                'pf_qscore': 38.07,
                                'pf_read1_q30': 89.65,
                                'pf_read2_q30': 93.1,
                                'pf_yield': 311681516,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A621_GGATCTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A622_TGATGTCC': {'barcodes': 1229220,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26148,
                                'perfect_barcodes': 1203072,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1229220,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.48,
                                'pf_read2_q30': 91.59,
                                'pf_yield': 371224440,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A622_TGATGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A623_ATGCCAAC': {'barcodes': 1353459,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26928,
                                'perfect_barcodes': 1326531,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1353459,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.7,
                                'pf_read2_q30': 91.31,
                                'pf_yield': 408744618,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A623_ATGCCAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A624_AACGGTCA': {'barcodes': 1111093,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 29017,
                                'perfect_barcodes': 1082076,
                                'pf_Q30': 91.73,
                                'pf_clusters': 1111093,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.16,
                                'pf_read2_q30': 90.29,
                                'pf_yield': 335550086,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A624_AACGGTCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A625_CAGCGATT': {'barcodes': 1227278,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23662,
                                'perfect_barcodes': 1203616,
                                'pf_Q30': 92.13,
                                'pf_clusters': 1227278,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 90.68,
                                'pf_yield': 370637956,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A625_CAGCGATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A626_ATCTCGCT': {'barcodes': 1218990,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23323,
                                'perfect_barcodes': 1195667,
                                'pf_Q30': 90.95,
                                'pf_clusters': 1218990,
                                'pf_qscore': 37.99,
                                'pf_read1_q30': 93.24,
                                'pf_read2_q30': 88.66,
                                'pf_yield': 368134980,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A626_ATCTCGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A627_CGAGTATG': {'barcodes': 1060848,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27578,
                                'perfect_barcodes': 1033270,
                                'pf_Q30': 91.15,
                                'pf_clusters': 1060848,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 93.31,
                                'pf_read2_q30': 88.99,
                                'pf_yield': 320376096,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A627_CGAGTATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A628_TCTTGACG': {'barcodes': 1189998,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23893,
                                'perfect_barcodes': 1166105,
                                'pf_Q30': 91.97,
                                'pf_clusters': 1189998,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.07,
                                'pf_read2_q30': 90.88,
                                'pf_yield': 359379396,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A628_TCTTGACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A629_TATCGGTC': {'barcodes': 1246086,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 26125,
                                'perfect_barcodes': 1219961,
                                'pf_Q30': 91.88,
                                'pf_clusters': 1246086,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 92.72,
                                'pf_read2_q30': 91.03,
                                'pf_yield': 376317972,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A629_TATCGGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A630_TCAGATCC': {'barcodes': 1256636,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 23195,
                                'perfect_barcodes': 1233441,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1256636,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 91.8,
                                'pf_read2_q30': 92.27,
                                'pf_yield': 379504072,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A630_TCAGATCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A631_CGTGTGTA': {'barcodes': 1290678,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 36113,
                                'perfect_barcodes': 1254565,
                                'pf_Q30': 91.98,
                                'pf_clusters': 1290678,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.07,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 389784756,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A631_CGTGTGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A632_GTCTTGCA': {'barcodes': 943965,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 18327,
                                'perfect_barcodes': 925638,
                                'pf_Q30': 91.39,
                                'pf_clusters': 943965,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 91.05,
                                'pf_read2_q30': 91.72,
                                'pf_yield': 285077430,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.2,
                                'sample_name': 'MIC2051A632_GTCTTGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A633_CCGGAATT': {'barcodes': 1283292,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 30279,
                                'perfect_barcodes': 1253013,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1283292,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.46,
                                'pf_read2_q30': 90.59,
                                'pf_yield': 387554184,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A633_CCGGAATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A634_AACCTCCT': {'barcodes': 1568147,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 31970,
                                'perfect_barcodes': 1536177,
                                'pf_Q30': 89.82,
                                'pf_clusters': 1568147,
                                'pf_qscore': 37.67,
                                'pf_read1_q30': 92.97,
                                'pf_read2_q30': 86.67,
                                'pf_yield': 473580394,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A634_AACCTCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A635_AGCAGATG': {'barcodes': 1360474,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27114,
                                'perfect_barcodes': 1333360,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1360474,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.62,
                                'pf_read2_q30': 90.6,
                                'pf_yield': 410863148,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A635_AGCAGATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A636_GGTCTTAG': {'barcodes': 1140750,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 28212,
                                'perfect_barcodes': 1112538,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1140750,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 91.92,
                                'pf_read2_q30': 91.57,
                                'pf_yield': 344506500,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A636_GGTCTTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A637_ACCTTCTC': {'barcodes': 1168386,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 27479,
                                'perfect_barcodes': 1140907,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1168386,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 90.68,
                                'pf_yield': 352852572,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A637_ACCTTCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A638_AGAATGCC': {'barcodes': 1455254,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 1455254,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1455254,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.63,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 439486708,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A638_AGAATGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A639_GCGTTCTA': {'barcodes': 1061961,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 25280,
                                'perfect_barcodes': 1036681,
                                'pf_Q30': 91.55,
                                'pf_clusters': 1061961,
                                'pf_qscore': 38.12,
                                'pf_read1_q30': 91.71,
                                'pf_read2_q30': 91.38,
                                'pf_yield': 320712222,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A639_GCGTTCTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A640_GTTACGCA': {'barcodes': 15067,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 5569,
                                'perfect_barcodes': 9498,
                                'pf_Q30': 89.59,
                                'pf_clusters': 15067,
                                'pf_qscore': 37.62,
                                'pf_read1_q30': 90.25,
                                'pf_read2_q30': 88.92,
                                'pf_yield': 4550234,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A640_GTTACGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A641_CCTAAGTC': {'barcodes': 22356,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 2492,
                                'perfect_barcodes': 19864,
                                'pf_Q30': 86.62,
                                'pf_clusters': 22356,
                                'pf_qscore': 36.84,
                                'pf_read1_q30': 88.8,
                                'pf_read2_q30': 84.43,
                                'pf_yield': 6751512,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A641_CCTAAGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A643_ATAGCGGT': {'barcodes': 24832,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 5662,
                                'perfect_barcodes': 19170,
                                'pf_Q30': 86.38,
                                'pf_clusters': 24832,
                                'pf_qscore': 36.78,
                                'pf_read1_q30': 88.2,
                                'pf_read2_q30': 84.57,
                                'pf_yield': 7499264,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A643_ATAGCGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A644_ACGATCAG': {'barcodes': 14970,
                                'flowcell': 'H37GVALXX',
                                'lane': '7',
                                'one_mismatch_barcodes': 2007,
                                'perfect_barcodes': 12963,
                                'pf_Q30': 90.14,
                                'pf_clusters': 14970,
                                'pf_qscore': 37.77,
                                'pf_read1_q30': 90.49,
                                'pf_read2_q30': 89.8,
                                'pf_yield': 4520940,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A644_ACGATCAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0}},
 '8': {'MIC2051A321_GACACAGT': {'barcodes': 1018430,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34129,
                                'perfect_barcodes': 984301,
                                'pf_Q30': 91.03,
                                'pf_clusters': 1018430,
                                'pf_qscore': 37.96,
                                'pf_read1_q30': 89.83,
                                'pf_read2_q30': 92.24,
                                'pf_yield': 307565860,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A321_GACACAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A322_TTGCTTGG': {'barcodes': 14005,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 7330,
                                'perfect_barcodes': 6675,
                                'pf_Q30': 86.11,
                                'pf_clusters': 14005,
                                'pf_qscore': 36.68,
                                'pf_read1_q30': 86.4,
                                'pf_read2_q30': 85.82,
                                'pf_yield': 4229510,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A322_TTGCTTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A323_TATGGCAC': {'barcodes': 1213269,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37051,
                                'perfect_barcodes': 1176218,
                                'pf_Q30': 91.33,
                                'pf_clusters': 1213269,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 92.98,
                                'pf_read2_q30': 89.68,
                                'pf_yield': 366407238,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A323_TATGGCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A324_GGCGAATA': {'barcodes': 2674302,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 86164,
                                'perfect_barcodes': 2588138,
                                'pf_Q30': 91.77,
                                'pf_clusters': 2674302,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.63,
                                'pf_read2_q30': 89.91,
                                'pf_yield': 807639204,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.64,
                                'sample_name': 'MIC2051A324_GGCGAATA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A325_TCACCTAG': {'barcodes': 9073,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 2578,
                                'perfect_barcodes': 6495,
                                'pf_Q30': 89.5,
                                'pf_clusters': 9073,
                                'pf_qscore': 37.57,
                                'pf_read1_q30': 88.69,
                                'pf_read2_q30': 90.32,
                                'pf_yield': 2740046,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A325_TCACCTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A326_AACCGTGT': {'barcodes': 1105259,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36972,
                                'perfect_barcodes': 1068287,
                                'pf_Q30': 90.7,
                                'pf_clusters': 1105259,
                                'pf_qscore': 37.89,
                                'pf_read1_q30': 92.88,
                                'pf_read2_q30': 88.52,
                                'pf_yield': 333788218,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A326_AACCGTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A327_ATAGGTCC': {'barcodes': 1143485,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36048,
                                'perfect_barcodes': 1107437,
                                'pf_Q30': 91.66,
                                'pf_clusters': 1143485,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 93.58,
                                'pf_read2_q30': 89.73,
                                'pf_yield': 345332470,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A327_ATAGGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A328_AGTATGCC': {'barcodes': 964559,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 964559,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.48,
                                'pf_clusters': 964559,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 93.68,
                                'pf_read2_q30': 89.28,
                                'pf_yield': 291296818,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A328_AGTATGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A329_GCATAACG': {'barcodes': 1090113,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36703,
                                'perfect_barcodes': 1053410,
                                'pf_Q30': 91.32,
                                'pf_clusters': 1090113,
                                'pf_qscore': 38.11,
                                'pf_read1_q30': 91.74,
                                'pf_read2_q30': 90.89,
                                'pf_yield': 329214126,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A329_GCATAACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A330_CCTGTCAA': {'barcodes': 1125346,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37365,
                                'perfect_barcodes': 1087981,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1125346,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.4,
                                'pf_read2_q30': 90.83,
                                'pf_yield': 339854492,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A330_CCTGTCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A331_ATAACGCC': {'barcodes': 1286387,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43292,
                                'perfect_barcodes': 1243095,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1286387,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.13,
                                'pf_read2_q30': 91.07,
                                'pf_yield': 388488874,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A331_ATAACGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A332_AAGTGCAG': {'barcodes': 1167662,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37966,
                                'perfect_barcodes': 1129696,
                                'pf_Q30': 90.57,
                                'pf_clusters': 1167662,
                                'pf_qscore': 37.88,
                                'pf_read1_q30': 90.37,
                                'pf_read2_q30': 90.76,
                                'pf_yield': 352633924,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A332_AAGTGCAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A333_CAAGTCGT': {'barcodes': 1212178,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40518,
                                'perfect_barcodes': 1171660,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1212178,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.48,
                                'pf_read2_q30': 90.75,
                                'pf_yield': 366077756,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A333_CAAGTCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A334_CGGTGGTA': {'barcodes': 1158218,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49991,
                                'perfect_barcodes': 1108227,
                                'pf_Q30': 91.45,
                                'pf_clusters': 1158218,
                                'pf_qscore': 38.13,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 89.58,
                                'pf_yield': 349781836,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A334_CGGTGGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A335_ACCGACAA': {'barcodes': 1108761,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34364,
                                'perfect_barcodes': 1074397,
                                'pf_Q30': 91.83,
                                'pf_clusters': 1108761,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.38,
                                'pf_read2_q30': 90.27,
                                'pf_yield': 334845822,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A335_ACCGACAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A336_TACTGCTC': {'barcodes': 1247697,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37473,
                                'perfect_barcodes': 1210224,
                                'pf_Q30': 91.29,
                                'pf_clusters': 1247697,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 91.67,
                                'pf_read2_q30': 90.92,
                                'pf_yield': 376804494,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A336_TACTGCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A337_ACAGAGGT': {'barcodes': 1176139,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42012,
                                'perfect_barcodes': 1134127,
                                'pf_Q30': 92.19,
                                'pf_clusters': 1176139,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 91.01,
                                'pf_yield': 355193978,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A337_ACAGAGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A338_AGCCTATC': {'barcodes': 1299118,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42450,
                                'perfect_barcodes': 1256668,
                                'pf_Q30': 92.15,
                                'pf_clusters': 1299118,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.52,
                                'pf_read2_q30': 90.78,
                                'pf_yield': 392333636,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A338_AGCCTATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A339_GTAGTACC': {'barcodes': 1324454,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43255,
                                'perfect_barcodes': 1281199,
                                'pf_Q30': 91.79,
                                'pf_clusters': 1324454,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 92.62,
                                'pf_read2_q30': 90.96,
                                'pf_yield': 399985108,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A339_GTAGTACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A340_GCAATTCC': {'barcodes': 2346279,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 951055,
                                'perfect_barcodes': 1395224,
                                'pf_Q30': 92.02,
                                'pf_clusters': 2346279,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.7,
                                'pf_read2_q30': 91.34,
                                'pf_yield': 708576258,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.56,
                                'sample_name': 'MIC2051A340_GCAATTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A341_CTGTATGC': {'barcodes': 2537372,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 76381,
                                'perfect_barcodes': 2460991,
                                'pf_Q30': 91.89,
                                'pf_clusters': 2537372,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.27,
                                'pf_read2_q30': 90.51,
                                'pf_yield': 766286344,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.6,
                                'sample_name': 'MIC2051A341_CTGTATGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A342_CTAGCAGT': {'barcodes': 1321639,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 31160,
                                'perfect_barcodes': 1290479,
                                'pf_Q30': 91.78,
                                'pf_clusters': 1321639,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 93.5,
                                'pf_read2_q30': 90.06,
                                'pf_yield': 399134978,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A342_CTAGCAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A343_AACACTGG': {'barcodes': 1233882,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34349,
                                'perfect_barcodes': 1199533,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1233882,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 90.38,
                                'pf_yield': 372632364,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A343_AACACTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A344_TGCTTGCT': {'barcodes': 1499974,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50247,
                                'perfect_barcodes': 1449727,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1499974,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.73,
                                'pf_read2_q30': 91.44,
                                'pf_yield': 452992148,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A344_TGCTTGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A345_CCACTAAG': {'barcodes': 1054450,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32470,
                                'perfect_barcodes': 1021980,
                                'pf_Q30': 92.19,
                                'pf_clusters': 1054450,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.25,
                                'pf_read2_q30': 91.13,
                                'pf_yield': 318443900,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A345_CCACTAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A346_TGATCACG': {'barcodes': 1174298,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37624,
                                'perfect_barcodes': 1136674,
                                'pf_Q30': 91.19,
                                'pf_clusters': 1174298,
                                'pf_qscore': 38.0,
                                'pf_read1_q30': 90.52,
                                'pf_read2_q30': 91.86,
                                'pf_yield': 354637996,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A346_TGATCACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A347_CGCGTATT': {'barcodes': 1373116,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 52302,
                                'perfect_barcodes': 1320814,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1373116,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.2,
                                'pf_read2_q30': 90.85,
                                'pf_yield': 414681032,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A347_CGCGTATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A348_CTTCGCAA': {'barcodes': 1324254,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41899,
                                'perfect_barcodes': 1282355,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1324254,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.17,
                                'pf_read2_q30': 90.44,
                                'pf_yield': 399924708,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A348_CTTCGCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A349_AGTTCGCA': {'barcodes': 1213697,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38157,
                                'perfect_barcodes': 1175540,
                                'pf_Q30': 91.72,
                                'pf_clusters': 1213697,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 93.6,
                                'pf_read2_q30': 89.84,
                                'pf_yield': 366536494,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A349_AGTTCGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A350_ACCTCTTC': {'barcodes': 1282758,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46290,
                                'perfect_barcodes': 1236468,
                                'pf_Q30': 92.08,
                                'pf_clusters': 1282758,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.1,
                                'pf_read2_q30': 91.06,
                                'pf_yield': 387392916,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A350_ACCTCTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A351_ACCATAGG': {'barcodes': 1171594,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39380,
                                'perfect_barcodes': 1132214,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1171594,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.21,
                                'pf_read2_q30': 90.63,
                                'pf_yield': 353821388,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A351_ACCATAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A352_TCCACGTT': {'barcodes': 13718,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 2506,
                                'perfect_barcodes': 11212,
                                'pf_Q30': 89.73,
                                'pf_clusters': 13718,
                                'pf_qscore': 37.62,
                                'pf_read1_q30': 89.09,
                                'pf_read2_q30': 90.37,
                                'pf_yield': 4142836,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A352_TCCACGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A353_TGTTCCGT': {'barcodes': 1333403,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48624,
                                'perfect_barcodes': 1284779,
                                'pf_Q30': 90.72,
                                'pf_clusters': 1333403,
                                'pf_qscore': 37.93,
                                'pf_read1_q30': 90.35,
                                'pf_read2_q30': 91.08,
                                'pf_yield': 402687706,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A353_TGTTCCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A354_CCACATTG': {'barcodes': 1181636,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39832,
                                'perfect_barcodes': 1141804,
                                'pf_Q30': 92.19,
                                'pf_clusters': 1181636,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.28,
                                'pf_read2_q30': 91.09,
                                'pf_yield': 356854072,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A354_CCACATTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A355_ATCCACGA': {'barcodes': 1300953,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40385,
                                'perfect_barcodes': 1260568,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1300953,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.04,
                                'pf_read2_q30': 91.0,
                                'pf_yield': 392887806,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A355_ATCCACGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A356_CATTGACG': {'barcodes': 1238138,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40392,
                                'perfect_barcodes': 1197746,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1238138,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.89,
                                'pf_read2_q30': 90.97,
                                'pf_yield': 373917676,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A356_CATTGACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A357_ATCGGAGA': {'barcodes': 1344081,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49503,
                                'perfect_barcodes': 1294578,
                                'pf_Q30': 91.72,
                                'pf_clusters': 1344081,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.16,
                                'pf_read2_q30': 90.27,
                                'pf_yield': 405912462,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A357_ATCGGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A358_TACTAGCG': {'barcodes': 1353978,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38759,
                                'perfect_barcodes': 1315219,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1353978,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.1,
                                'pf_read2_q30': 90.69,
                                'pf_yield': 408901356,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A358_TACTAGCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A359_TCGATGAC': {'barcodes': 1393123,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42819,
                                'perfect_barcodes': 1350304,
                                'pf_Q30': 92.05,
                                'pf_clusters': 1393123,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.69,
                                'pf_read2_q30': 91.41,
                                'pf_yield': 420723146,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A359_TCGATGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A360_AACCAGAG': {'barcodes': 1388019,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46466,
                                'perfect_barcodes': 1341553,
                                'pf_Q30': 91.79,
                                'pf_clusters': 1388019,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.16,
                                'pf_read2_q30': 91.42,
                                'pf_yield': 419181738,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A360_AACCAGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A361_GATACCTG': {'barcodes': 1026767,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 27672,
                                'perfect_barcodes': 999095,
                                'pf_Q30': 85.69,
                                'pf_clusters': 1026767,
                                'pf_qscore': 36.46,
                                'pf_read1_q30': 78.5,
                                'pf_read2_q30': 92.88,
                                'pf_yield': 310083634,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A361_GATACCTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A362_TCGAGAGT': {'barcodes': 1403652,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 66744,
                                'perfect_barcodes': 1336908,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1403652,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.87,
                                'pf_read2_q30': 90.92,
                                'pf_yield': 423902904,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A362_TCGAGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A363_TAACGTCG': {'barcodes': 1269609,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39831,
                                'perfect_barcodes': 1229778,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1269609,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 92.98,
                                'pf_read2_q30': 90.54,
                                'pf_yield': 383421918,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A363_TAACGTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A364_TCTGGACA': {'barcodes': 1351058,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47542,
                                'perfect_barcodes': 1303516,
                                'pf_Q30': 91.69,
                                'pf_clusters': 1351058,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 91.88,
                                'pf_read2_q30': 91.49,
                                'pf_yield': 408019516,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A364_TCTGGACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A365_AAGTCCTC': {'barcodes': 1457657,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54761,
                                'perfect_barcodes': 1402896,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1457657,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.42,
                                'pf_read2_q30': 90.44,
                                'pf_yield': 440212414,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A365_AAGTCCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A366_ACAACAGC': {'barcodes': 1253658,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 35417,
                                'perfect_barcodes': 1218241,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1253658,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.29,
                                'pf_read2_q30': 90.22,
                                'pf_yield': 378604716,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A366_ACAACAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A367_GACTTGTG': {'barcodes': 631504,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 20116,
                                'perfect_barcodes': 611388,
                                'pf_Q30': 89.14,
                                'pf_clusters': 631504,
                                'pf_qscore': 37.4,
                                'pf_read1_q30': 84.31,
                                'pf_read2_q30': 93.97,
                                'pf_yield': 190714208,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.15,
                                'sample_name': 'MIC2051A367_GACTTGTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A368_ACGAACGA': {'barcodes': 1144457,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36994,
                                'perfect_barcodes': 1107463,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1144457,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 90.89,
                                'pf_yield': 345626014,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A368_ACGAACGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A369_AGCCGTAA': {'barcodes': 1352355,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43315,
                                'perfect_barcodes': 1309040,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1352355,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.21,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 408411210,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A369_AGCCGTAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A370_GGTCGTAT': {'barcodes': 1462212,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 57710,
                                'perfect_barcodes': 1404502,
                                'pf_Q30': 91.63,
                                'pf_clusters': 1462212,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 92.57,
                                'pf_read2_q30': 90.7,
                                'pf_yield': 441588024,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A370_GGTCGTAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A371_CCTTCCAT': {'barcodes': 1350296,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47289,
                                'perfect_barcodes': 1303007,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1350296,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.29,
                                'pf_read2_q30': 90.55,
                                'pf_yield': 407789392,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A371_CCTTCCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A372_GCTACAAC': {'barcodes': 1299257,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39565,
                                'perfect_barcodes': 1259692,
                                'pf_Q30': 91.7,
                                'pf_clusters': 1299257,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.3,
                                'pf_read2_q30': 91.11,
                                'pf_yield': 392375614,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A372_GCTACAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A373_TGGATGGT': {'barcodes': 1597441,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1597441,
                                'perfect_barcodes': 0,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1597441,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.22,
                                'pf_read2_q30': 91.11,
                                'pf_yield': 482427182,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A373_TGGATGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A374_CGCAATGT': {'barcodes': 1356517,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47043,
                                'perfect_barcodes': 1309474,
                                'pf_Q30': 92.31,
                                'pf_clusters': 1356517,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.49,
                                'pf_read2_q30': 91.13,
                                'pf_yield': 409668134,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A374_CGCAATGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A375_CCGTTATG': {'barcodes': 1387469,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 52095,
                                'perfect_barcodes': 1335374,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1387469,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.76,
                                'pf_read2_q30': 91.36,
                                'pf_yield': 419015638,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A375_CCGTTATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A376_ATAGTCGG': {'barcodes': 1462700,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46918,
                                'perfect_barcodes': 1415782,
                                'pf_Q30': 91.03,
                                'pf_clusters': 1462700,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 91.15,
                                'pf_read2_q30': 90.92,
                                'pf_yield': 441735400,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A376_ATAGTCGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A377_CTCCTGAA': {'barcodes': 1236285,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39814,
                                'perfect_barcodes': 1196471,
                                'pf_Q30': 91.97,
                                'pf_clusters': 1236285,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.75,
                                'pf_read2_q30': 91.18,
                                'pf_yield': 373358070,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A377_CTCCTGAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A378_ACAGGCAT': {'barcodes': 1387225,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42127,
                                'perfect_barcodes': 1345098,
                                'pf_Q30': 91.72,
                                'pf_clusters': 1387225,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 90.0,
                                'pf_yield': 418941950,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A378_ACAGGCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A379_GATCAAGG': {'barcodes': 883152,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 26818,
                                'perfect_barcodes': 856334,
                                'pf_Q30': 81.65,
                                'pf_clusters': 883152,
                                'pf_qscore': 35.44,
                                'pf_read1_q30': 68.01,
                                'pf_read2_q30': 95.28,
                                'pf_yield': 266711904,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A379_GATCAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A380_CCGATGTA': {'barcodes': 1441747,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48803,
                                'perfect_barcodes': 1392944,
                                'pf_Q30': 91.97,
                                'pf_clusters': 1441747,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.85,
                                'pf_read2_q30': 91.08,
                                'pf_yield': 435407594,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A380_CCGATGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A381_AGGTGTTG': {'barcodes': 14634,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 7820,
                                'perfect_barcodes': 6814,
                                'pf_Q30': 86.46,
                                'pf_clusters': 14634,
                                'pf_qscore': 36.81,
                                'pf_read1_q30': 87.43,
                                'pf_read2_q30': 85.49,
                                'pf_yield': 4419468,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A381_AGGTGTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A382_CAGTGCTT': {'barcodes': 1194225,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41874,
                                'perfect_barcodes': 1152351,
                                'pf_Q30': 91.11,
                                'pf_clusters': 1194225,
                                'pf_qscore': 38.02,
                                'pf_read1_q30': 93.49,
                                'pf_read2_q30': 88.72,
                                'pf_yield': 360655950,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A382_CAGTGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A383_CAACGAGT': {'barcodes': 15746,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 5947,
                                'perfect_barcodes': 9799,
                                'pf_Q30': 86.05,
                                'pf_clusters': 15746,
                                'pf_qscore': 36.68,
                                'pf_read1_q30': 87.31,
                                'pf_read2_q30': 84.8,
                                'pf_yield': 4755292,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A383_CAACGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A384_CCATGAAC': {'barcodes': 1330313,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42419,
                                'perfect_barcodes': 1287894,
                                'pf_Q30': 92.3,
                                'pf_clusters': 1330313,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 91.51,
                                'pf_yield': 401754526,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A384_CCATGAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A385_ACGAATCC': {'barcodes': 1251506,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39335,
                                'perfect_barcodes': 1212171,
                                'pf_Q30': 92.29,
                                'pf_clusters': 1251506,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.17,
                                'pf_read2_q30': 91.42,
                                'pf_yield': 377954812,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A385_ACGAATCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A386_GTGATCCA': {'barcodes': 1316655,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50023,
                                'perfect_barcodes': 1266632,
                                'pf_Q30': 91.96,
                                'pf_clusters': 1316655,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.32,
                                'pf_read2_q30': 91.6,
                                'pf_yield': 397629810,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A386_GTGATCCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A387_AAGCATCG': {'barcodes': 1322193,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39362,
                                'perfect_barcodes': 1282831,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1322193,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 93.31,
                                'pf_read2_q30': 90.54,
                                'pf_yield': 399302286,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A387_AAGCATCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A388_TAGGAGCT': {'barcodes': 1547800,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 56071,
                                'perfect_barcodes': 1491729,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1547800,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 90.93,
                                'pf_yield': 467435600,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A388_TAGGAGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A389_GACGAACT': {'barcodes': 1096736,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 31251,
                                'perfect_barcodes': 1065485,
                                'pf_Q30': 90.79,
                                'pf_clusters': 1096736,
                                'pf_qscore': 37.9,
                                'pf_read1_q30': 91.79,
                                'pf_read2_q30': 89.78,
                                'pf_yield': 331214272,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A389_GACGAACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A390_TCTAGGAG': {'barcodes': 1581011,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 69394,
                                'perfect_barcodes': 1511617,
                                'pf_Q30': 91.75,
                                'pf_clusters': 1581011,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 92.99,
                                'pf_read2_q30': 90.51,
                                'pf_yield': 477465322,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A390_TCTAGGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A391_TTACCGAC': {'barcodes': 1255067,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34457,
                                'perfect_barcodes': 1220610,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1255067,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.65,
                                'pf_read2_q30': 91.41,
                                'pf_yield': 379030234,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A391_TTACCGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A392_GAGCAATC': {'barcodes': 1466171,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49897,
                                'perfect_barcodes': 1416274,
                                'pf_Q30': 91.49,
                                'pf_clusters': 1466171,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 91.7,
                                'pf_read2_q30': 91.27,
                                'pf_yield': 442783642,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A392_GAGCAATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A393_AATGGTCG': {'barcodes': 1289133,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46799,
                                'perfect_barcodes': 1242334,
                                'pf_Q30': 91.3,
                                'pf_clusters': 1289133,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 91.06,
                                'pf_read2_q30': 91.55,
                                'pf_yield': 389318166,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A393_AATGGTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A394_TTCGTACG': {'barcodes': 1198262,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36035,
                                'perfect_barcodes': 1162227,
                                'pf_Q30': 91.3,
                                'pf_clusters': 1198262,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 92.12,
                                'pf_read2_q30': 90.48,
                                'pf_yield': 361875124,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A394_TTCGTACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A395_AGGATAGC': {'barcodes': 1194064,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36777,
                                'perfect_barcodes': 1157287,
                                'pf_Q30': 91.56,
                                'pf_clusters': 1194064,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 93.78,
                                'pf_read2_q30': 89.35,
                                'pf_yield': 360607328,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A395_AGGATAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A396_AACAAGGC': {'barcodes': 1235711,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32475,
                                'perfect_barcodes': 1203236,
                                'pf_Q30': 91.35,
                                'pf_clusters': 1235711,
                                'pf_qscore': 38.1,
                                'pf_read1_q30': 93.38,
                                'pf_read2_q30': 89.33,
                                'pf_yield': 373184722,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A396_AACAAGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A397_GTTCTTCG': {'barcodes': 1026160,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 27381,
                                'perfect_barcodes': 998779,
                                'pf_Q30': 90.9,
                                'pf_clusters': 1026160,
                                'pf_qscore': 37.91,
                                'pf_read1_q30': 90.25,
                                'pf_read2_q30': 91.54,
                                'pf_yield': 309900320,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A397_GTTCTTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A398_GATTGTCC': {'barcodes': 1055650,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34487,
                                'perfect_barcodes': 1021163,
                                'pf_Q30': 89.31,
                                'pf_clusters': 1055650,
                                'pf_qscore': 37.44,
                                'pf_read1_q30': 85.35,
                                'pf_read2_q30': 93.28,
                                'pf_yield': 318806300,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A398_GATTGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A400_CAACTTGG': {'barcodes': 1423915,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49037,
                                'perfect_barcodes': 1374878,
                                'pf_Q30': 90.94,
                                'pf_clusters': 1423915,
                                'pf_qscore': 38.01,
                                'pf_read1_q30': 91.25,
                                'pf_read2_q30': 90.63,
                                'pf_yield': 430022330,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A400_CAACTTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A401_CGCTACAT': {'barcodes': 1819311,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 58287,
                                'perfect_barcodes': 1761024,
                                'pf_Q30': 90.24,
                                'pf_clusters': 1819311,
                                'pf_qscore': 37.81,
                                'pf_read1_q30': 90.05,
                                'pf_read2_q30': 90.43,
                                'pf_yield': 549431922,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A401_CGCTACAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A402_ATGACAGG': {'barcodes': 8682,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 3050,
                                'perfect_barcodes': 5632,
                                'pf_Q30': 85.5,
                                'pf_clusters': 8682,
                                'pf_qscore': 36.55,
                                'pf_read1_q30': 86.82,
                                'pf_read2_q30': 84.18,
                                'pf_yield': 2621964,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A402_ATGACAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A403_GGCTCAAT': {'barcodes': 1668500,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 59982,
                                'perfect_barcodes': 1608518,
                                'pf_Q30': 92.22,
                                'pf_clusters': 1668500,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 90.92,
                                'pf_yield': 503887000,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A403_GGCTCAAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A404_CTCGGTAA': {'barcodes': 1245775,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47652,
                                'perfect_barcodes': 1198123,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1245775,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.55,
                                'pf_read2_q30': 90.63,
                                'pf_yield': 376224050,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A404_CTCGGTAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A405_TTCGCCAT': {'barcodes': 1541080,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44050,
                                'perfect_barcodes': 1497030,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1541080,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 93.39,
                                'pf_read2_q30': 90.65,
                                'pf_yield': 465406160,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A405_TTCGCCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A406_GGTACGAA': {'barcodes': 1315832,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42809,
                                'perfect_barcodes': 1273023,
                                'pf_Q30': 91.81,
                                'pf_clusters': 1315832,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 90.26,
                                'pf_yield': 397381264,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A406_GGTACGAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A407_CTGTACCA': {'barcodes': 1296227,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43190,
                                'perfect_barcodes': 1253037,
                                'pf_Q30': 91.47,
                                'pf_clusters': 1296227,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 93.61,
                                'pf_read2_q30': 89.34,
                                'pf_yield': 391460554,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A407_CTGTACCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A408_CCACAACA': {'barcodes': 1547483,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53705,
                                'perfect_barcodes': 1493778,
                                'pf_Q30': 91.06,
                                'pf_clusters': 1547483,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 91.5,
                                'pf_read2_q30': 90.61,
                                'pf_yield': 467339866,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A408_CCACAACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A409_GAGCTCAA': {'barcodes': 1316550,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40712,
                                'perfect_barcodes': 1275838,
                                'pf_Q30': 90.83,
                                'pf_clusters': 1316550,
                                'pf_qscore': 37.95,
                                'pf_read1_q30': 91.68,
                                'pf_read2_q30': 89.98,
                                'pf_yield': 397598100,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A409_GAGCTCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A410_TCGAAGGT': {'barcodes': 1229591,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51544,
                                'perfect_barcodes': 1178047,
                                'pf_Q30': 91.2,
                                'pf_clusters': 1229591,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 92.8,
                                'pf_read2_q30': 89.6,
                                'pf_yield': 371336482,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A410_TCGAAGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A411_GAGACGAT': {'barcodes': 1273909,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41169,
                                'perfect_barcodes': 1232740,
                                'pf_Q30': 91.61,
                                'pf_clusters': 1273909,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 92.22,
                                'pf_read2_q30': 91.01,
                                'pf_yield': 384720518,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A411_GAGACGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A412_ACCTAAGG': {'barcodes': 1218101,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41987,
                                'perfect_barcodes': 1176114,
                                'pf_Q30': 91.32,
                                'pf_clusters': 1218101,
                                'pf_qscore': 38.09,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 89.13,
                                'pf_yield': 367866502,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A412_ACCTAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A413_ATGACCAG': {'barcodes': 1351411,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41252,
                                'perfect_barcodes': 1310159,
                                'pf_Q30': 91.42,
                                'pf_clusters': 1351411,
                                'pf_qscore': 38.12,
                                'pf_read1_q30': 93.31,
                                'pf_read2_q30': 89.53,
                                'pf_yield': 408126122,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A413_ATGACCAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A414_ACATAGGC': {'barcodes': 1297945,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44357,
                                'perfect_barcodes': 1253588,
                                'pf_Q30': 91.87,
                                'pf_clusters': 1297945,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.21,
                                'pf_read2_q30': 90.53,
                                'pf_yield': 391979390,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A414_ACATAGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A415_CTTAGGAC': {'barcodes': 2706699,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 80640,
                                'perfect_barcodes': 2626059,
                                'pf_Q30': 91.86,
                                'pf_clusters': 2706699,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.26,
                                'pf_read2_q30': 90.47,
                                'pf_yield': 817423098,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.64,
                                'sample_name': 'MIC2051A415_CTTAGGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A416_TTGCTGGA': {'barcodes': 1422708,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43629,
                                'perfect_barcodes': 1379079,
                                'pf_Q30': 91.07,
                                'pf_clusters': 1422708,
                                'pf_qscore': 38.02,
                                'pf_read1_q30': 91.8,
                                'pf_read2_q30': 90.35,
                                'pf_yield': 429657816,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A416_TTGCTGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A417_GTGGTGTT': {'barcodes': 1513666,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 77628,
                                'perfect_barcodes': 1436038,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1513666,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.69,
                                'pf_read2_q30': 91.29,
                                'pf_yield': 457127132,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A417_GTGGTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A418_TACTGCGT': {'barcodes': 1573339,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 55023,
                                'perfect_barcodes': 1518316,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1573339,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.45,
                                'pf_read2_q30': 90.78,
                                'pf_yield': 475148378,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A418_TACTGCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A419_GCACTCAT': {'barcodes': 1360424,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46154,
                                'perfect_barcodes': 1314270,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1360424,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 91.83,
                                'pf_read2_q30': 92.24,
                                'pf_yield': 410848048,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A419_GCACTCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A420_CGTATTCG': {'barcodes': 1205150,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41923,
                                'perfect_barcodes': 1163227,
                                'pf_Q30': 91.77,
                                'pf_clusters': 1205150,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 92.62,
                                'pf_read2_q30': 90.92,
                                'pf_yield': 363955300,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A420_CGTATTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A421_TACCACAG': {'barcodes': 1235300,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37952,
                                'perfect_barcodes': 1197348,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1235300,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.37,
                                'pf_read2_q30': 91.67,
                                'pf_yield': 373060600,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A421_TACCACAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A422_TGATACGC': {'barcodes': 1320922,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40381,
                                'perfect_barcodes': 1280541,
                                'pf_Q30': 91.9,
                                'pf_clusters': 1320922,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.43,
                                'pf_read2_q30': 91.37,
                                'pf_yield': 398918444,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A422_TGATACGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A423_GTAACGAC': {'barcodes': 1117406,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40143,
                                'perfect_barcodes': 1077263,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1117406,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.07,
                                'pf_read2_q30': 91.53,
                                'pf_yield': 337456612,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A423_GTAACGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A424_AGTCAGGA': {'barcodes': 1596515,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 55633,
                                'perfect_barcodes': 1540882,
                                'pf_Q30': 90.82,
                                'pf_clusters': 1596515,
                                'pf_qscore': 37.98,
                                'pf_read1_q30': 91.18,
                                'pf_read2_q30': 90.47,
                                'pf_yield': 482147530,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A424_AGTCAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A425_TCGCTGTT': {'barcodes': 1568320,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 56098,
                                'perfect_barcodes': 1512222,
                                'pf_Q30': 91.22,
                                'pf_clusters': 1568320,
                                'pf_qscore': 38.07,
                                'pf_read1_q30': 91.13,
                                'pf_read2_q30': 91.3,
                                'pf_yield': 473632640,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A425_TCGCTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A426_CGTTGAGT': {'barcodes': 1509934,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 71651,
                                'perfect_barcodes': 1438283,
                                'pf_Q30': 91.32,
                                'pf_clusters': 1509934,
                                'pf_qscore': 38.1,
                                'pf_read1_q30': 91.57,
                                'pf_read2_q30': 91.07,
                                'pf_yield': 456000068,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A426_CGTTGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A427_GTCCACAT': {'barcodes': 2897492,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 93132,
                                'perfect_barcodes': 2804360,
                                'pf_Q30': 91.96,
                                'pf_clusters': 2897492,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.53,
                                'pf_read2_q30': 91.39,
                                'pf_yield': 875042584,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.69,
                                'sample_name': 'MIC2051A427_GTCCACAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A428_TTCTCTCG': {'barcodes': 1172545,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40617,
                                'perfect_barcodes': 1131928,
                                'pf_Q30': 91.97,
                                'pf_clusters': 1172545,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.38,
                                'pf_read2_q30': 91.56,
                                'pf_yield': 354108590,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A428_TTCTCTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A429_GAAGGAAG': {'barcodes': 1232861,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44959,
                                'perfect_barcodes': 1187902,
                                'pf_Q30': 90.76,
                                'pf_clusters': 1232861,
                                'pf_qscore': 37.88,
                                'pf_read1_q30': 88.87,
                                'pf_read2_q30': 92.65,
                                'pf_yield': 372324022,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A429_GAAGGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A430_AGAGTAGC': {'barcodes': 1407417,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54118,
                                'perfect_barcodes': 1353299,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1407417,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 92.97,
                                'pf_read2_q30': 91.22,
                                'pf_yield': 425039934,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A430_AGAGTAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A431_CATACCAC': {'barcodes': 1146519,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33409,
                                'perfect_barcodes': 1113110,
                                'pf_Q30': 91.67,
                                'pf_clusters': 1146519,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 93.15,
                                'pf_read2_q30': 90.2,
                                'pf_yield': 346248738,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A431_CATACCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A432_GATAGCGA': {'barcodes': 1373523,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40083,
                                'perfect_barcodes': 1333440,
                                'pf_Q30': 88.14,
                                'pf_clusters': 1373523,
                                'pf_qscore': 37.15,
                                'pf_read1_q30': 82.3,
                                'pf_read2_q30': 93.98,
                                'pf_yield': 414803946,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A432_GATAGCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A433_GTTCGGTT': {'barcodes': 1201301,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43321,
                                'perfect_barcodes': 1157980,
                                'pf_Q30': 91.37,
                                'pf_clusters': 1201301,
                                'pf_qscore': 38.06,
                                'pf_read1_q30': 90.67,
                                'pf_read2_q30': 92.07,
                                'pf_yield': 362792902,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A433_GTTCGGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A434_AGTGCAGT': {'barcodes': 2813305,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 92370,
                                'perfect_barcodes': 2720935,
                                'pf_Q30': 92.36,
                                'pf_clusters': 2813305,
                                'pf_qscore': 38.4,
                                'pf_read1_q30': 93.44,
                                'pf_read2_q30': 91.27,
                                'pf_yield': 849618110,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.67,
                                'sample_name': 'MIC2051A434_AGTGCAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A435_GTCAGTTG': {'barcodes': 942271,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 35892,
                                'perfect_barcodes': 906379,
                                'pf_Q30': 92.05,
                                'pf_clusters': 942271,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.4,
                                'pf_read2_q30': 91.69,
                                'pf_yield': 284565842,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A435_GTCAGTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A436_ATAAGGCG': {'barcodes': 1305586,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39047,
                                'perfect_barcodes': 1266539,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1305586,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.27,
                                'pf_read2_q30': 90.78,
                                'pf_yield': 394286972,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A436_ATAAGGCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A437_GAAGGTTC': {'barcodes': 1324677,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49626,
                                'perfect_barcodes': 1275051,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1324677,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.12,
                                'pf_read2_q30': 91.9,
                                'pf_yield': 400052454,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A437_GAAGGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A438_TGTGAAGC': {'barcodes': 1386147,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 45424,
                                'perfect_barcodes': 1340723,
                                'pf_Q30': 92.15,
                                'pf_clusters': 1386147,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.06,
                                'pf_read2_q30': 91.23,
                                'pf_yield': 418616394,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A438_TGTGAAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A439_CGCTTAAC': {'barcodes': 1422670,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43903,
                                'perfect_barcodes': 1378767,
                                'pf_Q30': 91.91,
                                'pf_clusters': 1422670,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.25,
                                'pf_read2_q30': 91.58,
                                'pf_yield': 429646340,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A439_CGCTTAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A440_GGACTAGA': {'barcodes': 1553018,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51513,
                                'perfect_barcodes': 1501505,
                                'pf_Q30': 91.53,
                                'pf_clusters': 1553018,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 91.78,
                                'pf_read2_q30': 91.28,
                                'pf_yield': 469011436,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A440_GGACTAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A441_GGCAAGTT': {'barcodes': 1450203,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 52771,
                                'perfect_barcodes': 1397432,
                                'pf_Q30': 91.69,
                                'pf_clusters': 1450203,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 94.09,
                                'pf_read2_q30': 89.29,
                                'pf_yield': 437961306,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A441_GGCAAGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A442_AGCCAAGT': {'barcodes': 1811173,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 58464,
                                'perfect_barcodes': 1752709,
                                'pf_Q30': 92.08,
                                'pf_clusters': 1811173,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.57,
                                'pf_read2_q30': 90.58,
                                'pf_yield': 546974246,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A442_AGCCAAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A443_GGCTATTG': {'barcodes': 1421762,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51027,
                                'perfect_barcodes': 1370735,
                                'pf_Q30': 91.7,
                                'pf_clusters': 1421762,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 93.63,
                                'pf_read2_q30': 89.78,
                                'pf_yield': 429372124,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A443_GGCTATTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A444_TTGATCCG': {'barcodes': 1151325,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 30718,
                                'perfect_barcodes': 1120607,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1151325,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 91.39,
                                'pf_read2_q30': 92.12,
                                'pf_yield': 347700150,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A444_TTGATCCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A445_CAGTCTTC': {'barcodes': 1277683,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41858,
                                'perfect_barcodes': 1235825,
                                'pf_Q30': 92.22,
                                'pf_clusters': 1277683,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.27,
                                'pf_read2_q30': 91.17,
                                'pf_yield': 385860266,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A445_CAGTCTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A446_ATCTGTCC': {'barcodes': 1433162,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47884,
                                'perfect_barcodes': 1385278,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1433162,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.66,
                                'pf_read2_q30': 91.35,
                                'pf_yield': 432814924,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A446_ATCTGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A447_ACAGCAAC': {'barcodes': 3046943,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1536721,
                                'perfect_barcodes': 1510222,
                                'pf_Q30': 92.03,
                                'pf_clusters': 3046943,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 91.27,
                                'pf_yield': 920176786,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.73,
                                'sample_name': 'MIC2051A447_ACAGCAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A448_GGTTGTCA': {'barcodes': 1463197,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 52216,
                                'perfect_barcodes': 1410981,
                                'pf_Q30': 91.49,
                                'pf_clusters': 1463197,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 91.16,
                                'pf_read2_q30': 91.83,
                                'pf_yield': 441885494,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A448_GGTTGTCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A449_TCAGGCTT': {'barcodes': 13555,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 2920,
                                'perfect_barcodes': 10635,
                                'pf_Q30': 89.74,
                                'pf_clusters': 13555,
                                'pf_qscore': 37.64,
                                'pf_read1_q30': 89.74,
                                'pf_read2_q30': 89.74,
                                'pf_yield': 4093610,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A449_TCAGGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A450_TGGACTCT': {'barcodes': 1521382,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53010,
                                'perfect_barcodes': 1468372,
                                'pf_Q30': 92.22,
                                'pf_clusters': 1521382,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 92.87,
                                'pf_read2_q30': 91.57,
                                'pf_yield': 459457364,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A450_TGGACTCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A451_TCCGTATG': {'barcodes': 1169889,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38251,
                                'perfect_barcodes': 1131638,
                                'pf_Q30': 91.84,
                                'pf_clusters': 1169889,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 91.83,
                                'pf_read2_q30': 91.86,
                                'pf_yield': 353306478,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A451_TCCGTATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A452_GAGATACG': {'barcodes': 1275735,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42235,
                                'perfect_barcodes': 1233500,
                                'pf_Q30': 91.79,
                                'pf_clusters': 1275735,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.02,
                                'pf_read2_q30': 91.55,
                                'pf_yield': 385271970,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A452_GAGATACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A453_GAGATGTC': {'barcodes': 1235176,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 45959,
                                'perfect_barcodes': 1189217,
                                'pf_Q30': 91.76,
                                'pf_clusters': 1235176,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 92.15,
                                'pf_read2_q30': 91.36,
                                'pf_yield': 373023152,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A453_GAGATGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A454_AAGACTCC': {'barcodes': 1578290,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46713,
                                'perfect_barcodes': 1531577,
                                'pf_Q30': 92.33,
                                'pf_clusters': 1578290,
                                'pf_qscore': 38.39,
                                'pf_read1_q30': 93.22,
                                'pf_read2_q30': 91.43,
                                'pf_yield': 476643580,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A454_AAGACTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A455_GCCACTTA': {'barcodes': 1368623,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49169,
                                'perfect_barcodes': 1319454,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1368623,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.03,
                                'pf_read2_q30': 91.32,
                                'pf_yield': 413324146,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A455_GCCACTTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A456_GCCTATCA': {'barcodes': 1354943,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43922,
                                'perfect_barcodes': 1311021,
                                'pf_Q30': 91.64,
                                'pf_clusters': 1354943,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 90.71,
                                'pf_read2_q30': 92.57,
                                'pf_yield': 409192786,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A456_GCCTATCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A457_GCTGGATT': {'barcodes': 1350729,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51650,
                                'perfect_barcodes': 1299079,
                                'pf_Q30': 91.82,
                                'pf_clusters': 1350729,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 91.27,
                                'pf_read2_q30': 92.37,
                                'pf_yield': 407920158,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A457_GCTGGATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A458_GTGTTCCT': {'barcodes': 1120738,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37701,
                                'perfect_barcodes': 1083037,
                                'pf_Q30': 91.32,
                                'pf_clusters': 1120738,
                                'pf_qscore': 38.07,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 89.54,
                                'pf_yield': 338462876,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A458_GTGTTCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A459_CTTGGATG': {'barcodes': 1279925,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46601,
                                'perfect_barcodes': 1233324,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1279925,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 93.5,
                                'pf_read2_q30': 89.71,
                                'pf_yield': 386537350,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A459_CTTGGATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A460_CAGGTTAG': {'barcodes': 1392986,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 56707,
                                'perfect_barcodes': 1336279,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1392986,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 93.43,
                                'pf_read2_q30': 90.55,
                                'pf_yield': 420681772,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A460_CAGGTTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A461_TCGTAGTC': {'barcodes': 1332680,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44977,
                                'perfect_barcodes': 1287703,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1332680,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.33,
                                'pf_read2_q30': 90.71,
                                'pf_yield': 402469360,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A461_TCGTAGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A462_AACTTGCC': {'barcodes': 1270265,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37482,
                                'perfect_barcodes': 1232783,
                                'pf_Q30': 91.94,
                                'pf_clusters': 1270265,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.29,
                                'pf_read2_q30': 90.59,
                                'pf_yield': 383620030,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A462_AACTTGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A463_CTGGAGTA': {'barcodes': 1516521,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 70071,
                                'perfect_barcodes': 1446450,
                                'pf_Q30': 92.13,
                                'pf_clusters': 1516521,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.08,
                                'pf_read2_q30': 91.19,
                                'pf_yield': 457989342,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A463_CTGGAGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A464_CTATCGCA': {'barcodes': 1873994,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 56389,
                                'perfect_barcodes': 1817605,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1873994,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 90.93,
                                'pf_yield': 565946188,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.45,
                                'sample_name': 'MIC2051A464_CTATCGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A465_CACGTTGT': {'barcodes': 1663468,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 67144,
                                'perfect_barcodes': 1596324,
                                'pf_Q30': 89.54,
                                'pf_clusters': 1663468,
                                'pf_qscore': 37.61,
                                'pf_read1_q30': 89.37,
                                'pf_read2_q30': 89.71,
                                'pf_yield': 502367336,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A465_CACGTTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A466_TTCAGCCT': {'barcodes': 1367887,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42885,
                                'perfect_barcodes': 1325002,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1367887,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.28,
                                'pf_read2_q30': 91.8,
                                'pf_yield': 413101874,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A466_TTCAGCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A467_AGTGTTGG': {'barcodes': 1320567,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 59668,
                                'perfect_barcodes': 1260899,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1320567,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.05,
                                'pf_read2_q30': 91.16,
                                'pf_yield': 398811234,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A467_AGTGTTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A468_TGAGCTAG': {'barcodes': 1397100,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 45854,
                                'perfect_barcodes': 1351246,
                                'pf_Q30': 91.14,
                                'pf_clusters': 1397100,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 93.13,
                                'pf_read2_q30': 89.15,
                                'pf_yield': 421924200,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A468_TGAGCTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A469_TAGAGCTC': {'barcodes': 1248141,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41162,
                                'perfect_barcodes': 1206979,
                                'pf_Q30': 91.57,
                                'pf_clusters': 1248141,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 93.33,
                                'pf_read2_q30': 89.8,
                                'pf_yield': 376938582,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A469_TAGAGCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A470_ACGTTACC': {'barcodes': 1282794,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38996,
                                'perfect_barcodes': 1243798,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1282794,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.98,
                                'pf_read2_q30': 91.08,
                                'pf_yield': 387403788,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A470_ACGTTACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A471_CATGGCTA': {'barcodes': 1384134,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41491,
                                'perfect_barcodes': 1342643,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1384134,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.02,
                                'pf_read2_q30': 91.05,
                                'pf_yield': 418008468,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A471_CATGGCTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A472_TGCTTCCA': {'barcodes': 2222956,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 73269,
                                'perfect_barcodes': 2149687,
                                'pf_Q30': 91.93,
                                'pf_clusters': 2222956,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.46,
                                'pf_read2_q30': 91.41,
                                'pf_yield': 671332712,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.53,
                                'sample_name': 'MIC2051A472_TGCTTCCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A473_ATCGGTGT': {'barcodes': 1640451,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 66150,
                                'perfect_barcodes': 1574301,
                                'pf_Q30': 91.24,
                                'pf_clusters': 1640451,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 91.62,
                                'pf_read2_q30': 90.87,
                                'pf_yield': 495416202,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A473_ATCGGTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A474_CGTGTACT': {'barcodes': 1248489,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43748,
                                'perfect_barcodes': 1204741,
                                'pf_Q30': 91.34,
                                'pf_clusters': 1248489,
                                'pf_qscore': 38.1,
                                'pf_read1_q30': 93.61,
                                'pf_read2_q30': 89.06,
                                'pf_yield': 377043678,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A474_CGTGTACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A475_GATACTGG': {'barcodes': 1067186,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37700,
                                'perfect_barcodes': 1029486,
                                'pf_Q30': 87.83,
                                'pf_clusters': 1067186,
                                'pf_qscore': 37.05,
                                'pf_read1_q30': 81.0,
                                'pf_read2_q30': 94.67,
                                'pf_yield': 322290172,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A475_GATACTGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A476_AGCTTGAG': {'barcodes': 1494577,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 110215,
                                'perfect_barcodes': 1384362,
                                'pf_Q30': 92.33,
                                'pf_clusters': 1494577,
                                'pf_qscore': 38.39,
                                'pf_read1_q30': 93.23,
                                'pf_read2_q30': 91.43,
                                'pf_yield': 451362254,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A476_AGCTTGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A477_TGGCTATC': {'barcodes': 951306,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 30048,
                                'perfect_barcodes': 921258,
                                'pf_Q30': 92.18,
                                'pf_clusters': 951306,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.17,
                                'pf_read2_q30': 91.19,
                                'pf_yield': 287294412,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A477_TGGCTATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A478_TCATCACC': {'barcodes': 1315551,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38583,
                                'perfect_barcodes': 1276968,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1315551,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 91.05,
                                'pf_read2_q30': 92.43,
                                'pf_yield': 397296402,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A478_TCATCACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A479_GCTGACTA': {'barcodes': 1158733,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32308,
                                'perfect_barcodes': 1126425,
                                'pf_Q30': 91.27,
                                'pf_clusters': 1158733,
                                'pf_qscore': 38.04,
                                'pf_read1_q30': 92.21,
                                'pf_read2_q30': 90.34,
                                'pf_yield': 349937366,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A479_GCTGACTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A480_GTGTGACA': {'barcodes': 1395050,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51694,
                                'perfect_barcodes': 1343356,
                                'pf_Q30': 91.83,
                                'pf_clusters': 1395050,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.13,
                                'pf_read2_q30': 91.53,
                                'pf_yield': 421305100,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A480_GTGTGACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A481_GTCACTGT': {'barcodes': 1483402,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53003,
                                'perfect_barcodes': 1430399,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1483402,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.31,
                                'pf_read2_q30': 91.55,
                                'pf_yield': 447987404,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A481_GTCACTGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A482_AGGTCACT': {'barcodes': 1295806,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44493,
                                'perfect_barcodes': 1251313,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1295806,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.72,
                                'pf_read2_q30': 90.29,
                                'pf_yield': 391333412,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A482_AGGTCACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A483_TTAAGCGG': {'barcodes': 1393860,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41735,
                                'perfect_barcodes': 1352125,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1393860,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.49,
                                'pf_read2_q30': 91.56,
                                'pf_yield': 420945720,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A483_TTAAGCGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A484_TGTTCGAG': {'barcodes': 1386028,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53375,
                                'perfect_barcodes': 1332653,
                                'pf_Q30': 92.47,
                                'pf_clusters': 1386028,
                                'pf_qscore': 38.41,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 92.14,
                                'pf_yield': 418580456,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A484_TGTTCGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A485_GCAAGATC': {'barcodes': 1415809,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48864,
                                'perfect_barcodes': 1366945,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1415809,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.39,
                                'pf_read2_q30': 91.95,
                                'pf_yield': 427574318,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A485_GCAAGATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A486_TACCAACC': {'barcodes': 1414235,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42143,
                                'perfect_barcodes': 1372092,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1414235,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.47,
                                'pf_read2_q30': 91.8,
                                'pf_yield': 427098970,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A486_TACCAACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A487_CCAGGATA': {'barcodes': 1550809,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51184,
                                'perfect_barcodes': 1499625,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1550809,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.04,
                                'pf_read2_q30': 91.37,
                                'pf_yield': 468344318,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A487_CCAGGATA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A488_GCACGTAA': {'barcodes': 1487706,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48567,
                                'perfect_barcodes': 1439139,
                                'pf_Q30': 91.46,
                                'pf_clusters': 1487706,
                                'pf_qscore': 38.11,
                                'pf_read1_q30': 90.84,
                                'pf_read2_q30': 92.08,
                                'pf_yield': 449287212,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A488_GCACGTAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A489_AGGATGGT': {'barcodes': 1348215,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1348215,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.67,
                                'pf_clusters': 1348215,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 90.55,
                                'pf_yield': 407160930,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A489_AGGATGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A490_GCACAACT': {'barcodes': 1438972,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 45050,
                                'perfect_barcodes': 1393922,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1438972,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.85,
                                'pf_read2_q30': 91.38,
                                'pf_yield': 434569544,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A490_GCACAACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A491_CTGTTAGG': {'barcodes': 1500925,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54455,
                                'perfect_barcodes': 1446470,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1500925,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.86,
                                'pf_read2_q30': 91.18,
                                'pf_yield': 453279350,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A491_CTGTTAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A492_CGATAGAG': {'barcodes': 1352078,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54392,
                                'perfect_barcodes': 1297686,
                                'pf_Q30': 91.89,
                                'pf_clusters': 1352078,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.63,
                                'pf_read2_q30': 91.14,
                                'pf_yield': 408327556,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A492_CGATAGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A493_ATGGTTGC': {'barcodes': 1404365,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44235,
                                'perfect_barcodes': 1360130,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1404365,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.27,
                                'pf_read2_q30': 90.91,
                                'pf_yield': 424118230,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A493_ATGGTTGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A494_AGGTGTAC': {'barcodes': 1352960,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49855,
                                'perfect_barcodes': 1303105,
                                'pf_Q30': 92.23,
                                'pf_clusters': 1352960,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 93.49,
                                'pf_read2_q30': 90.96,
                                'pf_yield': 408593920,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A494_AGGTGTAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A495_CGGATTGA': {'barcodes': 1281941,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41398,
                                'perfect_barcodes': 1240543,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1281941,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.01,
                                'pf_read2_q30': 90.7,
                                'pf_yield': 387146182,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A495_CGGATTGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A496_AGGAGGAA': {'barcodes': 1834259,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 76591,
                                'perfect_barcodes': 1757668,
                                'pf_Q30': 89.73,
                                'pf_clusters': 1834259,
                                'pf_qscore': 37.67,
                                'pf_read1_q30': 89.85,
                                'pf_read2_q30': 89.61,
                                'pf_yield': 553946218,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.44,
                                'sample_name': 'MIC2051A496_AGGAGGAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A497_AACAACCG': {'barcodes': 17690,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 4110,
                                'perfect_barcodes': 13580,
                                'pf_Q30': 86.05,
                                'pf_clusters': 17690,
                                'pf_qscore': 36.71,
                                'pf_read1_q30': 87.45,
                                'pf_read2_q30': 84.65,
                                'pf_yield': 5342380,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A497_AACAACCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A498_ACGGACTT': {'barcodes': 1460917,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47555,
                                'perfect_barcodes': 1413362,
                                'pf_Q30': 91.11,
                                'pf_clusters': 1460917,
                                'pf_qscore': 38.04,
                                'pf_read1_q30': 93.57,
                                'pf_read2_q30': 88.65,
                                'pf_yield': 441196934,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A498_ACGGACTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A499_ATCTCCTG': {'barcodes': 1145282,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39591,
                                'perfect_barcodes': 1105691,
                                'pf_Q30': 92.26,
                                'pf_clusters': 1145282,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.7,
                                'pf_read2_q30': 91.81,
                                'pf_yield': 345875164,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A499_ATCTCCTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A500_CGAACAAC': {'barcodes': 1183788,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38647,
                                'perfect_barcodes': 1145141,
                                'pf_Q30': 91.52,
                                'pf_clusters': 1183788,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 92.84,
                                'pf_read2_q30': 90.2,
                                'pf_yield': 357503976,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A500_CGAACAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A501_CTGTATGC': {'barcodes': 2537372,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 76381,
                                'perfect_barcodes': 2460991,
                                'pf_Q30': 91.89,
                                'pf_clusters': 2537372,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.27,
                                'pf_read2_q30': 90.51,
                                'pf_yield': 766286344,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.6,
                                'sample_name': 'MIC2051A501_CTGTATGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A502_GCCTTCTT': {'barcodes': 1223914,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44841,
                                'perfect_barcodes': 1179073,
                                'pf_Q30': 92.14,
                                'pf_clusters': 1223914,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.3,
                                'pf_read2_q30': 91.97,
                                'pf_yield': 369622028,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A502_GCCTTCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A503_GTCCTTGA': {'barcodes': 1235259,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43503,
                                'perfect_barcodes': 1191756,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1235259,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 91.85,
                                'pf_read2_q30': 92.16,
                                'pf_yield': 373048218,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A503_GTCCTTGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A504_TCCGATCA': {'barcodes': 1291424,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39790,
                                'perfect_barcodes': 1251634,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1291424,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 91.52,
                                'pf_read2_q30': 92.62,
                                'pf_yield': 390010048,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A504_TCCGATCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A505_AACAGTCC': {'barcodes': 1730846,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51326,
                                'perfect_barcodes': 1679520,
                                'pf_Q30': 92.3,
                                'pf_clusters': 1730846,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 92.93,
                                'pf_read2_q30': 91.67,
                                'pf_yield': 522715492,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A505_AACAGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A506_ACTCGATC': {'barcodes': 1671619,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53855,
                                'perfect_barcodes': 1617764,
                                'pf_Q30': 92.08,
                                'pf_clusters': 1671619,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.15,
                                'pf_read2_q30': 92.01,
                                'pf_yield': 504828938,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A506_ACTCGATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A507_ATGCGCTT': {'barcodes': 2028715,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 66420,
                                'perfect_barcodes': 1962295,
                                'pf_Q30': 92.16,
                                'pf_clusters': 2028715,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.68,
                                'pf_read2_q30': 91.63,
                                'pf_yield': 612671930,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.48,
                                'sample_name': 'MIC2051A507_ATGCGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A508_CGACACTT': {'barcodes': 1716960,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 55866,
                                'perfect_barcodes': 1661094,
                                'pf_Q30': 92.12,
                                'pf_clusters': 1716960,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 91.44,
                                'pf_yield': 518521920,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A508_CGACACTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A509_CTTAGGAC': {'barcodes': 2706699,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 80640,
                                'perfect_barcodes': 2626059,
                                'pf_Q30': 91.86,
                                'pf_clusters': 2706699,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.26,
                                'pf_read2_q30': 90.47,
                                'pf_yield': 817423098,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.64,
                                'sample_name': 'MIC2051A509_CTTAGGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A510_GCTAAGGA': {'barcodes': 1443896,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48709,
                                'perfect_barcodes': 1395187,
                                'pf_Q30': 91.5,
                                'pf_clusters': 1443896,
                                'pf_qscore': 38.11,
                                'pf_read1_q30': 90.18,
                                'pf_read2_q30': 92.83,
                                'pf_yield': 436056592,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A510_GCTAAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A511_GTCTGAGT': {'barcodes': 1578590,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 66180,
                                'perfect_barcodes': 1512410,
                                'pf_Q30': 91.73,
                                'pf_clusters': 1578590,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 90.62,
                                'pf_read2_q30': 92.85,
                                'pf_yield': 476734180,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A511_GTCTGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A512_TCGAACCT': {'barcodes': 1750606,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 56182,
                                'perfect_barcodes': 1694424,
                                'pf_Q30': 92.26,
                                'pf_clusters': 1750606,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.42,
                                'pf_read2_q30': 92.1,
                                'pf_yield': 528683012,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.42,
                                'sample_name': 'MIC2051A512_TCGAACCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A513_AACGCACA': {'barcodes': 1417192,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38197,
                                'perfect_barcodes': 1378995,
                                'pf_Q30': 92.2,
                                'pf_clusters': 1417192,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.35,
                                'pf_read2_q30': 91.06,
                                'pf_yield': 427991984,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A513_AACGCACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A514_ACTGCTTG': {'barcodes': 1462390,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50244,
                                'perfect_barcodes': 1412146,
                                'pf_Q30': 92.17,
                                'pf_clusters': 1462390,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.85,
                                'pf_read2_q30': 91.48,
                                'pf_yield': 441641780,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A514_ACTGCTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A515_ATTCCGCT': {'barcodes': 1603398,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49559,
                                'perfect_barcodes': 1553839,
                                'pf_Q30': 92.3,
                                'pf_clusters': 1603398,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 92.94,
                                'pf_read2_q30': 91.67,
                                'pf_yield': 484226196,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A515_ATTCCGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A516_CGATCGAT': {'barcodes': 1335209,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38530,
                                'perfect_barcodes': 1296679,
                                'pf_Q30': 91.1,
                                'pf_clusters': 1335209,
                                'pf_qscore': 37.96,
                                'pf_read1_q30': 89.0,
                                'pf_read2_q30': 93.19,
                                'pf_yield': 403233118,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A516_CGATCGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A517_CTTGCTAG': {'barcodes': 1435430,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46534,
                                'perfect_barcodes': 1388896,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1435430,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 93.43,
                                'pf_read2_q30': 90.99,
                                'pf_yield': 433499860,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A517_CTTGCTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A518_GCTGAATC': {'barcodes': 1430460,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50062,
                                'perfect_barcodes': 1380398,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1430460,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.17,
                                'pf_read2_q30': 92.05,
                                'pf_yield': 431998920,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A518_GCTGAATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A519_GTGTGTTC': {'barcodes': 1432853,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 59733,
                                'perfect_barcodes': 1373120,
                                'pf_Q30': 92.26,
                                'pf_clusters': 1432853,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.39,
                                'pf_read2_q30': 92.13,
                                'pf_yield': 432721606,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A519_GTGTGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A520_TCGTCTGA': {'barcodes': 1415495,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46604,
                                'perfect_barcodes': 1368891,
                                'pf_Q30': 91.61,
                                'pf_clusters': 1415495,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 92.74,
                                'pf_read2_q30': 90.49,
                                'pf_yield': 427479490,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A520_TCGTCTGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A521_AAGACCGT': {'barcodes': 1625790,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51640,
                                'perfect_barcodes': 1574150,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1625790,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.5,
                                'pf_read2_q30': 91.62,
                                'pf_yield': 490988580,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A521_AAGACCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A522_AGAAGCCT': {'barcodes': 47503,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47503,
                                'perfect_barcodes': 0,
                                'pf_Q30': 81.56,
                                'pf_clusters': 47503,
                                'pf_qscore': 35.39,
                                'pf_read1_q30': 83.2,
                                'pf_read2_q30': 79.92,
                                'pf_yield': 14345906,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A522_AGAAGCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A523_CAAGCCAA': {'barcodes': 1322340,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42552,
                                'perfect_barcodes': 1279788,
                                'pf_Q30': 92.28,
                                'pf_clusters': 1322340,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.21,
                                'pf_read2_q30': 91.34,
                                'pf_yield': 399346680,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A523_CAAGCCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A524_CGGAGTAT': {'barcodes': 1717703,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 82971,
                                'perfect_barcodes': 1634732,
                                'pf_Q30': 92.13,
                                'pf_clusters': 1717703,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 92.64,
                                'pf_read2_q30': 91.63,
                                'pf_yield': 518746306,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A524_CGGAGTAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A525_GAACGGTT': {'barcodes': 1455671,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51391,
                                'perfect_barcodes': 1404280,
                                'pf_Q30': 90.9,
                                'pf_clusters': 1455671,
                                'pf_qscore': 37.92,
                                'pf_read1_q30': 88.23,
                                'pf_read2_q30': 93.58,
                                'pf_yield': 439612642,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A525_GAACGGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A526_GGAAGAGA': {'barcodes': 1499534,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 60966,
                                'perfect_barcodes': 1438568,
                                'pf_Q30': 91.56,
                                'pf_clusters': 1499534,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 91.38,
                                'pf_read2_q30': 91.74,
                                'pf_yield': 452859268,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A526_GGAAGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A527_GTTCCATG': {'barcodes': 1254272,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41550,
                                'perfect_barcodes': 1212722,
                                'pf_Q30': 91.27,
                                'pf_clusters': 1254272,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 89.37,
                                'pf_read2_q30': 93.17,
                                'pf_yield': 378790144,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A527_GTTCCATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A528_TCTTACGG': {'barcodes': 1627637,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53455,
                                'perfect_barcodes': 1574182,
                                'pf_Q30': 91.59,
                                'pf_clusters': 1627637,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 91.47,
                                'pf_read2_q30': 91.72,
                                'pf_yield': 491546374,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A528_TCTTACGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A529_AAGCGTTC': {'barcodes': 1656055,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54390,
                                'perfect_barcodes': 1601665,
                                'pf_Q30': 92.3,
                                'pf_clusters': 1656055,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 92.98,
                                'pf_read2_q30': 91.63,
                                'pf_yield': 500128610,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A529_AAGCGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A530_AGATCGTC': {'barcodes': 1685984,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51158,
                                'perfect_barcodes': 1634826,
                                'pf_Q30': 92.27,
                                'pf_clusters': 1685984,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.02,
                                'pf_read2_q30': 91.52,
                                'pf_yield': 509167168,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A530_AGATCGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A531_CAATGCGA': {'barcodes': 1590521,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 45886,
                                'perfect_barcodes': 1544635,
                                'pf_Q30': 91.94,
                                'pf_clusters': 1590521,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.58,
                                'pf_read2_q30': 91.29,
                                'pf_yield': 480337342,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A531_CAATGCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A532_CGGTAATC': {'barcodes': 1654137,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 56772,
                                'perfect_barcodes': 1597365,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1654137,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.66,
                                'pf_read2_q30': 91.4,
                                'pf_yield': 499549374,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A532_CGGTAATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A533_GAAGTGCT': {'barcodes': 1168615,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38646,
                                'perfect_barcodes': 1129969,
                                'pf_Q30': 91.26,
                                'pf_clusters': 1168615,
                                'pf_qscore': 38.02,
                                'pf_read1_q30': 89.02,
                                'pf_read2_q30': 93.5,
                                'pf_yield': 352921730,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A533_GAAGTGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A534_GGACTACT': {'barcodes': 1577553,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49022,
                                'perfect_barcodes': 1528531,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1577553,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 93.04,
                                'pf_read2_q30': 91.18,
                                'pf_yield': 476421006,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A534_GGACTACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A535_TAAGCGCA': {'barcodes': 1567537,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47468,
                                'perfect_barcodes': 1520069,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1567537,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.53,
                                'pf_read2_q30': 91.56,
                                'pf_yield': 473396174,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A535_TAAGCGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A536_TGAGACGA': {'barcodes': 1755488,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 57502,
                                'perfect_barcodes': 1697986,
                                'pf_Q30': 91.98,
                                'pf_clusters': 1755488,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.15,
                                'pf_read2_q30': 91.82,
                                'pf_yield': 530157376,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.42,
                                'sample_name': 'MIC2051A536_TGAGACGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A537_AATGACGC': {'barcodes': 1465945,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42242,
                                'perfect_barcodes': 1423703,
                                'pf_Q30': 91.46,
                                'pf_clusters': 1465945,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 91.53,
                                'pf_read2_q30': 91.4,
                                'pf_yield': 442715390,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A537_AATGACGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A538_AGCGTGTA': {'barcodes': 1224751,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1224751,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.59,
                                'pf_clusters': 1224751,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 92.49,
                                'pf_read2_q30': 90.68,
                                'pf_yield': 369874802,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A538_AGCGTGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A539_CATCAACC': {'barcodes': 1551895,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47750,
                                'perfect_barcodes': 1504145,
                                'pf_Q30': 92.15,
                                'pf_clusters': 1551895,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 92.34,
                                'pf_read2_q30': 91.97,
                                'pf_yield': 468672290,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A539_CATCAACC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A540_CGTCCATT': {'barcodes': 1895935,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 70392,
                                'perfect_barcodes': 1825543,
                                'pf_Q30': 91.43,
                                'pf_clusters': 1895935,
                                'pf_qscore': 38.13,
                                'pf_read1_q30': 91.61,
                                'pf_read2_q30': 91.26,
                                'pf_yield': 572572370,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.45,
                                'sample_name': 'MIC2051A540_CGTCCATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A541_GACGTCAT': {'barcodes': 1793169,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 65133,
                                'perfect_barcodes': 1728036,
                                'pf_Q30': 91.83,
                                'pf_clusters': 1793169,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.61,
                                'pf_read2_q30': 91.06,
                                'pf_yield': 541537038,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A541_GACGTCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A542_GGATGTAG': {'barcodes': 1645116,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 62377,
                                'perfect_barcodes': 1582739,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1645116,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 91.11,
                                'pf_read2_q30': 92.37,
                                'pf_yield': 496825032,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A542_GGATGTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A543_TACAGAGC': {'barcodes': 14390,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 2511,
                                'perfect_barcodes': 11879,
                                'pf_Q30': 89.35,
                                'pf_clusters': 14390,
                                'pf_qscore': 37.57,
                                'pf_read1_q30': 90.16,
                                'pf_read2_q30': 88.54,
                                'pf_yield': 4345780,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A543_TACAGAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A544_TGCACTTG': {'barcodes': 1646424,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54665,
                                'perfect_barcodes': 1591759,
                                'pf_Q30': 92.07,
                                'pf_clusters': 1646424,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.3,
                                'pf_read2_q30': 91.84,
                                'pf_yield': 497220048,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A544_TGCACTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A545_ACAAGACG': {'barcodes': 1638585,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50061,
                                'perfect_barcodes': 1588524,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1638585,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.58,
                                'pf_read2_q30': 91.48,
                                'pf_yield': 494852670,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A545_ACAAGACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A546_AGGCAATG': {'barcodes': 1550632,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 52438,
                                'perfect_barcodes': 1498194,
                                'pf_Q30': 92.25,
                                'pf_clusters': 1550632,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.45,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 468290864,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.37,
                                'sample_name': 'MIC2051A546_AGGCAATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A547_CATGAGCA': {'barcodes': 1382428,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44105,
                                'perfect_barcodes': 1338323,
                                'pf_Q30': 92.25,
                                'pf_clusters': 1382428,
                                'pf_qscore': 38.37,
                                'pf_read1_q30': 93.23,
                                'pf_read2_q30': 91.28,
                                'pf_yield': 417493256,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A547_CATGAGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A548_CTACAAGG': {'barcodes': 1413355,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43769,
                                'perfect_barcodes': 1369586,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1413355,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.26,
                                'pf_read2_q30': 90.8,
                                'pf_yield': 426833210,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A548_CTACAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A549_GAGACCAA': {'barcodes': 1451529,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 51823,
                                'perfect_barcodes': 1399706,
                                'pf_Q30': 91.56,
                                'pf_clusters': 1451529,
                                'pf_qscore': 38.13,
                                'pf_read1_q30': 90.5,
                                'pf_read2_q30': 92.62,
                                'pf_yield': 438361758,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A549_GAGACCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A550_GGCGAATA': {'barcodes': 2674302,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 86164,
                                'perfect_barcodes': 2588138,
                                'pf_Q30': 91.77,
                                'pf_clusters': 2674302,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 93.63,
                                'pf_read2_q30': 89.91,
                                'pf_yield': 807639204,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.64,
                                'sample_name': 'MIC2051A550_GGCGAATA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A551_TAGCAGGA': {'barcodes': 1068127,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1068127,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.51,
                                'pf_clusters': 1068127,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 91.84,
                                'pf_read2_q30': 91.17,
                                'pf_yield': 322574354,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A551_TAGCAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A552_TGCTCTAC': {'barcodes': 15431,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 2133,
                                'perfect_barcodes': 13298,
                                'pf_Q30': 90.77,
                                'pf_clusters': 15431,
                                'pf_qscore': 37.93,
                                'pf_read1_q30': 90.96,
                                'pf_read2_q30': 90.59,
                                'pf_yield': 4660162,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A552_TGCTCTAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A553_ACACGAGA': {'barcodes': 1411276,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47863,
                                'perfect_barcodes': 1363413,
                                'pf_Q30': 91.47,
                                'pf_clusters': 1411276,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 92.05,
                                'pf_read2_q30': 90.88,
                                'pf_yield': 426205352,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A553_ACACGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A554_AGGTTCCT': {'barcodes': 1748433,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 73501,
                                'perfect_barcodes': 1674932,
                                'pf_Q30': 91.99,
                                'pf_clusters': 1748433,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.57,
                                'pf_read2_q30': 91.41,
                                'pf_yield': 528026766,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.42,
                                'sample_name': 'MIC2051A554_AGGTTCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A555_CCAACTTC': {'barcodes': 1518177,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48319,
                                'perfect_barcodes': 1469858,
                                'pf_Q30': 92.0,
                                'pf_clusters': 1518177,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.28,
                                'pf_read2_q30': 91.71,
                                'pf_yield': 458489454,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.36,
                                'sample_name': 'MIC2051A555_CCAACTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A556_CTATCCAC': {'barcodes': 1283157,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38066,
                                'perfect_barcodes': 1245091,
                                'pf_Q30': 91.67,
                                'pf_clusters': 1283157,
                                'pf_qscore': 38.2,
                                'pf_read1_q30': 91.97,
                                'pf_read2_q30': 91.38,
                                'pf_yield': 387513414,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.31,
                                'sample_name': 'MIC2051A556_CTATCCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A557_GAGTAGAG': {'barcodes': 1323525,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 64320,
                                'perfect_barcodes': 1259205,
                                'pf_Q30': 90.68,
                                'pf_clusters': 1323525,
                                'pf_qscore': 37.86,
                                'pf_read1_q30': 87.93,
                                'pf_read2_q30': 93.43,
                                'pf_yield': 399704550,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A557_GAGTAGAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A558_GGTATAGG': {'barcodes': 1488169,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 60110,
                                'perfect_barcodes': 1428059,
                                'pf_Q30': 91.32,
                                'pf_clusters': 1488169,
                                'pf_qscore': 38.08,
                                'pf_read1_q30': 90.85,
                                'pf_read2_q30': 91.79,
                                'pf_yield': 449427038,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A558_GGTATAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A559_TAGCTTCC': {'barcodes': 1453408,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47101,
                                'perfect_barcodes': 1406307,
                                'pf_Q30': 91.5,
                                'pf_clusters': 1453408,
                                'pf_qscore': 38.15,
                                'pf_read1_q30': 91.46,
                                'pf_read2_q30': 91.55,
                                'pf_yield': 438929216,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.35,
                                'sample_name': 'MIC2051A559_TAGCTTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A560_TGGCTACA': {'barcodes': 1711603,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 55470,
                                'perfect_barcodes': 1656133,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1711603,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.86,
                                'pf_read2_q30': 91.21,
                                'pf_yield': 516904106,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A560_TGGCTACA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A561_ACAGCAAG': {'barcodes': 46678,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46678,
                                'perfect_barcodes': 0,
                                'pf_Q30': 79.81,
                                'pf_clusters': 46678,
                                'pf_qscore': 34.92,
                                'pf_read1_q30': 81.04,
                                'pf_read2_q30': 78.58,
                                'pf_yield': 14096756,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A561_ACAGCAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A562_AGTCGAAG': {'barcodes': 1682506,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 54534,
                                'perfect_barcodes': 1627972,
                                'pf_Q30': 92.13,
                                'pf_clusters': 1682506,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.93,
                                'pf_read2_q30': 91.32,
                                'pf_yield': 508116812,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A562_AGTCGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A563_CCGAAGAT': {'barcodes': 1724432,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 59794,
                                'perfect_barcodes': 1664638,
                                'pf_Q30': 92.09,
                                'pf_clusters': 1724432,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 92.62,
                                'pf_read2_q30': 91.56,
                                'pf_yield': 520778464,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.41,
                                'sample_name': 'MIC2051A563_CCGAAGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A564_CTCAGAAG': {'barcodes': 1381423,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44326,
                                'perfect_barcodes': 1337097,
                                'pf_Q30': 91.61,
                                'pf_clusters': 1381423,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 92.29,
                                'pf_read2_q30': 90.94,
                                'pf_yield': 417189746,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.33,
                                'sample_name': 'MIC2051A564_CTCAGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A565_GATCCACT': {'barcodes': 1012178,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 30026,
                                'perfect_barcodes': 982152,
                                'pf_Q30': 80.34,
                                'pf_clusters': 1012178,
                                'pf_qscore': 35.12,
                                'pf_read1_q30': 64.69,
                                'pf_read2_q30': 95.99,
                                'pf_yield': 305677756,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A565_GATCCACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A566_GGTTAGCT': {'barcodes': 1068325,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37373,
                                'perfect_barcodes': 1030952,
                                'pf_Q30': 91.6,
                                'pf_clusters': 1068325,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 91.46,
                                'pf_read2_q30': 91.75,
                                'pf_yield': 322634150,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A566_GGTTAGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A567_TAGTGCCA': {'barcodes': 1347842,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46019,
                                'perfect_barcodes': 1301823,
                                'pf_Q30': 91.92,
                                'pf_clusters': 1347842,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.23,
                                'pf_read2_q30': 91.62,
                                'pf_yield': 407048284,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A567_TAGTGCCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A568_TGTCGACT': {'barcodes': 1435262,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49081,
                                'perfect_barcodes': 1386181,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1435262,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 91.99,
                                'pf_read2_q30': 91.48,
                                'pf_yield': 433449124,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A568_TGTCGACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A569_ACCGAATG': {'barcodes': 1973413,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 61853,
                                'perfect_barcodes': 1911560,
                                'pf_Q30': 89.49,
                                'pf_clusters': 1973413,
                                'pf_qscore': 37.59,
                                'pf_read1_q30': 89.14,
                                'pf_read2_q30': 89.84,
                                'pf_yield': 595970726,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.47,
                                'sample_name': 'MIC2051A569_ACCGAATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A570_AGTGGCAA': {'barcodes': 1594113,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 53350,
                                'perfect_barcodes': 1540763,
                                'pf_Q30': 91.68,
                                'pf_clusters': 1594113,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.51,
                                'pf_read2_q30': 90.86,
                                'pf_yield': 481422126,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A570_AGTGGCAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A571_CCGTAACT': {'barcodes': 1421674,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 43195,
                                'perfect_barcodes': 1378479,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1421674,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.21,
                                'pf_read2_q30': 91.5,
                                'pf_yield': 429345548,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A571_CCGTAACT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A572_CTCGTTCT': {'barcodes': 2726510,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1141139,
                                'perfect_barcodes': 1585371,
                                'pf_Q30': 92.25,
                                'pf_clusters': 2726510,
                                'pf_qscore': 38.36,
                                'pf_read1_q30': 93.02,
                                'pf_read2_q30': 91.49,
                                'pf_yield': 823406020,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.65,
                                'sample_name': 'MIC2051A572_CTCGTTCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A573_GATGGAGT': {'barcodes': 1157340,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49014,
                                'perfect_barcodes': 1108326,
                                'pf_Q30': 88.34,
                                'pf_clusters': 1157340,
                                'pf_qscore': 37.2,
                                'pf_read1_q30': 82.48,
                                'pf_read2_q30': 94.2,
                                'pf_yield': 349516680,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A573_GATGGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A574_GTAAGCAC': {'barcodes': 1695780,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50647,
                                'perfect_barcodes': 1645133,
                                'pf_Q30': 91.74,
                                'pf_clusters': 1695780,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 92.07,
                                'pf_read2_q30': 91.4,
                                'pf_yield': 512125560,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A574_GTAAGCAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A575_TATGACCG': {'barcodes': 1206725,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38427,
                                'perfect_barcodes': 1168298,
                                'pf_Q30': 91.59,
                                'pf_clusters': 1206725,
                                'pf_qscore': 38.17,
                                'pf_read1_q30': 91.73,
                                'pf_read2_q30': 91.45,
                                'pf_yield': 364430950,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A575_TATGACCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A576_TTCCAGGT': {'barcodes': 1659597,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48727,
                                'perfect_barcodes': 1610870,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1659597,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 91.52,
                                'pf_read2_q30': 92.09,
                                'pf_yield': 501198294,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A576_TTCCAGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A577_ACCTAGAC': {'barcodes': 1937719,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 50476,
                                'perfect_barcodes': 1887243,
                                'pf_Q30': 92.33,
                                'pf_clusters': 1937719,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 91.87,
                                'pf_yield': 585191138,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.46,
                                'sample_name': 'MIC2051A577_ACCTAGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A578_ATACTGGC': {'barcodes': 1657524,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47099,
                                'perfect_barcodes': 1610425,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1657524,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 91.23,
                                'pf_yield': 500572248,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.39,
                                'sample_name': 'MIC2051A578_ATACTGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A579_CCTAGAGA': {'barcodes': 1363799,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 45407,
                                'perfect_barcodes': 1318392,
                                'pf_Q30': 91.87,
                                'pf_clusters': 1363799,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.42,
                                'pf_read2_q30': 90.32,
                                'pf_yield': 411867298,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A579_CCTAGAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A580_CTCTGGAT': {'barcodes': 1824212,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 64590,
                                'perfect_barcodes': 1759622,
                                'pf_Q30': 91.93,
                                'pf_clusters': 1824212,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 92.43,
                                'pf_read2_q30': 91.44,
                                'pf_yield': 550912024,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.43,
                                'sample_name': 'MIC2051A580_CTCTGGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A581_GCACACAA': {'barcodes': 1260259,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42575,
                                'perfect_barcodes': 1217684,
                                'pf_Q30': 91.87,
                                'pf_clusters': 1260259,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.07,
                                'pf_read2_q30': 91.66,
                                'pf_yield': 380598218,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A581_GCACACAA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A582_GTACGATC': {'barcodes': 1423328,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46393,
                                'perfect_barcodes': 1376935,
                                'pf_Q30': 91.86,
                                'pf_clusters': 1423328,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 91.52,
                                'pf_read2_q30': 92.21,
                                'pf_yield': 429845056,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A582_GTACGATC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A583_TCACTCGA': {'barcodes': 968940,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33702,
                                'perfect_barcodes': 935238,
                                'pf_Q30': 91.58,
                                'pf_clusters': 968940,
                                'pf_qscore': 38.14,
                                'pf_read1_q30': 90.6,
                                'pf_read2_q30': 92.56,
                                'pf_yield': 292619880,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A583_TCACTCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A584_TTCGGCTA': {'barcodes': 1574648,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 49946,
                                'perfect_barcodes': 1524702,
                                'pf_Q30': 91.65,
                                'pf_clusters': 1574648,
                                'pf_qscore': 38.19,
                                'pf_read1_q30': 91.69,
                                'pf_read2_q30': 91.61,
                                'pf_yield': 475543696,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.38,
                                'sample_name': 'MIC2051A584_TTCGGCTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A585_AGCGTGTT': {'barcodes': 1670007,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1670007,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.94,
                                'pf_clusters': 1670007,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.41,
                                'pf_read2_q30': 91.46,
                                'pf_yield': 504342114,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.4,
                                'sample_name': 'MIC2051A585_AGCGTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A586_AGAAGCGT': {'barcodes': 2807298,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1529360,
                                'perfect_barcodes': 1277938,
                                'pf_Q30': 91.1,
                                'pf_clusters': 2807298,
                                'pf_qscore': 38.05,
                                'pf_read1_q30': 91.48,
                                'pf_read2_q30': 90.72,
                                'pf_yield': 847803996,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.67,
                                'sample_name': 'MIC2051A586_AGAAGCGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A587_ACCACGAT': {'barcodes': 1162501,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36136,
                                'perfect_barcodes': 1126365,
                                'pf_Q30': 91.96,
                                'pf_clusters': 1162501,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 93.53,
                                'pf_read2_q30': 90.4,
                                'pf_yield': 351075302,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A587_ACCACGAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A588_TTCCAAGG': {'barcodes': 905952,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37690,
                                'perfect_barcodes': 868262,
                                'pf_Q30': 90.69,
                                'pf_clusters': 905952,
                                'pf_qscore': 37.88,
                                'pf_read1_q30': 92.05,
                                'pf_read2_q30': 89.34,
                                'pf_yield': 273597504,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A588_TTCCAAGG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A589_TGGTACAG': {'barcodes': 977841,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34344,
                                'perfect_barcodes': 943497,
                                'pf_Q30': 90.73,
                                'pf_clusters': 977841,
                                'pf_qscore': 37.92,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 88.15,
                                'pf_yield': 295307982,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A589_TGGTACAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A590_GTTAAGGC': {'barcodes': 890883,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 25464,
                                'perfect_barcodes': 865419,
                                'pf_Q30': 90.82,
                                'pf_clusters': 890883,
                                'pf_qscore': 37.91,
                                'pf_read1_q30': 92.02,
                                'pf_read2_q30': 89.62,
                                'pf_yield': 269046666,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A590_GTTAAGGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A591_AACAGGAC': {'barcodes': 1166637,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34097,
                                'perfect_barcodes': 1132540,
                                'pf_Q30': 92.16,
                                'pf_clusters': 1166637,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.32,
                                'pf_read2_q30': 91.0,
                                'pf_yield': 352324374,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A591_AACAGGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A592_GCAATGGA': {'barcodes': 1077290,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37783,
                                'perfect_barcodes': 1039507,
                                'pf_Q30': 90.59,
                                'pf_clusters': 1077290,
                                'pf_qscore': 37.87,
                                'pf_read1_q30': 89.91,
                                'pf_read2_q30': 91.26,
                                'pf_yield': 325341580,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A592_GCAATGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A593_CACCTGTT': {'barcodes': 1272834,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44972,
                                'perfect_barcodes': 1227862,
                                'pf_Q30': 92.23,
                                'pf_clusters': 1272834,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.15,
                                'pf_read2_q30': 91.32,
                                'pf_yield': 384395868,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A593_CACCTGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A594_GTAGGAGT': {'barcodes': 1120521,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 60723,
                                'perfect_barcodes': 1059798,
                                'pf_Q30': 91.65,
                                'pf_clusters': 1120521,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 91.57,
                                'pf_read2_q30': 91.73,
                                'pf_yield': 338397342,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A594_GTAGGAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A595_CTAGGCAT': {'barcodes': 1234063,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40278,
                                'perfect_barcodes': 1193785,
                                'pf_Q30': 91.8,
                                'pf_clusters': 1234063,
                                'pf_qscore': 38.24,
                                'pf_read1_q30': 92.69,
                                'pf_read2_q30': 90.9,
                                'pf_yield': 372687026,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A595_CTAGGCAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A596_GCAATTCG': {'barcodes': 32484,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32484,
                                'perfect_barcodes': 0,
                                'pf_Q30': 82.07,
                                'pf_clusters': 32484,
                                'pf_qscore': 35.51,
                                'pf_read1_q30': 82.52,
                                'pf_read2_q30': 81.63,
                                'pf_yield': 9810168,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A596_GCAATTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A597_GCTGTAAG': {'barcodes': 1070300,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33370,
                                'perfect_barcodes': 1036930,
                                'pf_Q30': 91.66,
                                'pf_clusters': 1070300,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 92.15,
                                'pf_read2_q30': 91.18,
                                'pf_yield': 323230600,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A597_GCTGTAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A599_GATGAGAC': {'barcodes': 929540,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 30631,
                                'perfect_barcodes': 898909,
                                'pf_Q30': 90.91,
                                'pf_clusters': 929540,
                                'pf_qscore': 37.92,
                                'pf_read1_q30': 88.42,
                                'pf_read2_q30': 93.4,
                                'pf_yield': 280721080,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A599_GATGAGAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A600_TACCAGGA': {'barcodes': 1434907,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1434907,
                                'perfect_barcodes': 0,
                                'pf_Q30': 92.23,
                                'pf_clusters': 1434907,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 92.67,
                                'pf_read2_q30': 91.8,
                                'pf_yield': 433341914,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.34,
                                'sample_name': 'MIC2051A600_TACCAGGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A601_CGTAGGTT': {'barcodes': 1037350,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 41855,
                                'perfect_barcodes': 995495,
                                'pf_Q30': 91.65,
                                'pf_clusters': 1037350,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 93.52,
                                'pf_read2_q30': 89.79,
                                'pf_yield': 313279700,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A601_CGTAGGTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A602_TTCGCAGT': {'barcodes': 1274872,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 46312,
                                'perfect_barcodes': 1228560,
                                'pf_Q30': 92.15,
                                'pf_clusters': 1274872,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.09,
                                'pf_read2_q30': 91.22,
                                'pf_yield': 385011344,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A602_TTCGCAGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A603_CGGCTAAT': {'barcodes': 1092555,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36925,
                                'perfect_barcodes': 1055630,
                                'pf_Q30': 91.85,
                                'pf_clusters': 1092555,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 93.86,
                                'pf_read2_q30': 89.83,
                                'pf_yield': 329951610,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A603_CGGCTAAT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A604_CTAACTCG': {'barcodes': 1199292,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 39318,
                                'perfect_barcodes': 1159974,
                                'pf_Q30': 91.98,
                                'pf_clusters': 1199292,
                                'pf_qscore': 38.28,
                                'pf_read1_q30': 92.57,
                                'pf_read2_q30': 91.38,
                                'pf_yield': 362186184,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.29,
                                'sample_name': 'MIC2051A604_CTAACTCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A605_AACCGAAG': {'barcodes': 1100476,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38435,
                                'perfect_barcodes': 1062041,
                                'pf_Q30': 91.95,
                                'pf_clusters': 1100476,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.05,
                                'pf_read2_q30': 90.84,
                                'pf_yield': 332343752,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A605_AACCGAAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A606_AACTGAGC': {'barcodes': 1092818,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32811,
                                'perfect_barcodes': 1060007,
                                'pf_Q30': 92.19,
                                'pf_clusters': 1092818,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.23,
                                'pf_read2_q30': 91.14,
                                'pf_yield': 330031036,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A606_AACTGAGC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A607_AAGGACAC': {'barcodes': 1059592,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33037,
                                'perfect_barcodes': 1026555,
                                'pf_Q30': 92.15,
                                'pf_clusters': 1059592,
                                'pf_qscore': 38.34,
                                'pf_read1_q30': 92.95,
                                'pf_read2_q30': 91.34,
                                'pf_yield': 319996784,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A607_AAGGACAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A608_GAATCCGA': {'barcodes': 897367,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 31656,
                                'perfect_barcodes': 865711,
                                'pf_Q30': 90.36,
                                'pf_clusters': 897367,
                                'pf_qscore': 37.77,
                                'pf_read1_q30': 86.97,
                                'pf_read2_q30': 93.76,
                                'pf_yield': 271004834,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A608_GAATCCGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A609_GGTGTCTT': {'barcodes': 1138145,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 47289,
                                'perfect_barcodes': 1090856,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1138145,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.4,
                                'pf_read2_q30': 91.73,
                                'pf_yield': 343719790,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A609_GGTGTCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A610_CTGGTTCT': {'barcodes': 44914,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44914,
                                'perfect_barcodes': 0,
                                'pf_Q30': 80.13,
                                'pf_clusters': 44914,
                                'pf_qscore': 35.01,
                                'pf_read1_q30': 81.73,
                                'pf_read2_q30': 78.53,
                                'pf_yield': 13564028,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A610_CTGGTTCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A611_GCTTCTTG': {'barcodes': 947466,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 30271,
                                'perfect_barcodes': 917195,
                                'pf_Q30': 91.3,
                                'pf_clusters': 947466,
                                'pf_qscore': 38.03,
                                'pf_read1_q30': 89.89,
                                'pf_read2_q30': 92.72,
                                'pf_yield': 286134732,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A611_GCTTCTTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A612_TATCAGCG': {'barcodes': 825576,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 25550,
                                'perfect_barcodes': 800026,
                                'pf_Q30': 91.57,
                                'pf_clusters': 825576,
                                'pf_qscore': 38.16,
                                'pf_read1_q30': 93.63,
                                'pf_read2_q30': 89.5,
                                'pf_yield': 249323952,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.2,
                                'sample_name': 'MIC2051A612_TATCAGCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A613_CTTCGTTC': {'barcodes': 927622,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33529,
                                'perfect_barcodes': 894093,
                                'pf_Q30': 91.93,
                                'pf_clusters': 927622,
                                'pf_qscore': 38.27,
                                'pf_read1_q30': 93.64,
                                'pf_read2_q30': 90.23,
                                'pf_yield': 280141844,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A613_CTTCGTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A614_CATGTTCC': {'barcodes': 1005096,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32323,
                                'perfect_barcodes': 972773,
                                'pf_Q30': 92.03,
                                'pf_clusters': 1005096,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.33,
                                'pf_read2_q30': 90.73,
                                'pf_yield': 303538992,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A614_CATGTTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A615_GCCATAAC': {'barcodes': 1061102,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33968,
                                'perfect_barcodes': 1027134,
                                'pf_Q30': 92.16,
                                'pf_clusters': 1061102,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.69,
                                'pf_read2_q30': 90.63,
                                'pf_yield': 320452804,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A615_GCCATAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A616_ACGACAGA': {'barcodes': 995255,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34855,
                                'perfect_barcodes': 960400,
                                'pf_Q30': 91.85,
                                'pf_clusters': 995255,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 92.79,
                                'pf_read2_q30': 90.91,
                                'pf_yield': 300567010,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A616_ACGACAGA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A617_AGTCGCTT': {'barcodes': 1027118,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38335,
                                'perfect_barcodes': 988783,
                                'pf_Q30': 92.06,
                                'pf_clusters': 1027118,
                                'pf_qscore': 38.32,
                                'pf_read1_q30': 93.81,
                                'pf_read2_q30': 90.31,
                                'pf_yield': 310189636,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A617_AGTCGCTT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A618_CCGTATCT': {'barcodes': 1002126,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34678,
                                'perfect_barcodes': 967448,
                                'pf_Q30': 91.77,
                                'pf_clusters': 1002126,
                                'pf_qscore': 38.21,
                                'pf_read1_q30': 93.5,
                                'pf_read2_q30': 90.04,
                                'pf_yield': 302642052,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A618_CCGTATCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A619_CCTGATTG': {'barcodes': 976594,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32007,
                                'perfect_barcodes': 944587,
                                'pf_Q30': 92.33,
                                'pf_clusters': 976594,
                                'pf_qscore': 38.38,
                                'pf_read1_q30': 93.3,
                                'pf_read2_q30': 91.36,
                                'pf_yield': 294931388,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A619_CCTGATTG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A620_TAATGCCG': {'barcodes': 881478,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 27139,
                                'perfect_barcodes': 854339,
                                'pf_Q30': 91.38,
                                'pf_clusters': 881478,
                                'pf_qscore': 38.09,
                                'pf_read1_q30': 90.92,
                                'pf_read2_q30': 91.84,
                                'pf_yield': 266206356,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.21,
                                'sample_name': 'MIC2051A620_TAATGCCG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A621_GGATCTTC': {'barcodes': 905114,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 35175,
                                'perfect_barcodes': 869939,
                                'pf_Q30': 91.28,
                                'pf_clusters': 905114,
                                'pf_qscore': 38.04,
                                'pf_read1_q30': 89.46,
                                'pf_read2_q30': 93.1,
                                'pf_yield': 273344428,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A621_GGATCTTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A622_TGATGTCC': {'barcodes': 1069699,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 36783,
                                'perfect_barcodes': 1032916,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1069699,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 92.36,
                                'pf_read2_q30': 91.66,
                                'pf_yield': 323049098,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A622_TGATGTCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A623_ATGCCAAC': {'barcodes': 1177448,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 34755,
                                'perfect_barcodes': 1142693,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1177448,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 92.6,
                                'pf_read2_q30': 91.48,
                                'pf_yield': 355589296,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A623_ATGCCAAC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A624_AACGGTCA': {'barcodes': 962838,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 38555,
                                'perfect_barcodes': 924283,
                                'pf_Q30': 91.75,
                                'pf_clusters': 962838,
                                'pf_qscore': 38.22,
                                'pf_read1_q30': 93.08,
                                'pf_read2_q30': 90.42,
                                'pf_yield': 290777076,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A624_AACGGTCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A625_CAGCGATT': {'barcodes': 1056549,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 31063,
                                'perfect_barcodes': 1025486,
                                'pf_Q30': 92.21,
                                'pf_clusters': 1056549,
                                'pf_qscore': 38.35,
                                'pf_read1_q30': 93.56,
                                'pf_read2_q30': 90.86,
                                'pf_yield': 319077798,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A625_CAGCGATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A626_ATCTCGCT': {'barcodes': 1050046,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 31453,
                                'perfect_barcodes': 1018593,
                                'pf_Q30': 90.98,
                                'pf_clusters': 1050046,
                                'pf_qscore': 38.0,
                                'pf_read1_q30': 93.19,
                                'pf_read2_q30': 88.77,
                                'pf_yield': 317113892,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A626_ATCTCGCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A627_CGAGTATG': {'barcodes': 912599,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37746,
                                'perfect_barcodes': 874853,
                                'pf_Q30': 91.18,
                                'pf_clusters': 912599,
                                'pf_qscore': 38.04,
                                'pf_read1_q30': 93.26,
                                'pf_read2_q30': 89.1,
                                'pf_yield': 275604898,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A627_CGAGTATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A628_TCTTGACG': {'barcodes': 1032888,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 33056,
                                'perfect_barcodes': 999832,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1032888,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 93.0,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 311932176,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.25,
                                'sample_name': 'MIC2051A628_TCTTGACG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A629_TATCGGTC': {'barcodes': 1076720,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 35836,
                                'perfect_barcodes': 1040884,
                                'pf_Q30': 91.88,
                                'pf_clusters': 1076720,
                                'pf_qscore': 38.23,
                                'pf_read1_q30': 92.65,
                                'pf_read2_q30': 91.11,
                                'pf_yield': 325169440,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A629_TATCGGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A630_TCAGATCC': {'barcodes': 1088164,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 32004,
                                'perfect_barcodes': 1056160,
                                'pf_Q30': 92.02,
                                'pf_clusters': 1088164,
                                'pf_qscore': 38.26,
                                'pf_read1_q30': 91.72,
                                'pf_read2_q30': 92.31,
                                'pf_yield': 328625528,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A630_TCAGATCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A631_CGTGTGTA': {'barcodes': 1122037,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 48417,
                                'perfect_barcodes': 1073620,
                                'pf_Q30': 92.01,
                                'pf_clusters': 1122037,
                                'pf_qscore': 38.29,
                                'pf_read1_q30': 92.98,
                                'pf_read2_q30': 91.05,
                                'pf_yield': 338855174,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.27,
                                'sample_name': 'MIC2051A631_CGTGTGTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A632_GTCTTGCA': {'barcodes': 821779,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 26286,
                                'perfect_barcodes': 795493,
                                'pf_Q30': 91.35,
                                'pf_clusters': 821779,
                                'pf_qscore': 38.07,
                                'pf_read1_q30': 90.92,
                                'pf_read2_q30': 91.78,
                                'pf_yield': 248177258,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.2,
                                'sample_name': 'MIC2051A632_GTCTTGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A633_CCGGAATT': {'barcodes': 1105090,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 42478,
                                'perfect_barcodes': 1062612,
                                'pf_Q30': 92.04,
                                'pf_clusters': 1105090,
                                'pf_qscore': 38.3,
                                'pf_read1_q30': 93.37,
                                'pf_read2_q30': 90.72,
                                'pf_yield': 333737180,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.26,
                                'sample_name': 'MIC2051A633_CCGGAATT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A634_AACCTCCT': {'barcodes': 1352471,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 44185,
                                'perfect_barcodes': 1308286,
                                'pf_Q30': 89.74,
                                'pf_clusters': 1352471,
                                'pf_qscore': 37.65,
                                'pf_read1_q30': 92.91,
                                'pf_read2_q30': 86.58,
                                'pf_yield': 408446242,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.32,
                                'sample_name': 'MIC2051A634_AACCTCCT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A635_AGCAGATG': {'barcodes': 1180047,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37780,
                                'perfect_barcodes': 1142267,
                                'pf_Q30': 92.11,
                                'pf_clusters': 1180047,
                                'pf_qscore': 38.33,
                                'pf_read1_q30': 93.51,
                                'pf_read2_q30': 90.7,
                                'pf_yield': 356374194,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.28,
                                'sample_name': 'MIC2051A635_AGCAGATG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A636_GGTCTTAG': {'barcodes': 984432,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 40543,
                                'perfect_barcodes': 943889,
                                'pf_Q30': 91.76,
                                'pf_clusters': 984432,
                                'pf_qscore': 38.18,
                                'pf_read1_q30': 91.86,
                                'pf_read2_q30': 91.66,
                                'pf_yield': 297298464,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.23,
                                'sample_name': 'MIC2051A636_GGTCTTAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A637_ACCTTCTC': {'barcodes': 1005647,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 37263,
                                'perfect_barcodes': 968384,
                                'pf_Q30': 92.1,
                                'pf_clusters': 1005647,
                                'pf_qscore': 38.31,
                                'pf_read1_q30': 93.4,
                                'pf_read2_q30': 90.81,
                                'pf_yield': 303705394,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.24,
                                'sample_name': 'MIC2051A637_ACCTTCTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A638_AGAATGCC': {'barcodes': 1267386,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 1267386,
                                'perfect_barcodes': 0,
                                'pf_Q30': 91.78,
                                'pf_clusters': 1267386,
                                'pf_qscore': 38.25,
                                'pf_read1_q30': 92.53,
                                'pf_read2_q30': 91.04,
                                'pf_yield': 382750572,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.3,
                                'sample_name': 'MIC2051A638_AGAATGCC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A639_GCGTTCTA': {'barcodes': 918499,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 35238,
                                'perfect_barcodes': 883261,
                                'pf_Q30': 91.51,
                                'pf_clusters': 918499,
                                'pf_qscore': 38.1,
                                'pf_read1_q30': 91.61,
                                'pf_read2_q30': 91.41,
                                'pf_yield': 277386698,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.22,
                                'sample_name': 'MIC2051A639_GCGTTCTA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A640_GTTACGCA': {'barcodes': 15751,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 6942,
                                'perfect_barcodes': 8809,
                                'pf_Q30': 88.54,
                                'pf_clusters': 15751,
                                'pf_qscore': 37.32,
                                'pf_read1_q30': 89.12,
                                'pf_read2_q30': 87.96,
                                'pf_yield': 4756802,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A640_GTTACGCA',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A641_CCTAAGTC': {'barcodes': 21118,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 3035,
                                'perfect_barcodes': 18083,
                                'pf_Q30': 86.12,
                                'pf_clusters': 21118,
                                'pf_qscore': 36.7,
                                'pf_read1_q30': 88.23,
                                'pf_read2_q30': 84.0,
                                'pf_yield': 6377636,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A641_CCTAAGTC',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A643_ATAGCGGT': {'barcodes': 26390,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 8878,
                                'perfect_barcodes': 17512,
                                'pf_Q30': 85.64,
                                'pf_clusters': 26390,
                                'pf_qscore': 36.56,
                                'pf_read1_q30': 87.25,
                                'pf_read2_q30': 84.04,
                                'pf_yield': 7969780,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.01,
                                'sample_name': 'MIC2051A643_ATAGCGGT',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0},
       'MIC2051A644_ACGATCAG': {'barcodes': 14518,
                                'flowcell': 'H37GVALXX',
                                'lane': '8',
                                'one_mismatch_barcodes': 2608,
                                'perfect_barcodes': 11910,
                                'pf_Q30': 89.15,
                                'pf_clusters': 14518,
                                'pf_qscore': 37.5,
                                'pf_read1_q30': 89.38,
                                'pf_read2_q30': 88.91,
                                'pf_yield': 4384436,
                                'pf_yield_pc': 100.0,
                                'raw_clusters_pc': 0.0,
                                'sample_name': 'MIC2051A644_ACGATCAG',
                                'undetermined_pc': 0.0,
                                'undetermined_proc': 0}}}
